using UnityEngine;
using System.Collections;
using System.Collections.Generic;


[RequireComponent(typeof(CharacterController))]
public class SimController : MonoBehaviour 
{
	#region enums
	/// <summary>
	/// States for player
	/// </summary>
	public enum State
	{
		/// <summary>
		/// Infected a civil by a cat
		/// </summary>
		INFECT_CAT,		
		/// <summary>
		/// Infected a civil by a zombie
		/// </summary>
		INFECT_ZOMBIE,	
		
		/// <summary>
		/// State fall
		/// </summary>
		SIM_FALL,
		
		/// <summary>
		/// IDLE for civil.
		/// </summary>
		C_IDLE,
		
		// <summary>
		/// civil patrol by distance
		/// </summary>
		C_PATROL_DISTANCE,
		
		/// <summary>
		/// Warn state after seeing zombie
		/// </summary>
		C_WARN,
		
		/// <summary>
		/// Alert state after seeing zombie (poly)
		/// </summary>
		C_ALERT,
		
		/// <summary>
		/// Shoot a zombie (poly)
		/// </summary>
		C_SHOOT,
		
	
		/// <summary>
		/// State idle for zombie
		/// </summary>
		Z_IDLE,	
		
		/// <summary>
		/// State hunt zombie to civil
		/// </summary>
		Z_HUNT,		
		/// <summary>
		/// State chasing zombie to civil
		/// </summary>
		Z_CHASE,		
		/// <summary>
		/// Zombie attacking
		/// </summary>
		Z_ATTACK,
		
		/// <summary>
		/// State zombie of hurt
		/// </summary>
		Z_HURT,
		
		/// <summary>
		/// State zombie death
		/// </summary>
		Z_DEATH,
		
		/// <summary>
		/// Stunned for civils and zombies
		/// </summary>
		STUNNED,
		
		/// <summary>
		/// Inactive state
		/// </summary>
		INACTIVE
		
	}
	
	
	/// <summary>
	/// TypeDirection
	/// </summary>
	public enum DirectionType
	{		
		LEFT = 0,
		RIGHT,		
	}
	
	/// <summary>
	/// TypeDirection
	/// </summary>
	public enum VerticalDirectionType
	{		
		UP = 0,
		DOWN,		
		
	}
	
	
	/// <summary>
	/// Type of idle
	/// </summary>
	public enum TIDLE
	{
		IDLE_NORMAL,
		IDLE_TALKING,
		IDLE_SIGNAL,
	}
	
	
	/// <summary>
	/// Type officer
	/// </summary>
	public enum TOfficer
	{
		OFFICER_NONE = 0,
		OFFICER_COMMON,	
		OFFICER_FALL,
		OFFICER_SPECIAL_ENFORCEMENT,
	}
	
	/// <summary>
	/// Type skin of SIM
	/// </summary>
	public enum Skin
	{
		/// <summary>
		/// None skin
		/// </summary>
		NONE_SKIN = 0,
		/// <summary>
		/// Bussines skin
		/// </summary>
		BUSINESS_SKIN,		
		/// <summary>
		/// Geek skin
		/// </summary>
		GEEK_SKIN,	
		
		/// <summary>
		/// Music skin, this guys, cannot listen
		/// </summary>
		MUSIC_SKIN,		
		/// <summary>
		/// Mobile skin, this guys, cannot listen and seen another sims
		/// </summary>
		MOBILE_SKIN,		
		
		/// <summary>
		/// Poly skin
		/// </summary>
		POLY_SKIN,
		
		/// <summary>
		/// Leader skin
		/// </summary>
		LEADER_SKIN,

	}
			
	#endregion enums		
	
	#region SimStats
	
	/// <summary>
	/// Time and delta to reinit hunt after infected a civil
	/// </summary>
	private float deltaTimeResetHunt = 0.7f; 
	private float updateTimeResetHunt = 0.0f;	
	
	
	[System.SerializableAttribute]
	public class SimStats
	{
		/// <summary>
		/// ID for the sim
		/// </summary>
		public int IDSim = 0;	
		
	
		/// <summary>
		/// Current State
		/// </summary>
		public State CurrentState;
		
		/// <summary>
		/// Current direction
		/// </summary>
		public SimController.DirectionType CurrentDirection;
		
		
		/// <summary>
		/// Type of idle for the sim (normal, speak, argue...)
		/// </summary>
		public TIDLE TypeIdle = SimController.TIDLE.IDLE_NORMAL;
		
		/// <summary>
		/// Type of officer
		/// </summary>
		public SimController.TOfficer TypeOfficer = SimController.TOfficer.OFFICER_NONE;
		
		/// <summary>
		/// Type of skin for sim
		/// </summary>
		public Skin TSkin = SimController.Skin.NONE_SKIN;
		
		/// <summary>
		/// Speed movement SIM
		/// </summary>
		public float SpeedMovement = 0.0f;
		
		/// <summary>
		/// Whether if sim is infected or not
		/// </summary>
		public bool infected = false;
		public bool Infected
		{
		    get { return infected; }
		    set { infected = value; }
		}
		
		// Is infecting
		/// <summary>
		/// Sim is infecting by another zombie or cat
		/// </summary>
        private bool isInfecting = false;
		public bool IsInfecting
		{
		    get { return isInfecting; }
		    set { isInfecting = value; }
		}
	
		/// <summary>
		/// Sim I chase
		/// </summary>
		public SimController SimChase;
		/// <summary>
		/// Sim is hunted at me
		/// </summary>
		public SimController SimHunted;
		/// <summary>
		/// Sim zombie
		/// </summary>
		public SimController SimZombie;
		/// <summary>
		/// Cat for special enforcement
		/// </summary>
		public Transform CatChasePoly;
		
		/// <summary>
		/// Type of direction
		/// </summary>
		private DirectionType direction = DirectionType.LEFT;
		public SimController.DirectionType Direction 
		{
	        get { return this.direction; }
			set { this.direction = value; }
	    }
		
		/// <summary>
		/// Type of direction
		/// </summary>
		private VerticalDirectionType vDirection = VerticalDirectionType.UP;
		public SimController.VerticalDirectionType VDirection 
		{
	        get { return this.vDirection; }
			set { this.vDirection = value; }
	    }
		
		/// <summary>
		/// Whether sim is on idle
		/// </summary>
		private bool idle = true;
		public bool Idle 
		{
	        get { return this.idle; }
			set { this.idle = value; }
	    }
		
		
		/// <summary>
		/// Sim is inside or outside of scenary (we can see sim in stage or is behind..)
		/// </summary>
		public bool outsideScenery = false;
		public bool OutsideScenery 
		{
	        get { return this.outsideScenery; }
			set { this.outsideScenery = value; }
	    }	
		
		/// <summary>
		/// Lives for zombies
		/// </summary>
		//[System.NonSerializedAttribute]
		public int Lives = 2;		
		
		/// <summary>
		/// Whether is sim has sield
		/// </summary>
		public bool HasShield  = false;
		
		
		/// <summary>
		/// Number of floor
		/// </summary>
		public int Number_Floor = 0;	
		
		
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SimStats Stats;
	
	#endregion SimStats		
	
	#region SettingInitSim
	[System.SerializableAttribute]
	public class SettingInitSim
	{		
		
		/// <summary>
		/// State initial of sim
		/// </summary>
		public State IntialState;
		
		/// <summary>
		/// Initial state
		/// </summary>
		private State initState;
		public State InitState
		{
		    get { return initState; }
		    set { initState = value; }
		}
		
		
		/// <summary>
		/// Direction in Horizontal for Sim
		/// </summary>
		public SimController.DirectionType InitialDirection = SimController.DirectionType.RIGHT;	
		
		
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingInitSim SetupInitSim;
	#endregion SettingInitSim	
	
	#region SettingSimFall
	//[System.SerializableAttribute]
	public class SettingSimFall
	{
		/// <summary>
		/// Distance when sim is on attack state by a zombie
		/// </summary>		
		private float distanceFall = 90.0f;	
		public float DistanceFall
		{
		    get { return this.distanceFall; }
		    set { this.distanceFall = value; }
		}
		
		
		private int currentFloor = 0;
		public int CurrentFloor
		{
			get { return this.currentFloor; }
			set { this.currentFloor = value; }
		}
		
		private bool isOutsideBunker = false;
		public bool IsOutsideBunker
		{
			get { return this.isOutsideBunker; }
			set { this.isOutsideBunker = value; }
		}
	}
	/// <summary>
	/// Setup for sim is on state fall
	/// </summary>
	public SettingSimFall SetupSimFall = new SettingSimFall();
	
	#endregion SettingSimFall
	
	#region SettingAlertSim
	[System.SerializableAttribute]
	public class SettingAlertSim
	{
		/// <summary>
		/// Distance when sim is on attack state by a zombie and search
		/// </summary>	
		public float DistanceSearchWarn = 150.0f;
	
		
		/// <summary>
		/// Direction of warn
		/// </summary>
		private SimController.DirectionType directionWarn = SimController.DirectionType.LEFT;
		public SimController.DirectionType DirectionWarn
		{
		    get { return this.directionWarn; }
		    set { this.directionWarn = value; }
		}
	}
	
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingAlertSim SetupAlertSim;		
	#endregion SettingAlertSim
	
	#region SettingSimPatrol
	[System.SerializableAttribute]
	public class SettingSimPatrol
	{
		/// <summary>
		/// Distance patrol
		/// </summary>
		public float DistancePatrol;
		
		/// <summary>
		/// Time to wait sim patrol
		/// </summary>
		public float TimeWaitPatrol;
	}
	
	/// <summary>
	/// Setup for sim where is patrol
	/// </summary>
	public SettingSimPatrol SetupSimPatrol;		
	#endregion SettingSimPatrol
	
	#region SettingsZombieSim	
	[System.SerializableAttribute]
	public class SettingZombieSim
	{
		/// <summary>
		/// Prefab for zombieCloud
		/// </summary>
		public GameObject ZombieCloudPrefab;
		
		/// <summary>
		/// Particle Cloud
		/// </summary>
		private GameObject instanceZCloudPrefab;
		public GameObject InstZCloudPrefab
		{
		    get { return instanceZCloudPrefab; }
		    set { instanceZCloudPrefab = value; }
		}
		//[System.NonSerializedAttribute]
		private bool isEndCloud = false;
		public bool IsEndCloud
		{
		    get { return isEndCloud; }
		    set { isEndCloud = value; }
		}
		//[System.NonSerializedAttribute]
		private float updateTimeShowCloud = 0.0f;
		public float UpdateTimeShowCloud
		{
		    get { return updateTimeShowCloud; }
		    set { updateTimeShowCloud = value; }
		}
		//[System.NonSerializedAttribute]
		private float deltaTimeShowCloud = 1.0f;
		public float DeltaTimeShowCloud
		{
		    get { return deltaTimeShowCloud; }
		    set { deltaTimeShowCloud = value; }
		}
	
	}
	/// <summary>
	/// Setup different member for zombies
	/// </summary>
	public SettingZombieSim SetupZombieSim;	
	#endregion SettingsZombieSim				
	
	#region SettingsSim	
	
	[System.SerializableAttribute]
	public class SettingsSim
	{
		
		#region customSpeed
		/// <summary>
		/// Idle speed
		/// </summary>
		//public static float IDLE_SPEED;
		public float Idle_Speed = 0.0f;
		/// <summary>
		/// Fall speed
		/// </summary>
		public float Fall_Speed = 175.0f;
		/// <summary>
		/// Patrol speed for civils
		/// </summary>
		//public static float C_PATROL_SPEED;
		public float C_Patrol_Speed = 70.0f;
		
		/// <summary>
		/// speed for warn  state on civils
		/// </summary>
		//public static float C_Warn_Speed;
		public float C_Warn_Speed = 120.0f;
		
		/// <summary>
		/// Hunt speed for zombies
		/// </summary>
		//public static float Z_HUNT_SPEED;
		//[System.NonSerializedAttribute]
		public float Z_Hunt_Speed = 80.0f;
		/// <summary>
		/// Chase speed for chase zombies
		/// </summary>
		//public static float Z_CHASE_SPEED;
		//[System.NonSerializedAttribute]
		public float Z_Chase_Speed = 140.0f;
		
		
		/// <summary>
		/// bonus for speed
		/// </summary>
		public float BonusSpeed = 30.0f;
		
		#endregion customspeed
		
		private Transform shadowSim;
		public Transform ShadowSim 
		{
	        get { return this.shadowSim; }
			set { this.shadowSim = value; }
	    }

        public HelperCircleWarning HelperCircleWarn;
		
	}
	
	/// <summary>
	/// Settings components
	/// </summary>
	public SettingsSim Settings;	
	
	
	#endregion SettingsSim
	
	#region SettingBarrierSim
	[System.SerializableAttribute]
	public class SettingBarrierSim
	{			
		
		/// <summary>
		/// Is sim crossing bar
		/// </summary>
		private bool isSimCrossBar = false;
		public bool IsSimCrossBar
	    {
	        set { this.isSimCrossBar = value; }
	        get { return this.isSimCrossBar; }
	    }
	
		
		/// <summary>
		/// First Point of Barrier target
		/// </summary>
		private Vector3 firstTargetBar;
		public Vector3 FirstTargetBar
	    {
	        set { this.firstTargetBar = value; }
	        get { return this.firstTargetBar; }
	    }
		
		/// <summary>
		/// Check if must be setup first target
		/// </summary>
		private bool isHasSetFirstTargetBar = false;		
		public bool IsHasSetFirstTargetBar
   	 	{
        	get { return this.isHasSetFirstTargetBar; }
			set { this.isHasSetFirstTargetBar = value; }
    	}
		
		/// <summary>
		/// Check if sim reach barrier first target
		/// </summary>
		private bool isReachFirstTargetBar = false;		
		public bool IsReachFirstTargetBar
   	 	{
        	get { return this.isReachFirstTargetBar; }
			set { this.isReachFirstTargetBar = value; }
    	}
		
		/// <summary>
		/// First Point of Barrier target
		/// </summary>
		private Vector3 secondTargetBar;
		public Vector3 SecondTargetBar
	    {
	        set { this.secondTargetBar = value; }
	        get { return this.secondTargetBar; }
	    }
		
		/// <summary>
		/// Check if must be setup second target
		/// </summary>
		private bool isHasSetSecondTargetBar = false;		
		public bool IsHasSetSecondTargetBar
   	 	{
        	get { return this.isHasSetSecondTargetBar; }
			set { this.isHasSetSecondTargetBar = value; }
    	}
		
		/// <summary>
		/// Check if sim reach barrier first target
		/// </summary>
		private bool isReachSecondTargetBar = false;		
		public bool IsReachSecondTargetBar
   	 	{
        	get { return this.isReachSecondTargetBar; }
			set { this.isReachSecondTargetBar = value; }
    	}
		
		/// <summary>
		/// Current Point of Barrier target
		/// </summary>
		private Vector3 currentTargetBar;
		public Vector3 CurrentTargetBar
	    {
	        set { this.currentTargetBar = value; }
	        get { return this.currentTargetBar; }
	    }
		
		/// <summary>
		/// Barrier target
		/// </summary>
		private Vector3 targetBar;
		public Vector3 TargetBar
	    {
	        set { this.targetBar = value; }
	        get { return this.targetBar; }
	    }
		
		/// <summary>
		/// Check if there is has to setup BarrierTarget
		/// </summary>
		private bool isHasSetTargetBar = false;
		
		public bool IsHasSetTargetBar
   	 	{
        	get { return this.isHasSetTargetBar; }
			set { this.isHasSetTargetBar = value; }
    	}
		
		/// <summary>
		/// Check if setup of new target is done
		/// </summary>
		private bool isSetTargetBar = false;
		
		public bool IsSetTargetBar
   	 	{
        	get { return this.isSetTargetBar; }
			set { this.isSetTargetBar = value; }
    	}
		
		/// <summary>
		/// Check if sim reach barrier target
		/// </summary>
		private bool isReachTargetBar = false;
		
		public bool IsReachTargetBar
   	 	{
        	get { return this.isReachTargetBar; }
			set { this.isReachTargetBar = value; }
    	}
		
		/// <summary>
		/// Check if there is has to setup BarrierTarget
		/// </summary>
		private bool isSetupBarrierTarget = false;
		
		public bool IsSetupBarrierTarget
   	 	{
        	get { return this.isSetupBarrierTarget; }
			set { this.isSetupBarrierTarget = value; }
    	}
		
		/// <summary>
		/// Indicating if is settup the behaviour of barrier Target
		/// </summary>
		private bool setupBarrierTarget = false;
		public bool SetupBarrierTarget
	    {
	        set { this.setupBarrierTarget = value; }
	        get { return this.setupBarrierTarget; }
	    }
		
		/// <summary>
		/// Check if sim reach barrier target
		/// </summary>
		private bool reachBarrierTarget = false;
		
		public bool ReachBarrierTarget
   	 	{
        	get { return this.reachBarrierTarget; }
			set { this.reachBarrierTarget = value; }
    	}
		
		/// <summary>
		/// Barrier target
		/// </summary>
		private Vector3 barrierTarget;
		public Vector3 BarrierTarget
	    {
	        set { this.barrierTarget = value; }
	        get { return this.barrierTarget; }
	    }
		
	}
	public SettingBarrierSim SetupBarrierSim;
	#endregion SettingBarrierSim
	
	#region SettingWallSim
	[System.SerializableAttribute]
	public class SettingWallSim
	{
		
		/// <summary>
		/// Sim change direction
		/// </summary>
		private bool isSetupWall = false;
		public bool IsSetupWall
	    {
	        set { this.isSetupWall = value; }
	        get { return this.isSetupWall; }
	    }
		
		
		private Vector3 posWall = Vector3.zero;
		public Vector3 PosWall
	    {
	        set { this.posWall = value; }
	        get { return this.posWall; }
	    }
	}
	public SettingWallSim SetupWallSim;
	#endregion SettingWallSim	
	
	#region SettingCoins
	[System.SerializableAttribute]
	public class SettingCoinsSim
	{
		
		/// <summary>
		/// Coins
		/// </summary>
		public GameObject PrefabCoin;
	
		/// <summary>
		/// Number of coins instance after convert to zombie
		/// </summary>
		private int numInstanceCoins = 3;
		public int NumInstanceCoins
	    {
	        set { this.numInstanceCoins = value; }
	        get { return this.numInstanceCoins; }
	    }
		
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingCoinsSim SetupCoinsSim;
	#endregion SettingCoins	
	
	#region SettingIcon
	[System.SerializableAttribute]
	public class SettingIconSim
	{
		public GameObject PrefabIconSim;
		
		// List of special icons for music and mobile guy
		private List<GameObject> listSpecialIconSim;
		public  List<GameObject> ListSpecialIconSim
	    {
	        set { this.listSpecialIconSim = value; }
	        get { return this.listSpecialIconSim; }
	    }
		private GameObject instIconSim;
		public GameObject InstIconSim
	    {
	        set { this.instIconSim = value; }
	        get { return this.instIconSim; }
	    }		
		
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingIconSim SetupIconSim;
	#endregion SettingIcon		
	
	#region SettingStunned
	[System.SerializableAttribute]
	public class SettingStunnedSim
	{
		public GameObject GameObjectStarParticle;
		
		private GameObject instStarParticle;
		public GameObject InstStarParticle
	    {
	        set { this.instStarParticle = value; }
	        get { return this.instStarParticle; }
	    }
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingStunnedSim SetupStunnedSim;
	#endregion SettingStunned
	
	#region SettingLoveCatSim
	[System.SerializableAttribute]
	public class SettingLoveCatSim
	{
        // Arrow Love cat		
		public GameObject PrefabArrowLoveCat;			
		private GameObject instArrowLoveCat;	
		public GameObject InstArrowLoveCat
	    {
	        set { this.instArrowLoveCat = value; }
	        get { return this.instArrowLoveCat; }
	    }

       
		/// <summary>
		/// Check if there is love cat
		/// </summary>
		private bool isLoveCat = false;
		
		public bool IsLoveCat
   	 	{
        	get { return this.isLoveCat; }
			set { this.isLoveCat = value; }
    	}


        /// <summary>
        /// Check if love cat affect all zombies (with/without cat in head)
        /// </summary>
        private bool isAffectAllZombies = true;
        public bool IsAffectAllZombies
        {
            get { return this.isAffectAllZombies; }
        }		
		

		/// <summary>
		/// Check if affected by cat
		/// </summary>
		private bool isAffectedLoveCat = false;		
		public bool IsAffectedLoveCat
   	 	{
        	get { return this.isAffectedLoveCat; }
			set { this.isAffectedLoveCat = value; }
    	}
		
		
		/// <summary>
		/// Position of cat
		/// </summary>
		private Vector2 positionLoveCat = new Vector2(0,0);		
		public Vector2 PositionLoveCat
   	 	{
        	get { return this.positionLoveCat; }
			set { this.positionLoveCat = value; }
    	}
		
		/// <summary>
		/// Position of cat
		/// </summary>
		private float radiusLoveCat = 0;		
		public float RadiusLoveCat
   	 	{
        	get { return this.radiusLoveCat; }
			set { this.radiusLoveCat = value; }
    	}		
		
		/// <summary>
		/// Indicating if is settup the behaviour of love cat
		/// </summary>
		private bool setupLoveCat = false;
		public bool SetupLoveCat
	    {
	        set { this.setupLoveCat = value; }
	        get { return this.setupLoveCat; }
	    }
	}
	
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingLoveCatSim SetupLoveCatSim;
	#endregion SettingLoveCatSim	
	
	#region SettingFastCatSim
		
	/// <summary>
	/// Bonus for speed for zombie sim (chase and hunt state) 20% more of speed
	/// </summary>
	public void SetFastCat()
	{
		
		this.Settings.Z_Hunt_Speed += this.Settings.BonusSpeed;	
		int incrementChaseBonus = (int)(this.Settings.BonusSpeed * 0.5f);
		this.Settings.Z_Chase_Speed += (this.Settings.BonusSpeed + incrementChaseBonus);
	}
		
	
	#endregion SettingFastCatSim		
	
	#region InfectedBar
	
	[System.SerializableAttribute]
	public class SettingsBarInfectedSim
	{
		
		/// <summary>
		/// Infected Bar
		/// </summary>
		public GameObject InfectedBar;
		
		private GameObject instanceIB;
		public GameObject InstanceIB
	    {
	        set { this.instanceIB = value; }
	        get { return this.instanceIB; }
	    }
		
		/// <summary>
		/// Percent of infected
		/// </summary>
		private float percentInfected;	
		public float PercentInfected
	    {
	        set { this.percentInfected = value; }
	        get { return this.percentInfected; }
	    }
		
		/// <summary>
		/// The amount infected each time
		/// </summary>
		private float amountInfected = 10.0f;
		public float AmountInfected
	    {
	        set { this.amountInfected = value; }
	        get { return this.amountInfected; }
	    }
		/// <summary>
		/// The amount infected each time for clipped (10.0f/100.0f)
		/// </summary>
		private float amountInfectedClipped = 0.1f;
		public float AmountInfectedClipped
	    {
	        set { this.amountInfectedClipped = value; }
	        get { return this.amountInfectedClipped; }
	    }
		private float deltaTimePInfected = 0.25f; 
		public float DeltaTimePInfected
	    {
	        set { this.deltaTimePInfected = value; }
	        get { return this.deltaTimePInfected; }
	    }
		private float updateTimePInfected = 0.0f;
		public float UpdateTimePInfected
	    {
	        set { this.updateTimePInfected = value; }
	        get { return this.updateTimePInfected; }
	    }
		
		/// <summary>
		/// Whether if infected bar has been instanced yet
		/// </summary>
		private bool isInstanceInfectedBar;	
		public bool IsInstanceInfectedBar
	    {
	        set { this.isInstanceInfectedBar = value; }
	        get { return this.isInstanceInfectedBar; }
	    }
		
		private tk2dClippedSprite instaceClippedIB;
		public tk2dClippedSprite InstaceClippedIB
	    {
	        set { this.instaceClippedIB = value; }
	        get { return this.instaceClippedIB; }
	    }
		
		/// <summary>
		/// The amount infected clipped each time.
		/// </summary>
		private float amountInfectedClippedTime;
		public float AmountInfectedClippedTime
	    {
	        set { this.amountInfectedClippedTime = value; }
	        get { return this.amountInfectedClippedTime; }
	    }
		
		private float deltaTimeUpdateClippedX = 0.0f; 
		public float DeltaTimeUpdateClippedX
	    {
	        set { this.deltaTimeUpdateClippedX = value; }
	        get { return this.deltaTimeUpdateClippedX; }
	    }
		private float updateTimeClippedX  = 0.0f;
		public float UpdateTimeClippedX
	    {
	        set { this.updateTimeClippedX = value; }
	        get { return this.updateTimeClippedX; }
	    }
		
		private bool infectedIStrightwawy  = false;
		public bool InfectedIStrightwawy
	    {
	        set { this.infectedIStrightwawy = value; }
	        get { return this.infectedIStrightwawy; }
	    }
	
	}
	/// <summary>
	/// Setup for sim where is alerting others sims
	/// </summary>
	public SettingsBarInfectedSim SetupBarInfectedSim;
	
	
	#endregion InfectedBar
	
	#region txtStateDebug
	public GameObject prefabTxtStateDebug;
	private GameObject instanceStateDebug;
	private tk2dTextMesh txtStateDebug;	
	#endregion txtStateDebug	
		
	#region SettingPolyEnforce
	[System.SerializableAttribute]
	public class SettingPolyEnforce
	{
		/// <summary>
		/// Distance to search
		/// </summary>
		public float DistanceSearch;
		
		
		/// <summary>
		/// Time to wait to shoot
		/// </summary>
		public float TimeToWaitShoot;
		
		/// <summary>
		/// Time in alert state when no zombie
		/// </summary>
		public float TimeInAlertState;	

		
		/// <summary>
		/// Prefab to instance bulled
		/// </summary>
		public GameObject PrefabBullet;
		
		/// <summary>
		/// Instance bullet
		/// </summary>
		private bool instBullet;
		public bool InstBullet 
		{
	        get { return this.instBullet; }
			set { this.instBullet = value; }
	    }
		
		/// <summary>
		/// Time to spawn between bullets
		/// </summary>
		public float TimeSpawnBullet = 0.7f;
		
		/// <summary>
		/// Update time to spawn next bullet
		/// </summary>
		private float updateTimeSpawnBullet = 10.0f;
		public float UpdateTimeSpawnBullet 
		{
	        get { return this.updateTimeSpawnBullet; }
			set { this.updateTimeSpawnBullet = value; }
	    }

		
		#region Enforcement
		
		/// <summary>
		/// Prefab with special animation for office
		/// </summary>
		public GameObject PrefabSpecialAnimationOfficer;
		
		private bool activeOfficer = true;	
		public bool ActiveOfficer
	    {
	        set { this.activeOfficer = value; }
	        get { return this.activeOfficer; }
	    }
		
		
		private GameObject instAnimOfficer;
		public GameObject InstAnimOfficer
	    {
	        set { this.instAnimOfficer = value; }
	        get { return this.instAnimOfficer; }
	    }
		
		private bool initAnimRope;
		public bool InitAnimRope
	    {
	        set { this.initAnimRope = value; }
	        get { return this.initAnimRope; }
	    }
		
		private float timeUpdateAnimRope = 0.0f;
		public float TimeUpdateAnimRope
	    {
	        set { this.timeUpdateAnimRope = value; }
	        get { return this.timeUpdateAnimRope; }
	    }
		private float delayAnimRope;
		public float DelayAnimRope
	    {
	        set { this.delayAnimRope = value; }
	        get { return this.delayAnimRope; }
	    }
		
		
		#endregion SettingsSim
	}
	/// <summary>
	/// Bullet for enforce
	/// </summary>
	public SettingPolyEnforce SetupPolyEnforce;
	
	
	#endregion SettingPolyEnforce	

	#region Sounds
	/// <summary>
	/// Sound when sim stunned
	/// </summary>
	public AudioClip Stunned_Sound;

	/// <summary>
	/// Sound when zombie infected another sim
	/// </summary>
	public AudioClip Infected_Sound;

	/// <summary>
	/// Sound when sim is infected by cat
	/// </summary>
	public AudioClip Infected_ByCat_Sound;

	#endregion Sounds
	
	[System.NonSerializedAttribute]
	/// <summary>
	/// Finite state machine
	/// </summary>
	public FSMSystem FSMSim;	
	
	void Awake()
	{
        
    }
		
	void Start () 
	{		
		this.checkFloor();
		InitializeFSM();		
				
		// Setup zomibie
		if ((this.SetupInitSim.IntialState == State.Z_IDLE) ||
			(this.SetupInitSim.IntialState == State.Z_HUNT))
		{
			this.Stats.Infected = true;	
			GameManager.Instance.NumberInfected += 1;
            GameManager.Instance.TotalNumberInfected += 1;
		}

		this.Stats.CurrentState = this.SetupInitSim.IntialState;		
		this.SetupInitSim.InitState = this.Stats.CurrentState;
		this.Stats.CurrentDirection =  this.SetupInitSim.InitialDirection;
		this.Stats.Direction = this.SetupInitSim.InitialDirection;
		if (this.SetupInitSim.IntialState == State.C_WARN)
		{
			 this.SetupAlertSim.DirectionWarn = this.SetupInitSim.InitialDirection;
		}
		
		// Special Icon
		if ((this.Stats.TSkin == Skin.MOBILE_SKIN) || 
			(this.Stats.TSkin == Skin.MUSIC_SKIN))
		{
			this.SetupIconSim.ListSpecialIconSim = new List<GameObject>();
			this.SetupIconSim.ListSpecialIconSim.Add(this.transform.FindChild("icons").FindChild("noListenIconSprite").gameObject);
			this.SetupIconSim.ListSpecialIconSim.Add(this.transform.FindChild("icons").FindChild("noSeeIconSprite").gameObject);			
		}

		
		if (GameSettings.DEBUG_MODE)
		{
			this.initStateDebug();
		}	
		

		// Oficer
		this.initOfficer();
		this.initIconShield();		
			
		// Shadow
		this.Settings.ShadowSim = this.transform.FindChild("bodyColliderShadow");
		
        this.Settings.HelperCircleWarn.DisableCircle();	
		
		this.ChangeState(this.SetupInitSim.IntialState);
	}	
	
	/// <summary>
	/// Inits the text for state debug
	/// </summary>
	private void initStateDebug()
	{
		Vector3 auxPos = new Vector3(this.transform.position.x,
			this.transform.position.y,
			this.transform.position.z - 5.0f);
		this.instanceStateDebug = Instantiate(this.prefabTxtStateDebug,auxPos, Quaternion.identity) as GameObject;
		this.txtStateDebug = this.instanceStateDebug.transform.GetComponent<tk2dTextMesh>();
		
	}
	
	
	/// <summary>
	/// Method to init shield
	/// </summary>
	private void initIconShield()
	{
		if (this.SetupIconSim.InstIconSim == null)
		{
			if (this.Stats.HasShield)
			{
				Vector3 auxPos = new Vector3(this.transform.position.x,
					this.transform.position.y + 135.0f,
					this.transform.position.z - 5.0f);
				this.SetupIconSim.InstIconSim = Instantiate(this.SetupIconSim.PrefabIconSim,auxPos, Quaternion.identity) as GameObject;
				tk2dSprite spriteIcon = this.SetupIconSim.InstIconSim.transform.GetComponent<tk2dSprite>();
				if (spriteIcon != null)
				{
					spriteIcon.SetSprite(spriteIcon.GetSpriteIdByName("icon_inmune"));
				}
			}
			
		}
		
	}
	
	private void changeIconShield()
	{
		if (this.SetupIconSim.InstIconSim != null)
		{
			tk2dSprite spriteIcon = this.SetupIconSim.InstIconSim.transform.GetComponent<tk2dSprite>();
			if (spriteIcon != null)
			{
				spriteIcon.SetSprite(spriteIcon.GetSpriteIdByName("icon_noInmune"));
			}
		}
		
	}
	
	/// <summary>
	/// Init icon of sim according of type of skin
	/// </summary>
	public void InitIconSim()
	{
		
		if ((this.SetupIconSim.InstIconSim == null) && (this.Stats.TSkin != Skin.NONE_SKIN) && (this.Stats.TSkin != Skin.POLY_SKIN))
		{
			Vector3 auxPos = new Vector3(this.transform.position.x,
				this.transform.position.y + 135.0f,
				this.transform.position.z - 5.0f);
			this.SetupIconSim.InstIconSim = Instantiate(this.SetupIconSim.PrefabIconSim,auxPos, Quaternion.identity) as GameObject;
			tk2dSprite spriteIcon = this.SetupIconSim.InstIconSim.transform.GetComponent<tk2dSprite>();
			if (spriteIcon != null)
			{
				switch (this.Stats.TSkin)
				{
					case Skin.BUSINESS_SKIN:
					case Skin.GEEK_SKIN:	
					case Skin.LEADER_SKIN:
						
						spriteIcon.SetSprite(spriteIcon.GetSpriteIdByName("icon_alert"));
					break;
					
				}				
				this.activeSpecialIcon(false);				
			}
			
		}else if (this.SetupIconSim.InstIconSim != null)
		{
			if (this.Stats.HasShield)
			{	
				tk2dSprite spriteIcon = this.SetupIconSim.InstIconSim.transform.GetComponent<tk2dSprite>();
				spriteIcon.SetSprite(spriteIcon.GetSpriteIdByName("icon_alert"));
			}
			
		}

	}
	
	private void activeSpecialIcon(bool active)
	{
		if ((this.Stats.TSkin == Skin.MUSIC_SKIN) || (this.Stats.TSkin == Skin.MOBILE_SKIN))
		{
			if (this.SetupIconSim.ListSpecialIconSim != null)
			{
				foreach (GameObject auxGO in this.SetupIconSim.ListSpecialIconSim)
				{
					auxGO.SetActive(active);	
				}
			}
		}
		
	}
	
	/// <summary>
	/// Inits the officer.
	/// </summary>
	private void initOfficer()
	{
		switch(this.Stats.TypeOfficer)
		{
			case SimController.TOfficer.OFFICER_NONE:
				this.SetupPolyEnforce.ActiveOfficer = false;
			
			break;
			
			case SimController.TOfficer.OFFICER_COMMON:
				this.SetupPolyEnforce.ActiveOfficer = true;
			
			break;
			
			case SimController.TOfficer.OFFICER_FALL:			
				this.SetupPolyEnforce.ActiveOfficer = true;	
			
			break;
			
		}
		
	}
	
	private void updateOfficer()
	{
		if (this.Stats.TypeOfficer == TOfficer.OFFICER_FALL)
		{
			if (!this.SetupPolyEnforce.InitAnimRope)
			{
				if ((this.SetupPolyEnforce.ActiveOfficer) && (this.SetupPolyEnforce.InstAnimOfficer == null))
				{
					this.SetupPolyEnforce.InstAnimOfficer = GameObject.Instantiate(this.SetupPolyEnforce.PrefabSpecialAnimationOfficer,this.SetupPolyEnforce.PrefabSpecialAnimationOfficer.transform.position, Quaternion.identity) as GameObject;
					this.SetupPolyEnforce.InitAnimRope = true;
				}
				
			}else
			{
				this.SetupPolyEnforce.TimeUpdateAnimRope += Time.deltaTime;
				if (this.SetupPolyEnforce.TimeUpdateAnimRope >= this.SetupPolyEnforce.DelayAnimRope)
				{
					
					Destroy(this.SetupPolyEnforce.InstAnimOfficer);
					// Active officer
					this.Stats.TypeOfficer = TOfficer.OFFICER_COMMON;
					this.ChangeState(State.C_IDLE);
				}				
			}			
		}		
	}
	
	/// <summary>
	/// Method to remove shield of sim
	/// </summary>
	public void RemoveShield()
	{
		this.Stats.HasShield = false;
		if (this.Stats.CurrentState == State.C_IDLE)
		{
			this.FSMSim.ForceChangeState((int)State.C_IDLE);
			
		}else
		{
			
			this.ChangeState(State.C_IDLE);	
		}
		this.changeIconShield();
		
	}
	
	/// <summary>
	/// Method to infect a civil by cat
	/// </summary>	
	public bool InfectedSimByCat(bool infectStraightaway)
	{
		if ((this.Stats.CurrentState != State.INFECT_CAT) && (!this.Stats.Infected))
		{
			this.SetupBarInfectedSim.InfectedIStrightwawy = infectStraightaway;
			if (!infectStraightaway)
			{			
				// Change state to infected by cat
				this.ChangeState(State.INFECT_CAT);
			
				// If there is love cat, infected by cat
				if (this.SetupLoveCatSim.IsLoveCat)
				{				
					this.SetupLoveCatSim.IsAffectedLoveCat = true;
				}	
			}else
			{
				this.endIsInfected();
			}
			
			return true;					
		}
		
		return false;		
	}
	
	/// <summary>
	///	Method to check if a Sim is stunned
	/// </summary>
	public bool Stunned()
	{
		if ((!this.Stats.IsInfecting) && (!this.Stats.Infected) && 
			((this.Stats.CurrentState) == (SimController.State.C_ALERT))|| 
			((this.Stats.CurrentState) == (SimController.State.C_IDLE))|| 
			((this.Stats.CurrentState) == (SimController.State.C_PATROL_DISTANCE))|| 
			((this.Stats.CurrentState) == (SimController.State.C_SHOOT))|| 
			((this.Stats.CurrentState) == (SimController.State.C_WARN))
			)
			
		{
			if (this.Stats.HasShield)
			{
				this.RemoveShield();
			}else
			{
				this.ChangeState(State.STUNNED);
				// Instance star for stunned
				if (this.SetupStunnedSim.InstStarParticle == null)
				{
					Vector3 auxPosition = this.transform.position;
					if (this.Stats.CurrentDirection == DirectionType.LEFT)
					{					
						auxPosition.x +=5.0f; 
					}else if (this.Stats.CurrentDirection == DirectionType.RIGHT)
					{
						auxPosition.x -=5.0f; 
					}
					auxPosition.z -=0.2f; 
					this.SetupStunnedSim.InstStarParticle = GameObject.Instantiate(this.SetupStunnedSim.GameObjectStarParticle, 
						auxPosition, 
						Quaternion.identity) as GameObject;
				}
			}
			return true;
		}
		return false;
	}
	
	void InitializeFSM()
	{
		 FSMSim = new FSMSystem();
		
		// Create states
		CatInfectState catInfect = new CatInfectState(this.gameObject,(int)State.INFECT_CAT);
		ZombieInfectState zombieInfect = new ZombieInfectState(this.gameObject,(int)State.INFECT_ZOMBIE);
		FallState fallState = new FallState(this.gameObject,(int)State.SIM_FALL);
		
		C_IdleState c_idle = new C_IdleState(this.gameObject,(int)State.C_IDLE);
		
		C_PatrollDistanceState c_distpatroll = new C_PatrollDistanceState(this.gameObject,(int)State.C_PATROL_DISTANCE);
		C_WarnState c_warn = new C_WarnState(this.gameObject,(int)State.C_WARN);
		C_AlertState c_alert = new C_AlertState(this.gameObject,(int)State.C_ALERT);
		C_ShootState c_shoot = new C_ShootState(this.gameObject,(int)State.C_SHOOT);	
	
		
		Z_IdleState z_idle = new Z_IdleState(this.gameObject,(int)State.Z_IDLE);
		Z_HuntState z_hunt = new Z_HuntState(this.gameObject,(int)State.Z_HUNT);
		Z_ChaseState z_chase = new Z_ChaseState(this.gameObject,(int)State.Z_CHASE);
		Z_AttackState z_attack = new Z_AttackState(this.gameObject,(int)State.Z_ATTACK);
		Z_HurtState z_hurt = new Z_HurtState(this.gameObject,(int)State.Z_HURT);
		Z_DeathState z_death = new Z_DeathState(this.gameObject,(int)State.Z_DEATH);
		
		StunnedState stunned = new StunnedState(this.gameObject,(int)State.STUNNED);
		InactiveState inactive = new InactiveState(this.gameObject,(int)State.INACTIVE);
		
		// Add states
		FSMSim.AddState(catInfect);
		FSMSim.AddState(zombieInfect);
		FSMSim.AddState(fallState);
		
		FSMSim.AddState(c_idle);		
		FSMSim.AddState(c_distpatroll);	
		FSMSim.AddState(c_warn);
		FSMSim.AddState(c_alert);
		FSMSim.AddState(c_shoot);
		
		
		FSMSim.AddState(z_idle);
		FSMSim.AddState(z_hunt);		
		FSMSim.AddState(z_chase);
		FSMSim.AddState(z_attack);
		FSMSim.AddState(z_hurt);
		FSMSim.AddState(z_death);
		FSMSim.AddState(stunned);
		FSMSim.AddState(inactive);		

		
	}
	
	public void ChangeState(SimController.State newState)
	{
		this.FSMSim.ChangeState((int)newState);
		this.Stats.CurrentState = newState;
	}
	
	// Update is called once per frame
	void Update () 
	{
		if (!GameManager.Instance.GameStats.EndLevel)
		{
			// Update Finite State machine
			FSMSim.UpdateState();	
			this.Stats.CurrentState = (SimController.State)this.FSMSim.CurrentStateID;
			
			if (GameSettings.DEBUG_MODE)
			{
				this.updateStateDebug();
			}
			this.Stats.CurrentDirection = this.Stats.Direction;
			
			// Update icon of sim
			this.updateIconSim();
			
			// Idate arrow love cat
			this.updateArrowLoveCat();
			
			// Manage States
			this.manageStates();
			
			// ManageOfficer
			this.updateOfficer();
			
			this.checkSimHunted();			
		}	
	}   

	/// <summary>
	/// Method to check sim hunted
	/// </summary>
	private void checkSimHunted()
	{
		// Not infected and has sim hunted
		if ((!this.Stats.Infected) && (this.Stats.SimHunted != null))
		{
			// Check distance (other sim is far from me) more than distance war
			Vector2 auxPosSim = new Vector2(this.transform.position.x, 0);
			Vector2 auxHuntedPosSim = new Vector2(this.Stats.SimHunted.transform.position.x, 0);
			float auxDistanceSims = Vector2.Distance(auxPosSim,auxHuntedPosSim);
			
			// Check if sim hunted is behind me (according to direction of zombie)
			bool isBehind = false;
			// Me: -400, Hunted: -300 , hunted looking at right (he is behind)
			if (this.transform.position.x < this.Stats.SimHunted.transform.position.x)
			{
				if (this.Stats.SimHunted.Stats.Direction == DirectionType.RIGHT)
				{
					isBehind = true;
				}
				
			// Me: -100, Hunted: -200 , hunted looking at left (he is behind)
			}else if (this.transform.position.x > this.Stats.SimHunted.transform.position.x)
			{
				if (this.Stats.SimHunted.Stats.Direction == DirectionType.LEFT)
				{
					isBehind = true;
				}				
			}			
			
			if ((auxDistanceSims >= 300.0f) || (isBehind))
			{
				this.Stats.SimHunted = null;
			}
		}		
	}
	
	private void updateStateDebug()
	{
		if (txtStateDebug != null)
		{
			Vector3 auxPos = new Vector3(this.transform.position.x,
				this.transform.position.y,
				this.transform.position.z - 5.0f);
			this.instanceStateDebug.transform.position = auxPos;
			this.txtStateDebug.text = this.Stats.CurrentState.ToString();
			this.txtStateDebug.Commit();
		}		
	}
	
	private void updateIconSim()
	{
		if (this.SetupIconSim.InstIconSim != null)
		{
			Vector3 auxPos = new Vector3(this.transform.position.x,
				this.transform.position.y + 135.0f,
				this.transform.position.z - 5.0f);
			this.SetupIconSim.InstIconSim.transform.position = auxPos;
			this.activeSpecialIcon(false);
			
		}
	}
	public void RemoveIconSim()
	{
		if (this.SetupIconSim.InstIconSim != null)
		{
			Destroy(this.SetupIconSim.InstIconSim);
			this.Stats.SimZombie = null;
		}
		this.activeSpecialIcon(false);
		
	}
	
	public void RemoveStarsStunned()
	{
		if (this.SetupStunnedSim.InstStarParticle != null)
		{
			Destroy(this.SetupStunnedSim.InstStarParticle);			
		}
		
	}
	
	/// <summary>
	/// Method to check, wich
	/// </summary>
	public void checkFloor()
	{
		if (GameManager.Instance.SetupInitLevel.VariousHeights)
		{
			float auxY = this.transform.position.y;
			if (auxY > 220.0f)
			{
				this.Stats.Number_Floor = 1;
			}else
			{
				this.Stats.Number_Floor = 0;
			}
		}else
		{
			this.Stats.Number_Floor = 0;
		}
		
	}
	
	
	
	void LateUpdate() 
	{
		if (!GameManager.Instance.GameStats.EndLevel)
		{		
			if ((this.Stats.CurrentState != SimController.State.SIM_FALL) && 
				(this.Stats.CurrentState != SimController.State.INACTIVE) && 
				(this.Stats.CurrentState != SimController.State.Z_DEATH))
			{
				this.relocatePositionSim();
			}
			
			this.checkLinkSims();
			this.checkBoundaries();
			
			this.controlZPosShadow();

		}
    }
	
	/// <summary>
	/// Relocates the position of Sim respect a maximun and minium Z and Y
	/// </summary>
	private void relocatePositionSim()
	{
		if (!this.Stats.OutsideScenery)
		{
			// Clamping elements
			float posY = ConfigZ.clamp_y(this.Stats.Number_Floor,this.transform.position.y);
			float posZ = ConfigZ.clamp_z(this.Stats.Number_Floor,posY);
			this.transform.position = new Vector3(this.transform.position.x, posY,posZ);

		}
	}
	
	private void controlZPosShadow()
	{
		if (this.Settings.ShadowSim != null)
		{
			if (this.Stats.Direction == DirectionType.LEFT)
			{
				this.Settings.ShadowSim.localPosition = new Vector3(this.Settings.ShadowSim.localPosition.x,this.Settings.ShadowSim.localPosition.y, 0.1f);
			}else 
			{
				this.Settings.ShadowSim.localPosition = new Vector3(this.Settings.ShadowSim.localPosition.x,this.Settings.ShadowSim.localPosition.y, -0.1f);
			}
		}
	}
	
	
	private void checkBoundaries()
	{	
		// Check if end level
		float offset = 60.0f;
		if ((this.transform.position.x < (GameManager.Instance.SetupPrefabs.LeftBoundary.transform.position.x - offset )) ||
			(this.transform.position.x > (GameManager.Instance.SetupPrefabs.RightBoundary.transform.position.x + offset ) ))
		{
            GameManager.Instance.SetupInitLevel.TimeShowPanel = GameManager.Instance.SetupInitLevel.DelayShowPanelSimBoundaries;
			CameraController.Instance.ZoomEndLevel(this.transform);
			GameManager.Instance.ShowEndPanel(GameManager.TypePanel.NONE_PANEL,true);
		}	
	}
	
	
	private void alertSimByDistance()
	{
		// Modo hunt but not simChase, find
		List<SimController.State> auxLStates = new List<SimController.State>();
		auxLStates.Add(SimController.State.C_IDLE);
		auxLStates.Add(SimController.State.C_PATROL_DISTANCE);
		auxLStates.Add(SimController.State.C_SHOOT);		
		
		List<SimController> auxList = GameManager.Instance.GetSimsInDistance(this,this.SetupAlertSim.DistanceSearchWarn,false,auxLStates);
		
		if (auxList != null)
		{			
			foreach (SimController auxSim in auxList)
			{
				switch (auxSim.Stats.TSkin)
				{
					case Skin.BUSINESS_SKIN:
					case Skin.GEEK_SKIN:
					case Skin.LEADER_SKIN:
						if (this.transform.position.x <= auxSim.transform.position.x)
						{
							auxSim.SetupAlertSim.DirectionWarn = DirectionType.RIGHT;
						}else
						{
							auxSim.SetupAlertSim.DirectionWarn = DirectionType.LEFT	;
						}
						auxSim.ChangeState(SimController.State.C_WARN);
					break;
					case Skin.POLY_SKIN:
						if (this.transform.position.x <= auxSim.transform.position.x)
						{
							auxSim.SetupAlertSim.DirectionWarn = DirectionType.RIGHT;
						}else
						{
							auxSim.SetupAlertSim.DirectionWarn = DirectionType.LEFT;
						}
						auxSim.Stats.SimZombie = this.Stats.SimHunted;
						auxSim.ChangeState(SimController.State.C_ALERT);
					
					break;
				}						
			}
		}
		
	}

	/// <summary>
	/// Manages states of sim
	/// </summary>
	private void manageStates()
	{
		if (this.Stats.CurrentState == SimController.State.STUNNED)
		{
			this.RemoveIconSim();
		}		
			
		if (this.Stats.CurrentState == SimController.State.INFECT_ZOMBIE)
		{
			// Update bar
			this.RemoveIconSim();
			
			this.RemoveStarsStunned();
			
			this.updateInfectedBar();
			
			if (this.SetupBarInfectedSim.PercentInfected >= 10.0f)
			{
				this.alertSimByDistance();
			}
		}
		
		if (this.Stats.CurrentState == SimController.State.SIM_FALL)
		{
			// Update bar
			this.RemoveIconSim();	
			if (this.Stats.Infected)
			{
				// Remove sim chase and sim hunted if if is me
				if (this.Stats.SimChase != null)
				{
					if (this.Stats.SimChase.Stats.SimHunted != null)
					{
						if (this.Stats.SimChase.Stats.SimHunted.Stats.IDSim == this.Stats.IDSim)
						{
							this.Stats.SimChase.Stats.SimHunted = null;
						}
					}
					this.Stats.SimChase = null;
				}
			}
			
		}
		
		// Infected by another cat
		if (this.Stats.CurrentState == SimController.State.INFECT_CAT)
		{
			this.RemoveStarsStunned();
			this.RemoveIconSim();
			this.updateInfectedBar();
			
		}
		
		if (this.Stats.CurrentState == SimController.State.Z_IDLE)
		{
			// Few seconds in idle and change HUNT 
			this.updateTimeResetHunt+= Time.deltaTime;
			if (this.updateTimeResetHunt >= this.deltaTimeResetHunt)
			{
				this.updateTimeResetHunt = 0.0f;
				this.ChangeState(State.Z_HUNT);
			}
		}
		
		// Attack another civil
		if (this.Stats.CurrentState == SimController.State.Z_ATTACK)
		{
			if (this.Stats.SimChase  != null)
			{
				// If this sim is infected by cat, this sim goto state hunt
				if (this.Stats.SimChase.Stats.CurrentState == SimController.State.INFECT_CAT)
				{
					this.updateTimeResetHunt = 0;	
					// Reset bar
					this.Stats.SimChase.ResetBar();
					this.Stats.SimChase.Stats.SimHunted = null;
					this.Stats.SimChase = null;
					this.ChangeState(State.Z_HUNT);
					
				}else
				{
					// If infected is true, percent is 100 (wait seconds to change estate)
					if (this.Stats.SimChase.Stats.Infected)
					{
						this.Stats.SimChase.Stats.SimHunted = null;
						this.Stats.SimChase = null;
						this.ChangeState(State.Z_IDLE);
									
					}
				}
				
			}
			
		}
		
		if (this.Stats.CurrentState == SimController.State.Z_CHASE)
		{
			// If its sim is not in mode hurt, go for it
			if (this.Stats.SimChase != null)
			{
				// If other sim is on attack, return to hunt 
				if ((this.Stats.SimChase.Stats.CurrentState == State.Z_ATTACK) || 
					(this.Stats.SimChase.Stats.CurrentState == State.INFECT_CAT) ||
					(this.Stats.SimChase.Stats.CurrentState == State.SIM_FALL) ||
					(this.Stats.SimChase.Stats.CurrentState == State.INACTIVE)
					)
				{
					this.Stats.SimChase.Stats.SimHunted = null;
					this.Stats.SimChase = null;
					this.ChangeState(State.Z_HUNT);
				}
			}
			
		}
		
		if (this.Stats.CurrentState == SimController.State.C_SHOOT)
		{
			
			this.SetupPolyEnforce.InstBullet = true;	
			this.updateBullet();	
			// if sim is on state fall change to c_warn
			if (this.Stats.SimZombie != null)
			{
				if (this.Stats.SimZombie.Stats.CurrentState == State.SIM_FALL)
				{
					this.Stats.SimZombie = null;
					this.ChangeState(this.SetupInitSim.InitState);
				}
			}
			
		}
		
		if (this.Stats.CurrentState == SimController.State.Z_DEATH)
		{	
			if (!this.SetupZombieSim.IsEndCloud)
			{
				if (this.SetupZombieSim.InstZCloudPrefab == null)
				{
					Vector3 auxP = new Vector3(this.transform.position.x, 
						this.transform.position.y, this.transform.position.z - 0.1f);
					this.SetupZombieSim.InstZCloudPrefab = Instantiate(this.SetupZombieSim.ZombieCloudPrefab,auxP, Quaternion.identity) as GameObject;
					this.SetupZombieSim.IsEndCloud = false;
					this.SetupZombieSim.UpdateTimeShowCloud = 0.0f;
					this.transform.position = new Vector3(this.transform.position.x, this.transform.position.y,500.0f);
				}else
				{
					this.SetupZombieSim.UpdateTimeShowCloud += Time.deltaTime;
					if (this.SetupZombieSim.UpdateTimeShowCloud >= this.SetupZombieSim.DeltaTimeShowCloud)
					{
						this.SetupZombieSim.IsEndCloud = true;
						Destroy(this.SetupZombieSim.InstZCloudPrefab);	
						this.DestroySim();
                        GameManager.Instance.NumberDeathZombies += 1;

					}
					
				}
			}
			
		}
		
		// FindCivil
		this.findCivilByDistance();
		
		// Find zombie
		this.findZombieByDistance();
		
	}
	
	public void DestroySim()
	{
		this.RemoveIconSim();
		this.RemoveStarsStunned();
		if (this.txtStateDebug != null)
		{
			Destroy(this.txtStateDebug);							
		}
		RemoveCat();
	}
	/// <summary>
	/// Method to remove cat
	/// </summary>
	public void RemoveCat()
	{
		CatAttacher auxCat = this.transform.GetComponent<CatAttacher>();
        if (auxCat != null)
		{
            if (auxCat.cat != null)
            {
                auxCat.cat.GetComponent<CatController>().DestroyCat();
                Destroy(auxCat.cat);
            }
		}
	}
	
	
	/// <summary>
	/// Method to find a zombie in any state
	/// </summary>
	private void findZombieByDistance()
	{
		// Find zombie
		if ((!this.Stats.Infected) && 
			(this.Stats.TSkin != Skin.MOBILE_SKIN) && 
			(this.Stats.TSkin != Skin.MUSIC_SKIN))
		{
			
			if ((this.Stats.CurrentState == SimController.State.C_IDLE) ||
				(this.Stats.CurrentState == SimController.State.C_PATROL_DISTANCE) ||
				(this.Stats.CurrentState == SimController.State.C_WARN))
			{				
					
				// List of states to find
				List<SimController.State> auxLStates = new List<SimController.State>();
				auxLStates.Add(SimController.State.Z_ATTACK);
				auxLStates.Add(SimController.State.Z_CHASE);
				auxLStates.Add(SimController.State.Z_HUNT);
				auxLStates.Add(SimController.State.Z_IDLE);
				
				// Special officer, find sim infected by cat
				bool isInfecting = false;
								
				GameManager.DirectionFindType auxDir = GameManager.DirectionFindType.LEFT;				
				
				if (this.Stats.Direction == DirectionType.LEFT)
				{
					auxDir = GameManager.DirectionFindType.LEFT;
					
				}else if (this.Stats.Direction == DirectionType.RIGHT)
				{
					auxDir = GameManager.DirectionFindType.RIGHT;
					
				}
				
				// Setup distance
				float distanceToSearch = this.SetupAlertSim.DistanceSearchWarn;
				if (this.Stats.TSkin == Skin.POLY_SKIN)
				{
					distanceToSearch = this.SetupPolyEnforce.DistanceSearch;
				}
				
				Transform auxCat = null;
				bool checkCat = false;
				if ((this.Stats.TSkin == Skin.POLY_SKIN) && 
					(this.Stats.TypeOfficer == TOfficer.OFFICER_SPECIAL_ENFORCEMENT))
				{
					// Find cat	
					bool findCat = GameManager.Instance.GetCatByDirection(this, out auxCat, auxDir,distanceToSearch);
					if (findCat)
					{
						checkCat = true;
					}
				}
				
				SimController auxZombie = null;	

				bool findZombie = GameManager.Instance.GetSimByDirection(this, out auxZombie, auxDir, true, isInfecting,auxLStates,distanceToSearch);
												
				if ((findZombie) && (auxZombie.Stats.IDSim != this.Stats.IDSim))
				{	
					// Type skin poly, change ai,
					if (this.Stats.TSkin == Skin.POLY_SKIN)
					{
						if (checkCat)
						{
							// Check distance between cat and zombie
							float auxDistanceToCat = Vector2.Distance(new Vector2(auxCat.position.x, auxCat.position.y),
								new Vector2(this.transform.position.x, this.transform.position.y));
							float auxDistanceToZombie = Vector2.Distance(new Vector2(auxZombie.transform.position.x, auxZombie.transform.position.y),
								new Vector2(this.transform.position.x, this.transform.position.y));
							//bool setupCat = false;
							if (auxDistanceToCat < auxDistanceToZombie )
							{
								//setupCat = true;
								this.Stats.CatChasePoly = auxCat;
								this.Stats.SimZombie = null;

								this.ChangeState(State.C_ALERT);
							}else
							{
								this.Stats.SimZombie = auxZombie;

								this.Stats.CatChasePoly = null;
								this.ChangeState(State.C_ALERT);
							}
						}else
						{
						
							this.Stats.SimZombie = auxZombie;

							this.Stats.CatChasePoly = null;
							this.ChangeState(State.C_ALERT);
						}
					}else
					{						
						bool setZombie = false;
												
						// Another zombie and is diferent from new zombie
						if (this.Stats.SimZombie != null)
						{  
							if (auxZombie.Stats.IDSim != this.Stats.SimZombie.Stats.IDSim)
							{
								setZombie = true;
							}					
						
						}else
						{
							setZombie = true;						
						}
					
						if (setZombie)
						{
														
							this.Stats.SimZombie = auxZombie;
							
							// Setup direction of warn, opossit form new sim
							if (this.Stats.SimZombie.transform.position.x <= this.transform.position.x)
							{
								this.SetupAlertSim.DirectionWarn = DirectionType.RIGHT;
							}else
							{
								this.SetupAlertSim.DirectionWarn = DirectionType.LEFT;
							}							
							
							// If not on state C_WARN
							if (this.Stats.CurrentState != SimController.State.C_WARN)
							{
						
								// Change state warn
								this.ChangeState(State.C_WARN);								
								
							}else 
							{								
								this.ChangeState(State.C_WARN);
								
							}
						}
					}					
				}else if (checkCat)
				{
					this.Stats.SimZombie = null;
					this.Stats.CatChasePoly = auxCat;
					this.ChangeState(State.C_ALERT);
				}
			}
		}
		
	}
	
	/// <summary>
	/// Method to find a civil in any state
	/// </summary>
	private void findCivilByDistance()
	{
				
		if (this.Stats.Infected)
		{
		
			if ((this.Stats.CurrentState == SimController.State.Z_HUNT) ||
				(this.Stats.CurrentState == SimController.State.Z_IDLE) ||
				(this.Stats.CurrentState == SimController.State.Z_CHASE))
			{
				SimController auxChase;
					
				
				// Modo hunt but not simChase, find
				List<SimController.State> auxLStates = new List<SimController.State>();
				auxLStates.Add(SimController.State.C_IDLE);
				auxLStates.Add(SimController.State.C_PATROL_DISTANCE);
				auxLStates.Add(SimController.State.C_WARN);
				auxLStates.Add(SimController.State.C_ALERT);
				auxLStates.Add(SimController.State.C_SHOOT);		
				auxLStates.Add(SimController.State.STUNNED);
				
				// Find in this direction
				GameManager.DirectionFindType auxDir = GameManager.DirectionFindType.BOTH;
				if (this.Stats.Direction == DirectionType.LEFT)
				{
					auxDir = GameManager.DirectionFindType.LEFT;
				}else if (this.Stats.Direction == DirectionType.RIGHT)
				{
					auxDir = GameManager.DirectionFindType.RIGHT;
				}
				float distanceSearch = this.SetupAlertSim.DistanceSearchWarn;
				if (GameManager.Instance.GetSimByDirection(this, out auxChase,auxDir, false, false,auxLStates, distanceSearch))
				{
                   								
					// If chase
					if (this.Stats.CurrentState == State.Z_CHASE)
					{
						
						// If has sim hunted and is distint to current, check distance
						if ((this.Stats.SimChase != null) && (this.Stats.SimChase.Stats.IDSim != auxChase.Stats.IDSim) && 
							(this.Stats.IDSim != auxChase.Stats.IDSim) && 
							(auxChase.Stats.SimHunted == null))
						{
							// Check distances between zombie and sim
							float distToCurrentChase = Vector2.Distance(this.transform.position, this.Stats.SimChase.transform.position);
							float distToOtherChase = Vector2.Distance(this.transform.position, auxChase.transform.position);
							// If other hunted is near to zombie, change hunted
							if (distToCurrentChase > distToOtherChase)
							{
								// Change hunted
								this.Stats.SimChase.Stats.SimHunted = null;
								this.Stats.SimChase = auxChase;
								auxChase.Stats.SimHunted = this;
								
							}						
						}					
					}else
					{
						// Not me
						if (this.Stats.IDSim != auxChase.Stats.IDSim)
						{							
							// Check if other sim have simHunted yet, and is me
							bool auxGotoSim = false;							
							if (auxChase.Stats.SimHunted != null)
							{

                                // Check distances between me and other Zombie hunted
                                float distMeFromSimChase = Vector2.Distance(this.transform.position, auxChase.transform.position);
                                float distFromOtherHunt = Vector2.Distance(auxChase.Stats.SimHunted.transform.position, auxChase.transform.position);
                                // If me is less than other
                                if (distMeFromSimChase < distFromOtherHunt)
                                {

                                    // Change other
                                    if (auxChase.Stats.SimHunted.Stats.CurrentState == State.Z_CHASE)
                                    {
                                        if (auxChase.Stats.SimHunted.Stats.Infected)
                                        {
                                            //auxChase.Stats.SimHunted = null;
                                            auxChase.Stats.SimHunted.Stats.SimChase = null;
                                            auxChase.Stats.SimHunted.ChangeState(State.Z_HUNT);
                                        }
                                    }
                                    this.Stats.SimChase = auxChase;
                                    auxChase.Stats.SimHunted = this;                                    
                                    auxGotoSim = true;

                                }	

								/*if (auxChase.Stats.SimHunted.Stats.IDSim == this.Stats.IDSim)
								{
									auxGotoSim = true;
								}*/
								
							}else
							{
								auxGotoSim = true;
							}                           
							
							if (auxGotoSim)
							{							
								// Change state a chase
								this.Stats.SimChase = auxChase;
								// Setup I'm hunted of other
								auxChase.Stats.SimHunted = this;
								this.ChangeState(State.Z_CHASE);
	
							}
						}
					}					
				}
				
			}	
		}
		
	}
	
	
	private void updateBullet()
	{
		if (this.SetupPolyEnforce.InstBullet)
		{
			this.SetupPolyEnforce.UpdateTimeSpawnBullet += Time.deltaTime;
			
			if (this.SetupPolyEnforce.UpdateTimeSpawnBullet >= this.SetupPolyEnforce.TimeSpawnBullet)
			{
				this.SetupPolyEnforce.UpdateTimeSpawnBullet = 0.0f;
				
				// Instance new bullet
				float radiusChar = this.transform.GetComponent<CharacterController>().radius;
				float auxXPosition = this.transform.position.x;
				float offsetX = 20.0f;
				Quaternion rotation = Quaternion.identity;
				if (this.Stats.Direction == DirectionType.LEFT)
				{
					auxXPosition -= (radiusChar + offsetX);
				}else if (this.Stats.Direction == DirectionType.RIGHT)
				{
					auxXPosition += (radiusChar + offsetX);

				}
				Vector3 auxPos = new Vector3(auxXPosition, this.transform.position.y + radiusChar + 20.0f, this.transform.position.z + 0.1f);
				GameObject auxGO = GameObject.Instantiate(this.SetupPolyEnforce.PrefabBullet, auxPos, rotation) as GameObject;
				Bullet auxBullet = auxGO.transform.GetComponent<Bullet>();
				if (auxBullet != null)
				{
					this.ChangeState(State.C_ALERT);
					if (this.Stats.SimZombie != null)
					{
                        auxBullet.SetBullet(this.Stats.SimZombie, this);
						
					}else if(this.Stats.CatChasePoly != null)
					{
                        // Check if catChase is alive
                        CatController auxCat = this.Stats.CatChasePoly.GetComponentInChildren<CatController>();
                        if (auxCat != null)
                        {
                            auxBullet.SetBullet(this.Stats.CatChasePoly, this);
                        }
                        else
                        {
                            this.Stats.CatChasePoly = null;
                            this.ChangeState(SimController.State.C_ALERT);
                            //this.ChangeState(this.SetupInitSim.InitState);
                        }
					}
					
				}
			}
		}
			
	}
	
	#region InfectedBar
	/// <summary>
	/// Method to reset bar
	/// </summary>
	public void ResetBar()
	{
		if (this.SetupBarInfectedSim.IsInstanceInfectedBar)
		{
			this.Stats.IsInfecting = false;
			this.SetupBarInfectedSim.PercentInfected = 0.0f;
			this.SetupBarInfectedSim.UpdateTimeClippedX = 0.0f;
			float auxParts = 7.0f;
			this.SetupBarInfectedSim.DeltaTimeUpdateClippedX = this.SetupBarInfectedSim.DeltaTimePInfected / auxParts;			
			this.SetupBarInfectedSim.AmountInfectedClippedTime = this.SetupBarInfectedSim.AmountInfectedClipped / auxParts;
			Rect auxClipRect = new Rect(-1.0f,
				this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.y,
				this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.width,
				this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.height);
			this.SetupBarInfectedSim.InstaceClippedIB.ClipRect = auxClipRect;
		}		
	}
	
	/// <summary>
	/// Method to instantiate and update the bar
	/// </summary>
	private void updateInfectedBar()
	{
		// Instance infected Bar
		if (!this.SetupBarInfectedSim.IsInstanceInfectedBar)
		{
			this.Stats.IsInfecting = true;
			
			Vector3 auxPosBar = new Vector3(this.transform.position.x,
				this.transform.position.y + 130.0f,
				this.transform.position.z - 5.0f);
			this.SetupBarInfectedSim.InstanceIB = Instantiate(this.SetupBarInfectedSim.InfectedBar, auxPosBar, Quaternion.identity) as GameObject;
			this.SetupBarInfectedSim.InstaceClippedIB = this.SetupBarInfectedSim.InstanceIB.transform.FindChild("InfectedBar").transform.GetComponent<tk2dClippedSprite>();
			this.SetupBarInfectedSim.IsInstanceInfectedBar = true;
			
			// Time to wait to updateClipped 7 parts
			float auxParts = 7.0f;
			this.SetupBarInfectedSim.PercentInfected = 0.0f;
			this.SetupBarInfectedSim.UpdateTimeClippedX = 0.0f;
			this.SetupBarInfectedSim.DeltaTimeUpdateClippedX = this.SetupBarInfectedSim.DeltaTimePInfected / auxParts;			
			this.SetupBarInfectedSim.AmountInfectedClippedTime = this.SetupBarInfectedSim.AmountInfectedClipped / auxParts;
			
		}
		
		// Increase infectedBar
		if (this.SetupBarInfectedSim.IsInstanceInfectedBar)
		{
			this.SetupBarInfectedSim.UpdateTimeClippedX +=Time.deltaTime;
			if (this.SetupBarInfectedSim.UpdateTimeClippedX >= this.SetupBarInfectedSim.DeltaTimeUpdateClippedX)
			{	this.SetupBarInfectedSim.UpdateTimeClippedX = 0.0f;
				float auxAmount = this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.x + this.SetupBarInfectedSim.AmountInfectedClippedTime;
				
				Rect auxClipRect = new Rect(auxAmount,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.y,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.width,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.height);
				this.SetupBarInfectedSim.InstaceClippedIB.ClipRect = auxClipRect;				
				
			}
		}
		
		// Increase percent
		this.SetupBarInfectedSim.UpdateTimePInfected += Time.deltaTime;
		if (this.SetupBarInfectedSim.UpdateTimePInfected >= this.SetupBarInfectedSim.DeltaTimePInfected)
		{	this.SetupBarInfectedSim.UpdateTimePInfected = 0.0f;
			
			this.SetupBarInfectedSim.PercentInfected += this.SetupBarInfectedSim.AmountInfected;
			
			float auxAmount =  Mathf.Clamp(this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.x + this.SetupBarInfectedSim.AmountInfectedClippedTime,-1.0f,0.0f);
				Rect auxClipRect = new Rect(auxAmount,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.y,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.width,
					this.SetupBarInfectedSim.InstaceClippedIB.ClipRect.height);
				this.SetupBarInfectedSim.InstaceClippedIB.ClipRect = auxClipRect;
			
			if (this.SetupBarInfectedSim.PercentInfected >= 100.0f)
			{
				this.DestroyInfectedBar();
				
				if (!this.Stats.Infected)
				{
					this.endIsInfected();
				}		
			}		
		}
	}
	/// <summary>
	/// Method to end infecting
	/// </summary>
	private void endIsInfected()
	{

        // Affect love cat
        // If there is love cat, infected by cat
        if ((this.SetupLoveCatSim.IsLoveCat) && (this.SetupLoveCatSim.IsAffectAllZombies))
        {
            this.SetupLoveCatSim.IsAffectedLoveCat = true;
        }	

		this.RemoveIconSim();
        // Change state to hunt
		this.ChangeState(State.Z_HUNT);
		this.Stats.Infected = true;		
		bool lastInfected = false;
		GameManager.Instance.NumberInfected +=1;
        GameManager.Instance.TotalNumberInfected += 1;
		lastInfected = GameManager.Instance.IsLastInfected();
		// Instance Coins
		float auxXPos = this.transform.position.x - 30.0f;
		for (int i= 0; i<this.SetupCoinsSim.NumInstanceCoins; i++)
		{						
			Vector3 auxPosCoin = new Vector3(auxXPos,
				this.transform.position.y + 10.0f, this.transform.position.z - 5.0f);
			Coin auxCoin = (Instantiate(this.SetupCoinsSim.PrefabCoin,auxPosCoin,Quaternion.identity) as GameObject).GetComponent<Coin>();
			auxCoin.GetComponent<ConfigZ>().Number_Floor = Stats.Number_Floor;
			auxXPos += 30.0f;
			auxCoin.directionSpawn = (i-1);						
			if (lastInfected)
			{
				auxCoin.coinFromLastSim = true;
			}					
		}
		this.Stats.IsInfecting = false;

        

	}
	
	public void DestroyInfectedBar()
	{
		this.removeArrowLoveCat();
        if (this.SetupBarInfectedSim.InstanceIB != null)
        {
            Destroy(this.SetupBarInfectedSim.InstanceIB);
            this.SetupBarInfectedSim.PercentInfected = 0;
            this.SetupBarInfectedSim.IsInstanceInfectedBar = false;
        }
		
	}
	#endregion InfectedBar
	
	#region Death
	
	public void SetDeath()
	{
		this.Stats.SimZombie = null;
		this.Stats.SimChase = null;
		this.Stats.SimHunted = null;
		if (this.Settings.HelperCircleWarn != null)
		{
			this.Settings.HelperCircleWarn.DisableCircle();

		}
		
	}
	
	/// <summary>
	/// Method to check if liked sim in this sim are death
	/// </summary>
	private void checkLinkSims()
	{	
		if (this.Stats.SimZombie != null)
		{
			if (this.Stats.SimZombie.Stats.CurrentState == SimController.State.Z_DEATH)
			{
				this.Stats.SimZombie = null;
			}
		}
		
		if (this.Stats.SimChase != null)
		{
			if (this.Stats.SimChase.Stats.CurrentState == SimController.State.Z_DEATH)
			{
				this.Stats.SimChase = null;
			}
		}
		
		if (this.Stats.SimHunted != null)
		{
			if (this.Stats.SimHunted.Stats.CurrentState == SimController.State.Z_DEATH)
			{
				this.Stats.SimHunted = null;
			}
		}
	}
	#endregion Death	
	
	#region LoveCat	
	
	public void SetLoveCat(Vector2 position,float radio)
	{

		this.SetupLoveCatSim.IsLoveCat = true;
		this.SetupLoveCatSim.PositionLoveCat = position;
		this.SetupLoveCatSim.RadiusLoveCat = radio;
		this.instArrowLoveCat();
		
	}
	
	private void instArrowLoveCat()
	{
		if (this.SetupLoveCatSim.InstArrowLoveCat == null)
		{
			float offset = 0.15f;
			Vector3 auxPos = new Vector3(this.transform.position.x, this.transform.position.y, this.transform.position.z + offset);
			this.SetupLoveCatSim.InstArrowLoveCat = GameObject.Instantiate(this.SetupLoveCatSim.PrefabArrowLoveCat,auxPos, Quaternion.identity) as GameObject ;
			
			if (this.SetupLoveCatSim.PositionLoveCat.x < this.transform.position.x)
			{
				setDirArrowLoveCat(DirectionType.LEFT);
			}else
			{
				setDirArrowLoveCat(DirectionType.RIGHT);				
			}
					
		}		
	}
	
	private void setDirArrowLoveCat(SimController.DirectionType dir)
	{
		if (this.SetupLoveCatSim.InstArrowLoveCat != null)
		{
			tk2dSprite spriteArrow = this.SetupLoveCatSim.InstArrowLoveCat.transform.GetComponent<tk2dSprite>();
			if (spriteArrow != null)
			{
				if (dir == DirectionType.LEFT)
				{
					spriteArrow.FlipX = true;	
					
				}else
				{
					spriteArrow.FlipX = false;					
				}				
			}
		}
	}
	
	private void updateArrowLoveCat()
	{
		if (this.SetupLoveCatSim.InstArrowLoveCat != null)
		{
			// Update position arrow
			float offset = 0.15f;
			Vector3 auxPos = new Vector3(this.transform.position.x,
				this.transform.position.y,
				this.transform.position.z + offset);
				this.SetupLoveCatSim.InstArrowLoveCat.transform.position = auxPos;
			
			// Update direction of arrow love cat
			if (!this.Stats.Idle)
			{
				if (this.SetupLoveCatSim.PositionLoveCat.x < this.transform.position.x)
				{
					setDirArrowLoveCat(DirectionType.LEFT);
				}else
				{
					setDirArrowLoveCat(DirectionType.RIGHT);
					
				}
			}			
			
			// If reach range, remove
			// If sim is moving check range
			if ((!this.Stats.Idle) && (!this.Stats.Infected))
			{
				Vector2 vectPosLoveCat = new Vector2(this.SetupLoveCatSim.PositionLoveCat.x,
					this.SetupLoveCatSim.PositionLoveCat.y);
				Vector2 vectPos = new Vector2(this.transform.position.x,
					this.transform.position.y);
				
				float distance = Vector2.Distance(vectPosLoveCat, vectPos);
				if (distance > this.SetupLoveCatSim.RadiusLoveCat)
				{
					this.removeArrowLoveCat();
				}
			}
			
			// Infected yet
			if (this.Stats.Infected)
			{
				this.removeArrowLoveCat();
			}
		}
	}
	
	private void removeArrowLoveCat()
	{
		if (this.SetupLoveCatSim.InstArrowLoveCat != null)
		{
			GameObject.Destroy(this.SetupLoveCatSim.InstArrowLoveCat);			
		}
        
	}

    /// <summary>
    /// Erase effect of love cat in sim controller
    /// </summary>
    public void removeEffectLoveCat()
    {
        this.removeArrowLoveCat();
        this.SetupLoveCatSim.IsLoveCat = false;
    }

    #endregion LoveCat

    void OnDrawGizmos()
	{
		Gizmos.color = Color.red;
		
		if (this.Stats.TSkin == Skin.POLY_SKIN)
		{
			Gizmos.DrawWireSphere(this.transform.position, this.SetupPolyEnforce.DistanceSearch);
		}else
		{
			Gizmos.DrawWireSphere(this.transform.position, this.SetupAlertSim.DistanceSearchWarn);
		}
			if (this.Stats.CurrentState == State.INFECT_ZOMBIE)
		{
			Gizmos.color = Color.blue;
			Gizmos.DrawWireSphere(this.transform.position, this.SetupAlertSim.DistanceSearchWarn);
		}
		
		if (SetupInitSim.IntialState == State.C_PATROL_DISTANCE)
		{
			Gizmos.color = Color.yellow;
			//Gizmos.DrawLine(this.transform.position,this.transform.position+Vector3.right*this.Settings.DistancePatrol);
			Gizmos.DrawLine(this.transform.position,this.transform.position + Vector3.right * this.SetupSimPatrol.DistancePatrol);
			Gizmos.DrawSphere(this.transform.position + Vector3.right * this.SetupSimPatrol.DistancePatrol, 5);
		}		
	}
	
}
