using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public abstract class SimStates:FSMState
{
	/// <summary>
	/// Character controller for player
	/// </summary>
	protected CharacterController charController;

	/// <summary>
	/// Sim controller for player
	/// </summary>
	protected SimController simController;	
	protected SkeletonAnimation sklAnim;
	
	/// <summary>
	/// Current Animation
	/// </summary>
	protected string currentAnimation;
	
	/// <summary>
	/// Direction of Sim
	/// </summary>
	protected Vector3 directionSim;
	
	/// <summary>
	/// Target Point to move
	/// </summary>
	protected Vector3 targetPoint; 
	
	/// <summary>
	/// Laste Point to move
	/// </summary>
	protected Vector3 lastTargetPoint;
	
	public SimStates(GameObject entity, int id): base(entity, id)
    {
		this.charController = entity.GetComponent<CharacterController>();
		this.simController = entity.GetComponent<SimController>();
		this.sklAnim = entity.GetComponent<SkeletonAnimation>();
		
		this.setDirection();
		
    }
	public override void UpdateState()
    {		
	}
	
	public void setDirection()
	{
		if (this.simController.Stats.Direction == SimController.DirectionType.RIGHT)
		{			
			this.charController.transform.rotation = new Quaternion(0.0f,180.0f,0.0f,0.0f);
		}
		if (this.simController.Stats.Direction == SimController.DirectionType.LEFT)
		{			
			this.charController.transform.rotation = new Quaternion(0.0f,0.0f,0.0f,0.0f);
		}		
	}	
	
	public void setAnimation(SimController.State cState)
	{
		if (this.sklAnim != null)
		{
			string nameAnim = "";
			switch(cState)
			{
				case SimController.State.C_IDLE:
					
					switch (this.simController.Stats.TypeIdle)
					{
						case SimController.TIDLE.IDLE_NORMAL:
					
							if (this.simController.Stats.HasShield)
							{
								nameAnim = "idle_shield";
							}else
							{
								
								nameAnim = "idle";
							}					
							
						break;
						case SimController.TIDLE.IDLE_TALKING:
							nameAnim = "talk";
						break;
					
						case SimController.TIDLE.IDLE_SIGNAL:
							nameAnim = "idle2";
						break;
					}				
				break;
				
				case SimController.State.C_PATROL_DISTANCE:
					
					if (this.simController.Stats.HasShield)
					{
						if (this.simController.Stats.TSkin != SimController.Skin.LEADER_SKIN)
						{
							nameAnim = "walk_shield";
							
						}else
						{
							nameAnim = "walk";
						}
					
						
					}else
					{
						nameAnim = "walk";
					}
					
				
				break;
				
							
				case SimController.State.C_WARN:
					
					if (this.simController.Stats.HasShield)
					{
						if (this.simController.Stats.TSkin != SimController.Skin.LEADER_SKIN)
						{
							nameAnim = "run_screaming_shield";
							
						}else
						{
							nameAnim = "run_screaming";
						}
					}else
					{
						nameAnim = "run_screaming";
					}
					
				break;
				
				
				case SimController.State.C_ALERT:
					
					if (this.simController.Stats.HasShield)
					{
						if (this.simController.Stats.TSkin != SimController.Skin.LEADER_SKIN)
						{
							nameAnim = "aimWalk_shield";
							
						}else
						{
							nameAnim = "aimWalk";
						}
					}else
					{
						nameAnim = "aimWalk";
					}
				break;
				
				// Shoot				
				case SimController.State.C_SHOOT:
					
					if (this.simController.Stats.HasShield)
					{
					
						if (this.simController.Stats.TSkin != SimController.Skin.LEADER_SKIN)
						{
							nameAnim = "aim_shield";
							
						}else
						{
							nameAnim = "aim";
						}

					}else
					{
						nameAnim = "aim";
					}
				
				break;
								
				
				case SimController.State.INFECT_CAT:
					
					nameAnim = "infecting_cat";
				
				break;
				
				case SimController.State.INFECT_ZOMBIE:
					
					nameAnim = "infecting_zombie";
				
				break;
				case SimController.State.Z_IDLE:
					
					nameAnim = "z_idle";
				
				break;
				
				case SimController.State.Z_HUNT:
					
					nameAnim = "z_hunt";
				
				break;
				
				case SimController.State.Z_CHASE:
					
					nameAnim = "z_hunt";
				
				break;
				
				case SimController.State.Z_ATTACK:
					
					nameAnim = "z_attack";
				
				break;
				case SimController.State.Z_HURT:
					
					nameAnim = "z_hurt";
				
				break;
				case SimController.State.Z_DEATH:
					
					nameAnim = "z_idle";
				
				break;
				case SimController.State.SIM_FALL:
					
					if (this.simController.Stats.Infected)
					{
						nameAnim = "z_hurt";
					}else
					{					
						nameAnim = "run_screaming";
					
					}
				
				break;
				
				case SimController.State.STUNNED:
					
					if (this.simController.Stats.Infected)
					{
						nameAnim = "z_iddle";
					}else
					{
						nameAnim = "stunned";
					}
				
				break;	
				
				
				case SimController.State.INACTIVE:
					
					if (this.simController.Stats.Infected)
					{
						nameAnim = "z_iddle";
					}else
					{
						if (this.simController.Stats.HasShield)
						{
							nameAnim = "idle_shield";
						}else
						{
							nameAnim = "idle";
						}
						
					}
				
				break;			
				
			}	
			this.sklAnim.animationName = nameAnim;
		}
		
	}
	
	protected void moveSim()
	{
		this.directionSim.Normalize();
		this.charController.Move(this.directionSim * this.simController.Stats.SpeedMovement * Time.deltaTime);		
	}
	
		
	#region Barrier	

	
	// good method!!
	protected void setChangeTargetToBar()
	{
		// If sim is crossing bar
		if (this.simController.SetupBarrierSim.IsSimCrossBar)
		{
			
			// If has to setup first target
			if (this.simController.SetupBarrierSim.IsHasSetFirstTargetBar)
			{
				// Hold current target
				this.lastTargetPoint = this.targetPoint;
				// Setup first target
				this.targetPoint = this.simController.SetupBarrierSim.FirstTargetBar;
				this.simController.SetupBarrierSim.CurrentTargetBar = this.targetPoint;
				this.directionSim = new Vector3(this.targetPoint.x - this.charController.transform.position.x,
				this.targetPoint.y - this.charController.transform.position.y,0.0f);
				this.simController.SetupBarrierSim.IsHasSetFirstTargetBar = false;
			}
			
			// If has to setup first target
			if (this.simController.SetupBarrierSim.IsHasSetSecondTargetBar)
			{		
				// Setup first target
				this.targetPoint = this.simController.SetupBarrierSim.SecondTargetBar;
				this.simController.SetupBarrierSim.CurrentTargetBar = this.targetPoint;
				this.directionSim = new Vector3(this.targetPoint.x - this.charController.transform.position.x,
				this.targetPoint.y - this.charController.transform.position.y,0.0f);
				this.simController.SetupBarrierSim.IsHasSetSecondTargetBar = false;
				

			}
		}
	}
	
	/// <summary>
	/// Setup currend direction with current target
	/// </summary>
	protected void setDirectionCurrentTargetBar()
	{
		this.directionSim = new Vector3(this.simController.SetupBarrierSim.CurrentTargetBar.x - this.charController.transform.position.x,
				this.simController.SetupBarrierSim.CurrentTargetBar.y - this.charController.transform.position.y,0.0f);
	}
	
	#endregion Barrier
		
}