﻿using UnityEngine;
using System.Collections;

/// <summary>
/// Alert state
/// </summary>
public class C_AlertState : SimStates
{
	private CharacterController simChaseController;
	private CharacterController catChaseController;
	private float zOffset = 0.0f;	
	// Time to waint until change alert to shoot
	private float timeUpdateAlert = 0.0f;
	public C_AlertState(GameObject entity, int id) : base(entity, id){}
	public override void Enter()
	{
		this.timeUpdateAlert = 0.0f;
		this.simController.Stats.Idle = false;
		
		if (this.simController.Stats.SimZombie != null)
		{
			this.simChaseController = this.simController.Stats.SimZombie.transform.GetComponent<CharacterController>();
			this.setTarget();
			this.simController.Stats.SpeedMovement =  this.simController.Settings.C_Warn_Speed;
			this.setAnimation(SimController.State.C_ALERT);
			
		}else if (this.simController.Stats.CatChasePoly != null)
		{			
			this.catChaseController = this.simController.Stats.CatChasePoly.transform.GetComponent<CharacterController>();
			this.setTarget();
			this.simController.Stats.SpeedMovement =  this.simController.Settings.C_Warn_Speed;
			this.setAnimation(SimController.State.C_ALERT);
			
		}else
		{
			this.simController.Stats.SpeedMovement =  this.simController.Settings.Idle_Speed;
			this.setAnimation(SimController.State.C_SHOOT);
		}
		
		if (this.simController.SetupBarrierSim.IsSimCrossBar)
		{
			// Update target
			this.setDirectionCurrentTargetBar();
			this.setChangeTargetToBar();			
		}
		
		this.simController.Stats.SpeedMovement =  this.simController.Settings.C_Warn_Speed;
		
	}
	
	private void setTarget()
	{
		
		if (this.simController.Stats.CatChasePoly != null)
		{
			if (this.simController.transform.position.x <= this.simController.Stats.CatChasePoly.position.x)
			{
				this.targetPoint = new Vector3(this.simController.Stats.CatChasePoly.position.x - this.catChaseController.radius,
				                               this.simController.Stats.CatChasePoly.position.y,
				                               this.simController.Stats.CatChasePoly.position.z);
				this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
				
			}else if (this.simController.transform.position.x > this.simController.Stats.CatChasePoly.position.x)
			{	
				this.targetPoint = new Vector3(this.simController.Stats.CatChasePoly.position.x + this.catChaseController.radius,
				                               this.simController.Stats.CatChasePoly.position.y,
				                               this.simController.Stats.CatChasePoly.position.z);
				this.simController.Stats.Direction = SimController.DirectionType.LEFT;
			}
		}else
		{
			if (this.simController.transform.position.x <= this.simController.Stats.SimZombie.transform.position.x)
			{
				this.targetPoint = new Vector3(this.simController.Stats.SimZombie.transform.position.x - this.simChaseController.radius,
				                               this.simController.Stats.SimZombie.transform.position.y,
				                               this.simController.Stats.SimZombie.transform.position.z);
				this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
				
			}else if (this.simController.transform.position.x > this.simController.Stats.SimZombie.transform.position.x)
			{
				
				this.targetPoint = new Vector3(this.simController.Stats.SimZombie.transform.position.x + this.simChaseController.radius,
				                               this.simController.Stats.SimZombie.transform.position.y,
				                               this.simController.Stats.SimZombie.transform.position.z);
				this.simController.Stats.Direction = SimController.DirectionType.LEFT;
			}
		}
		this.setDirection();
		
		
		this.directionSim = new Vector3(this.targetPoint.x - this.charController.transform.position.x,
		                                this.targetPoint.y - (this.charController.transform.position.y - this.charController.radius),0.0f);	
		
		if (this.targetPoint.y < this.charController.transform.position.y)
		{
			this.zOffset = -0.5f;
			
		}else if (this.targetPoint.y > this.charController.transform.position.y)
		{
			this.zOffset = -0.5f;
		}
	}
	
	public override void UpdateState()
	{
		base.UpdateState();
		
		
		if (this.simController.SetupBarrierSim.IsSimCrossBar)
		{
			this.setChangeTargetToBar();
			// Check target to bar
			this.moveSim();			
		}else
		{
			if (this.simController.Stats.SimZombie != null)
			{
				
				if (!this.simController.Stats.SimZombie.Stats.Idle)
				{
					this.setTarget();
				}
				
				// Moving
				if (!this.simController.Stats.SimZombie.Stats.Idle)
				{
					
					// Wait time to shoot
					this.timeUpdateAlert += Time.deltaTime;
					if (this.timeUpdateAlert >= this.simController.SetupPolyEnforce.TimeToWaitShoot)
					{
						this.timeUpdateAlert = 0.0f;
						this.stopSim();	
						
					}else
					{
						this.moveSimAlert();	
						
					}
				}else
				{
					// If zombie get away more to distance to search, move, if is inside range
					// stop and change to shoot state
					Vector3 auxCharVector = new Vector3(this.charController.transform.position.x + this.charController.radius,
					                                    this.charController.transform.position.y,
					                                    this.charController.transform.position.z);
					
					float auxDistanceToZombie = Vector3.Distance(auxCharVector,this.targetPoint);
					if (auxDistanceToZombie > this.simController.SetupPolyEnforce.DistanceSearch)
					{
						this.moveSimAlert();	
						
					}else
					{
						
						this.stopSim();
					}
					
				}			
			}else if (this.simController.Stats.CatChasePoly != null)
			{
				// If zombie get away more to distance to search, move, if is inside range
				// stop and change to shoot state
				Vector3 auxCharVector = new Vector3(this.charController.transform.position.x + this.charController.radius,
				                                    this.charController.transform.position.y,
				                                    this.charController.transform.position.z);
				
				float auxDistanceToCat = Vector3.Distance(auxCharVector,this.targetPoint);
				if (auxDistanceToCat > this.simController.SetupPolyEnforce.DistanceSearch)
				{
					this.moveSimAlert();	
					
				}else
				{					
					this.stopSim();
				}
				
				
			}else
			{
				this.setAnimation(SimController.State.C_SHOOT);
				
				// Wait in state alert until change to init state again
				this.timeUpdateAlert += Time.deltaTime;				
				if (this.timeUpdateAlert >= this.simController.SetupPolyEnforce.TimeInAlertState)
				{
					//this.simController.ChangeState(SimController.State.C_ALERT);
					this.simController.ChangeState(SimController.State.C_IDLE);
					//this.simController.ChangeState(this.simController.SetupInitSim.InitState);	
					
				}
			}
		}
	}
	
	private void moveSimAlert()
	{
		if (this.charController.transform.position.x < this.targetPoint.x)
		{
			this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
			this.setDirection();
			
		}else
		{
			this.simController.Stats.Direction = SimController.DirectionType.LEFT;
			this.setDirection();
		}
		
		this.moveSim();
		
	}
	
	/// <summary>
	/// Stop sim and change to c_shoot
	/// </summary>
	private void stopSim()
	{
		this.charController.transform.position = new Vector3(this.charController.transform.position.x,
		                                                     this.charController.transform.position.y, this.targetPoint.z + this.zOffset);
		this.simController.ChangeState(SimController.State.C_SHOOT);
		
	}
	
	public override void Exit()
	{
	}
	
}
