﻿using UnityEngine;
using System.Collections;


/// <summary>
/// Patrol for civils for distance
/// </summary>
public class C_PatrollDistanceState : SimStates
{
	
	bool wait = false;
	private float timeToWait = 0.0f;
	private float deltaToWait = 0.0f;
	private Vector2 originPoint = Vector2.zero;
	private float totalDistance = 0.0f;
	private float currentDistance = 0.0f;
	private bool onlyChangeDirection = false;
	
	public C_PatrollDistanceState(GameObject entity, int id) : base(entity, id){}
	
	
	public override void Enter()
	{
		this.onlyChangeDirection = false;
		this.wait = false;
		if ( this.simController.SetupSimPatrol.DistancePatrol == 0.0f)
		{
			if (this.simController.SetupSimPatrol.TimeWaitPatrol <= 0.0f)
			{
				this.simController.ChangeState(SimController.State.C_IDLE);
			}
			else
			{
				this.setAnimation(SimController.State.C_IDLE);
				this.onlyChangeDirection = true;
				this.setDirection();               
			}		
		}		
		
		this.deltaToWait = this.simController.SetupSimPatrol.TimeWaitPatrol;
		if (!this.onlyChangeDirection)
		{
			if (this.simController.SetupSimPatrol.DistancePatrol > 0)
			{
				this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
				this.directionSim = new Vector3(1.0f, 0.0f, 0.0f);
				
			}
			else
			{
				this.simController.Stats.Direction = SimController.DirectionType.LEFT;
				this.directionSim = new Vector3(-1.0f, 0.0f, 0.0f);
			}
			this.setDirection();
			
			this.setAnimation(SimController.State.C_PATROL_DISTANCE);
			this.simController.Stats.Idle = false;
			this.originPoint = new Vector2(this.simController.transform.position.x,
			                               this.simController.transform.position.y);
			this.totalDistance = Mathf.Abs(this.simController.SetupSimPatrol.DistancePatrol);
			this.currentDistance = 0.0f;
			
			
			if (this.simController.SetupBarrierSim.IsSimCrossBar)
			{
				// Update target
				this.setDirectionCurrentTargetBar();
				this.setChangeTargetToBar();
			}
			
			this.simController.Stats.SpeedMovement = this.simController.Settings.C_Patrol_Speed;
		}
		else
		{
			this.simController.Stats.SpeedMovement = this.simController.Settings.Idle_Speed;
		}
	}
	
	public override void UpdateState()
	{
		base.UpdateState();
		
		// Only change direction
		if (this.onlyChangeDirection)
		{
			this.timeToWait += Time.deltaTime;
			if (this.timeToWait >= this.deltaToWait)
			{
				this.timeToWait = 0.0f;
				
				// Change direction
				if (this.simController.Stats.Direction == SimController.DirectionType.LEFT)
				{
					this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
					this.directionSim = new Vector3(1.0f, 0.0f, 0.0f);
					
				}
				else if (this.simController.Stats.Direction == SimController.DirectionType.RIGHT)
				{
					this.simController.Stats.Direction = SimController.DirectionType.LEFT;
					this.directionSim = new Vector3(-1.0f, 0.0f, 0.0f);
					
				}
				this.setDirection();
			}		
		}
		else
		{
			if (!wait)
			{            
				if (this.simController.SetupBarrierSim.IsSimCrossBar)
				{
					this.setChangeTargetToBar();
					// Check target to bar
					this.moveSim();
				}
				else
				{
					if (this.simController.SetupWallSim.IsSetupWall)
					{
						this.totalDistance = Mathf.Abs(this.simController.SetupSimPatrol.DistancePatrol - this.currentDistance);
						this.wait = true;
						this.setAnimation(SimController.State.C_IDLE);
						this.simController.Stats.Idle = true;
						this.simController.SetupWallSim.IsSetupWall = false;
					}
					else
					{
						// Move
						this.simController.Stats.Idle = false;
						Vector2 auxCurrentPosition = new Vector2(this.simController.transform.position.x,
						                                         this.simController.transform.position.y);
						this.currentDistance = Vector2.Distance(auxCurrentPosition, this.originPoint);
						if (this.currentDistance <= this.totalDistance)
						{
							this.moveSim();
						}
						else
						{
							this.totalDistance = Mathf.Abs(this.simController.SetupSimPatrol.DistancePatrol);
							this.currentDistance = 0.0f;
							this.wait = true;
							this.setAnimation(SimController.State.C_IDLE);
							this.simController.Stats.Idle = true;
						}
					}
				}
				
			}else
			{			
				this.timeToWait += Time.deltaTime;
				if (this.timeToWait>= this.deltaToWait)
				{
					this.timeToWait = 0.0f;
					
					// Change direction
					if (this.simController.Stats.Direction == SimController.DirectionType.LEFT)
					{
						this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
						this.directionSim = new Vector3(1.0f,0.0f,0.0f);
						
						
					}else if (this.simController.Stats.Direction == SimController.DirectionType.RIGHT)
					{
						this.simController.Stats.Direction = SimController.DirectionType.LEFT;
						this.directionSim = new Vector3(-1.0f,0.0f,0.0f);
						
					}
					this.originPoint = new Vector2(this.simController.transform.position.x,
					                               this.simController.transform.position.y);
					this.currentDistance = 0.0f;
					this.setDirection();
					this.setAnimation(SimController.State.C_PATROL_DISTANCE);				
					this.wait = false;
					
				}		
				
			}
		}
	}
	public override void Exit(){}	
	
}

