﻿using UnityEngine;
using System.Collections;

/// <summary>
/// State for zombie chasing a civil
/// </summary>
public class Z_ChaseState : SimStates
{
	
	float radiusSimChase = 0.0f;	
	
	public Z_ChaseState(GameObject entity, int id) : base(entity, id)
	{
	}
	
	public override void Enter()
	{
		this.setAnimation(SimController.State.Z_HUNT);
		this.simController.Stats.Idle = false;		
		if (this.simController.Stats.SimChase != null)
		{
			CharacterController simChaseChar = this.simController.Stats.SimChase.transform.GetComponent<CharacterController>();
			this.radiusSimChase = simChaseChar.radius + (simChaseChar.radius/2);			
			
			if (this.simController.SetupBarrierSim.IsSimCrossBar)
			{
				// Update target
				this.setDirectionCurrentTargetBar();
				this.setChangeTargetToBar();
				
			}else
			{
				// Setup target to chase
				this.setTarget();	
			}
		}
		this.simController.Stats.SpeedMovement =  this.simController.Settings.Z_Chase_Speed; 
		
		this.checkChangeDirection();
	}
	
	private void setTarget()
	{
		if (this.simController.Stats.SimChase != null)
		{
			if (this.charController.transform.position.x <= this.simController.Stats.SimChase.transform.position.x)
			{
				this.targetPoint = new Vector3(this.simController.Stats.SimChase.transform.position.x - this.radiusSimChase,
				                               this.simController.Stats.SimChase.transform.position.y,
				                               this.simController.Stats.SimChase.transform.position.z);	
				
			}else if (this.charController.transform.position.x > this.simController.Stats.SimChase.transform.position.x)
			{
				
				this.targetPoint = new Vector3(this.simController.Stats.SimChase.transform.position.x + this.radiusSimChase,
				                               this.simController.Stats.SimChase.transform.position.y,
				                               this.simController.Stats.SimChase.transform.position.z);
			}
			
			
			this.directionSim = new Vector3(this.targetPoint.x - this.charController.transform.position.x,
			                                this.targetPoint.y - (this.charController.transform.position.y - this.charController.radius),0.0f);				
		}
		
	}
	public override void UpdateState()
	{
		base.UpdateState();
		
		// If has target bar
		if (this.simController.SetupBarrierSim.IsSimCrossBar)
		{
			// Check target to bar
			this.setChangeTargetToBar();
			// Move sim
			this.moveSim();
			
		}else
		{
			this.setTarget();
			if (this.simController.Stats.SimChase != null)
			{
				//Bea esta cansadita y no puede arreglar bugs IPD
				int dir = 1;
				if( simController.Stats.Direction == SimController.DirectionType.RIGHT )
				{
					dir = -1;
				}
				Vector3 auxCharVector = new Vector3(this.charController.transform.position.x + dir*this.charController.radius,
				                                    this.charController.transform.position.y,
				                                    this.charController.transform.position.z);
				
				float auxDistance = Vector2.Distance(auxCharVector,this.targetPoint);
				float mathAbsDistance = Mathf.Abs(auxDistance - 50.0f);
				bool errorOffset = false;
				if (mathAbsDistance <= 1.0f)
				{
					errorOffset = true;
				}
				if ((auxDistance > 50.0f) && (!errorOffset))
				{					
					if (this.charController.transform.position.x < this.simController.Stats.SimChase.transform.position.x )
					{
						this.simController.Stats.Direction = SimController.DirectionType.RIGHT;				
						
					}else if (this.charController.transform.position.x > this.simController.Stats.SimChase.transform.position.x )
					{
						this.simController.Stats.Direction = SimController.DirectionType.LEFT;
					}
					this.setDirection();
					this.moveSim();
					
				}else
				{
					float zOffset = 0.0f;
					if (this.charController.transform.position.y < this.simController.Stats.SimChase.transform.position.y)
					{
						zOffset = -0.5f;
						
					}else 
					{
						zOffset = 0.5f;
					}
					if (this.charController.transform.position.x  < this.simController.Stats.SimChase.transform.position.x )
					{
						
						this.charController.transform.position = new Vector3(this.simController.Stats.SimChase.transform.position.x  - this.radiusSimChase,
						                                                     this.simController.Stats.SimChase.transform.position.y, (this.simController.Stats.SimChase.transform.position.z + zOffset));
						this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
						
					}else if (this.charController.transform.position.x > this.simController.Stats.SimChase.transform.position.x )
					{
						
						this.charController.transform.position = new Vector3(this.simController.Stats.SimChase.transform.position.x  + this.radiusSimChase,
						                                                     this.simController.Stats.SimChase.transform.position.y, (this.simController.Stats.SimChase.transform.position.z + zOffset));
						this.simController.Stats.Direction = SimController.DirectionType.LEFT;
						
						
					}
					this.setDirection();					
					this.simController.ChangeState(SimController.State.Z_ATTACK);					
					this.simController.Stats.SimChase.ChangeState(SimController.State.INFECT_ZOMBIE);
					
				}
			}else
			{
				this.simController.Stats.SimChase = null;
				this.simController.ChangeState(SimController.State.Z_HUNT);		
				
			}
		}
	}
	
	private void checkChangeDirection()
	{
		if ((this.simController.SetupLoveCatSim.IsLoveCat) && 
		    (!this.simController.SetupLoveCatSim.SetupLoveCat) && (this.simController.SetupLoveCatSim.IsAffectedLoveCat))
		{
			
			if (this.simController.transform.position.x <this.simController.SetupLoveCatSim.PositionLoveCat.x)
			{
				// Turn right
				this.simController.Stats.Direction = SimController.DirectionType.RIGHT;
				
			}else
			{
				// Turn left
				this.simController.Stats.Direction = SimController.DirectionType.LEFT;
				
			}
			this.setDirection();
			this.simController.SetupLoveCatSim.SetupLoveCat = true;
			this.simController.Stats.SimChase = null;
			this.simController.ChangeState(SimController.State.Z_HUNT);
			
		}		
	}	
	
	public override void Exit()
	{
	}
}
