
using UnityEngine;
/// <summary>
/// State of a Finite State Machine. Only one state can be updated to the time.
/// </summary>
public abstract class FSMState
{
	
   	protected int id;
    public int ID { get { return id; } }
 	public GameObject 	entity;
	
	
	public FSMState(GameObject entity, int id)
	{
		this.entity = entity;
		this.id		= id;
	}
    /// <summary>
    /// Update the current state.
    /// </summary>
    public abstract void UpdateState();
	
	/// <summary>
    /// Late Update the current state.
    /// </summary
   //public abstract void LateUpdateState();
	
	/// <summary>
    /// When a state change FSMSystem calls this method from the current state before update the current state.
    /// </summary>
	public abstract void Enter();
	
	/// <summary>
    /// When a state changes, FSMSystem calls this method from the last state before change to new state.
    /// </summary>
	public abstract void Exit();
	
}//FSMState