using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
 
/// <summary>
/// Finite State Machine. This class controls all states of an entity.
/// </summary>
public class FSMSystem
{
    private List<FSMState> states;
 
    private 	int 			currentStateID;
    public 		int 			CurrentStateID { get { return currentStateID; } }
    private 	FSMState currentState;
    public 		FSMState CurrentState 		{ get { return currentState; } }
 
	private bool firstTime = true;
	
    public FSMSystem()
    {
        states = new List<FSMState>();
    }
  	/// <summary>
    /// Adds a new state.
    /// </summary>
	public void AddState(FSMState newState)
	{
		if (newState == null)
        {
	        Debug.LogError("FSM: Trying to add a null state");
        }
		else
		{
			if (states.Count == 0)
	        {
	            states.Add(newState);
	            currentState 		= newState;
	            currentStateID 	= newState.ID;
	        }
			else
			{
				foreach (FSMState state in states)
		        {
		            if ( state.ID == newState.ID )
		            {
		                Debug.LogWarning("FSM : Cannot add the state " + newState.ID +  " because has already been added");
		                return;
		            }//if
		        }//foreach
				states.Add(newState);
			}//else
		}
	}
	
  
    /// <summary>
    /// This method changes the state to other if is not already added.
    /// </summary>
    public void ChangeState(int newIdState)
    {
        if (currentState.ID != newIdState)
        {
	        currentStateID = newIdState;
	        foreach (FSMState state in states)
	        {
	            if (state.ID == currentStateID)
	            {
	                currentState.Exit();
	                currentState = state;
	 				currentState.Enter();
	                break;
	            }//if
	        }//foreach
		}//if
    }
 
 	 /// <summary>
    /// This method forces to change the state to other.
    /// </summary>
    public void ForceChangeState(int newIdState)
    {
        currentStateID = newIdState;
        foreach (FSMState state in states)
        {
            if (state.ID == currentStateID)
            {
                currentState.Exit();
                currentState = state;
 				currentState.Enter();
                break;
            }//if
        }//foreach
    }
	 /// <summary>
    /// Update current state.
    /// </summary>
	public void UpdateState()
	{
		if( firstTime )
		{
			firstTime = false;
			currentState.Enter();
		}
		currentState.UpdateState();
	}
}