﻿using UnityEngine;
using System.Collections;

public class Bullet : MonoBehaviour {
	
	
	public float Distance;
	public float Speed = 200.0f;
	/// <summary>
	/// Time to destroy bullet
	/// </summary>
	public float TimeToDestroyBullet = 500.0f;
	
	private float updateTimeDestroy;
	
	
	private Vector3 target;
	
	private bool initMovement = false;
	
	
	private Vector3 direction;
	
	private SimController targetSim;
	private Transform catTarget;
	private SimController currentSim;
	private CharacterController charController;

    /// <summary>
    /// Member to denote if this bullet can allow defeat cat
    /// </summary>
    private bool allowDefeatCat = false;
    public bool AllowDefeatCat
    {
        get { return this.allowDefeatCat; }
    }
	
	// Use this for initialization
	void Start () 
	{
		this.charController = this.transform.GetComponent<CharacterController>();
	}
	
	public void SetBullet(Transform targetCat, SimController cSim)
	{

		this.catTarget = targetCat;
		this.currentSim = cSim;
		if ((this.catTarget != null) && (this.currentSim != null))
		{
            if (currentSim.Stats.TypeOfficer == SimController.TOfficer.OFFICER_COMMON)
            {
                this.allowDefeatCat = false;
            }
            else if (currentSim.Stats.TypeOfficer == SimController.TOfficer.OFFICER_SPECIAL_ENFORCEMENT)
            {
                this.allowDefeatCat = true;
            }

            Transform pointCat = this.catTarget.FindChild("pointBullet");

            Vector2 vTarget = new Vector2(pointCat.position.x, pointCat.position.y);
            Vector2 vCurrent = new Vector2(this.transform.position.x, this.transform.position.y);		
			this.setDirRotationBullet(vCurrent,vTarget);								
			this.initMovement = true;
		}else
		{
			Destroy(this.gameObject);
			
		}
	}
	
	public void SetBullet(SimController target, SimController cSim)
	{
		this.targetSim = target;
		this.currentSim = cSim;

		if ((this.targetSim != null) && (this.currentSim != null))
		{
            if (currentSim.Stats.TypeOfficer == SimController.TOfficer.OFFICER_COMMON)
            {
                this.allowDefeatCat = false;
            }
            else if (currentSim.Stats.TypeOfficer == SimController.TOfficer.OFFICER_SPECIAL_ENFORCEMENT)
            {
                this.allowDefeatCat = true;
            }
            Vector2 vTarget = new Vector2(this.targetSim.transform.position.x, this.targetSim.transform.position.y);
            // Check if targetSim is on Infected by cat state
            if (this.targetSim.Stats.CurrentState == SimController.State.INFECT_CAT)
            {
                // Find point cat to set that direction
                CatController catControll = this.targetSim.GetComponentInChildren<CatController>();
               
                if (catControll != null)
                {
                    Transform pointCat = catControll.transform.FindChild("pointBullet");
                    if (pointCat != null)
                    {
                        vTarget = new Vector2(pointCat.position.x, pointCat.position.y);
                    }
                }
            }
            
			
			Vector2 vCurrent = new Vector2(this.currentSim.transform.position.x,this.currentSim.transform.position.y);			
			this.setDirRotationBullet(vCurrent,vTarget);
			
			this.initMovement = true;
		}else
		{
			Destroy(this.gameObject);
			
		}
	}
	
	private void setDirRotationBullet(Vector2 vCurrent, Vector2 vTarget)
	{
		float dY = vTarget.y - vCurrent.y;
		float dX = vTarget.x - vCurrent.x;
		
		float angleAtan = Mathf.Atan2(dY,dX);
		float angle = angleAtan * 180.0f / Mathf.PI;
		this.transform.rotation = Quaternion.AngleAxis(angle, Vector3.forward);
		this.direction = new Vector3(vTarget.x - vCurrent.x,
			vTarget.y - vCurrent.y,
			0.0f);
		this.direction.Normalize();		
		
		tk2dSprite spriteBullet = this.transform.GetComponent<tk2dSprite>();
		spriteBullet.FlipX = true;	
		
	}
	
	private void updateZ()
	{
		float auxUL = GameManager.Instance.SimSetupSettings.ListFloorLimit[0,1];
		float auxDistance = auxUL - this.transform.position.y;
		float auxZ = GameManager.Instance.SimSetupSettings.FarZSim - auxDistance;
		auxZ = Mathf.Clamp(auxZ,GameManager.Instance.SimSetupSettings.FarZ - GameManager.Instance.SimSetupSettings.WidthFloor, GameManager.Instance.SimSetupSettings.FarZ);
		this.transform.position = new Vector3(this.transform.position.x,
		                                      this.transform.position.y,
		                                      auxZ);
	}
	
	// Update is called once per frame
	void Update () 
	{
		// Move
		if (this.initMovement)
		{			
			this.charController.Move(this.direction * this.Speed * Time.deltaTime);			
			
			// Time to destroy bullet
			this.updateTimeDestroy += Time.deltaTime;
			if (this.updateTimeDestroy >= this.TimeToDestroyBullet)
			{
				Destroy(this.gameObject);
					
			}
		}
		
		// If current sim doesn't exist, destroy game object
		if (this.currentSim == null)
		{
			Destroy(this.gameObject);
		}
	}
	
	public void SetImpactCatBullet(Transform cat)
	{
		if (this.currentSim != null)
		{
            // Remove cat from floor and death cats
			GameManager.Instance.SettingCats.RemoveCatFloor(cat);
            GameManager.Instance.SettingCats.SetDeathCats();

			CatController aux_cat = cat.GetComponent<CatController>();
            if (aux_cat != null)
            {
                // Check if cat is love cat
                aux_cat.DestroyCat();
            }

			this.currentSim.Stats.CatChasePoly = null;	
			if ((this.currentSim.Stats.CurrentState != SimController.State.INFECT_ZOMBIE) &&
				(this.currentSim.Stats.CurrentState != SimController.State.INFECT_CAT))
			{
                this.currentSim.ChangeState(SimController.State.C_ALERT);
			}
            
			Destroy(this.gameObject);					
		}
	}
	
	public void SetImpactBullet(SimController simImpact)
	{	
		if (this.currentSim != null)
		{
			if ((simImpact.Stats.IDSim != this.currentSim.Stats.IDSim) && 
				(simImpact.Stats.IsInfecting) &&
				(simImpact.Stats.CurrentState == SimController.State.INFECT_CAT))
			{	
				simImpact.RemoveCat();
				simImpact.Stats.IsInfecting = false;
				simImpact.DestroyInfectedBar();
				this.currentSim.Stats.SimZombie = null;
				
				switch (simImpact.Stats.TSkin)
				{
					case SimController.Skin.BUSINESS_SKIN:
					case SimController.Skin.GEEK_SKIN:
					case SimController.Skin.MOBILE_SKIN:
					case SimController.Skin.MUSIC_SKIN:
					
						simImpact.ChangeState(SimController.State.C_WARN);	
					break;
					
					case SimController.Skin.POLY_SKIN:
						simImpact.ChangeState(SimController.State.C_ALERT);
					break;
					case SimController.Skin.NONE_SKIN:
						simImpact.ChangeState(SimController.State.C_IDLE);
					break;					
					
				}
				Destroy(this.gameObject);
			}else
			{
				// If not sim shoot and is infected
				if ((simImpact.Stats.IDSim != this.currentSim.Stats.IDSim) && (simImpact.Stats.Infected))
				{
					// Other sim is on attack
					if (simImpact.Stats.CurrentState == SimController.State.Z_ATTACK)			
					{
						//Stop attack
						simImpact.Stats.SimChase.DestroyInfectedBar();
						simImpact.Stats.SimChase.Stats.IsInfecting = false;						
						switch (simImpact.Stats.SimChase.Stats.TSkin)
						{
							case SimController.Skin.BUSINESS_SKIN:
							case SimController.Skin.GEEK_SKIN:
							case SimController.Skin.MOBILE_SKIN:
							case SimController.Skin.MUSIC_SKIN:

                                // Change other sim to warn estate
                                simImpact.Stats.SimChase.InitIconSim();
								simImpact.Stats.SimChase.ChangeState(SimController.State.C_WARN);	
							break;
							
							case SimController.Skin.POLY_SKIN:
                                // Change other sim to warn estate
                                bool auxInitIcon = true;
                                if (simImpact.Stats.SimChase.Stats.TypeOfficer == SimController.TOfficer.OFFICER_SPECIAL_ENFORCEMENT)
                                {
                                    if (simImpact.Stats.SimChase.Stats.IDSim == this.currentSim.Stats.IDSim)
                                    {
                                        auxInitIcon = false;
                                    }
                                }
                                if (auxInitIcon)
                                {
                                    simImpact.Stats.SimChase.InitIconSim();
                                }
								simImpact.Stats.SimChase.ChangeState(SimController.State.C_ALERT);
							break;
							case SimController.Skin.NONE_SKIN:
                                // Change other sim to warn estate
                                simImpact.Stats.SimChase.InitIconSim();
								simImpact.Stats.SimChase.ChangeState(SimController.State.C_IDLE);
							break;
							
						}				
					}
								
					// Change to hurt state
					simImpact.FSMSim.ForceChangeState((int)SimController.State.Z_HURT);				
									
					// Remove simZombie from sim shoot bullet
					this.currentSim.Stats.SimZombie = null;			
					// If this current sim is not attackig
					if ((this.currentSim.Stats.CurrentState != SimController.State.INFECT_ZOMBIE) &&
						(this.currentSim.Stats.CurrentState != SimController.State.INFECT_CAT))
					{
                        this.currentSim.ChangeState(SimController.State.C_ALERT);
					}
					
					Destroy(this.gameObject);					
				}
				
			}
			
		}
	}
	
	void OnDrawGizmos () 
	{
		Gizmos.color = Color.magenta;
		Vector3 fwd = transform.TransformDirection(Vector3.forward);
		Gizmos.DrawLine(this.transform.position,(fwd*this.Distance));
	}	
	
}
