using UnityEngine;
using System.Collections;

public class Brick : MonoBehaviour 
{
	public bool IsFall = false;
	public float Speed;
	public float InitAcceleration = 5.0f;
	public float RotationSpeed;
	private Vector3 positionObject;
	private Quaternion rotationObject;
	public GameObject InteractiveIcon;
	private bool endFall = false;
	private bool checkHit = true;
	
	public float YMax;
	public float ZPosition;
	
	public GameObject ParticleCloud;
	public float DelayParticle;
	private float timeUpdateParticle = 0.0f;
	private GameObject instPC;
	
	public GameObject ObjectToFall;

    /// <summary>
    /// Offset to setup raycast of object
    /// </summary>
    public float offsetSideRaycastLeft = 0.0f;
    public float offsetSideRaycastRigth = 0.0f;

	/// <summary>
	/// Sound for brick when click
	/// </summary>
	public AudioClip FallBrickSound;
	
	// Use this for initialization
	void Start () 
	{
		this.positionObject = this.ObjectToFall.transform.position;
		this.rotationObject = this.ObjectToFall.transform.rotation;
		this.positionObject.z = this.ZPosition;
		this.transform.position = this.positionObject;
	}
	
	// Update is called once per frame
	void Update () 
	{
		if (this.IsFall)
		{			
			if (!this.endFall)
			{
				if (this.ObjectToFall.transform.position.y > this.YMax)
				{
					this.positionObject.y += (Vector3.down.y * this.Speed * this.InitAcceleration *  Time.deltaTime);
					this.ObjectToFall.transform.position = this.positionObject;
					this.InitAcceleration  += Time.deltaTime;
					
					// Rotacion
					this.rotationObject.z -= (this.RotationSpeed * Time.deltaTime);
					this.ObjectToFall.transform.rotation = this.rotationObject;
					
					if (this.checkHit)
					{

                        RaycastHit hitLeft;
                        Vector3 auxVecHitLeft = this.ObjectToFall.transform.position;
                        auxVecHitLeft.x -= this.offsetSideRaycastLeft;
                        bool isHittingLeft = Physics.Raycast(auxVecHitLeft, Vector3.down, out hitLeft, 25.0f);

                        RaycastHit hitRight;
                        Vector3 auxVecHitRigth = this.ObjectToFall.transform.position;
                        auxVecHitRigth.x += this.offsetSideRaycastRigth;
                        bool isHittingRigth = Physics.Raycast(auxVecHitRigth, Vector3.down, out hitRight, 25.0f);


                        if ( isHittingLeft ||  isHittingRigth)
						{

						//RaycastHit hit;
						//if( Physics.Raycast( this.ObjectToFall.transform.position, Vector3.down, out hit, 25.0f ) )
						//{
                            RaycastHit hit = new RaycastHit();
                            if (isHittingLeft)
                            {
                                hit = hitLeft;
                            }
                            else if (isHittingRigth)
                            {
                                hit = hitRight;
                            }


							if (( hit.transform.gameObject.layer == LayerMask.NameToLayer("bodyCollider")) && 
								(hit.transform.parent.transform.position.z <= -10.0f) && 
								(hit.transform.parent.transform.position.z >= -38.0f))
							{
								
								if (hit.transform.parent.GetComponent<SimController>().Stunned())
								{
									
									this.endFall = true;
									Vector3 auxPos = this.ObjectToFall.transform.position;
									auxPos.x += 0.0f;
									auxPos.y -= 30.0f;
									auxPos.z -= 10.0f;
									this.instPC = GameObject.Instantiate(this.ParticleCloud,auxPos,Quaternion.identity) as GameObject;
									Destroy(this.ObjectToFall);
								}else
								{
									
									this.checkHit = false;	
								}
								
							}
						}
					}
				}else
				{
					this.positionObject.y = this.YMax;
					this.ObjectToFall.transform.position = this.positionObject;					
					this.endFall = true;	
					Vector3 auxPos = this.ObjectToFall.transform.position;
					auxPos.x += 0.0f;
					auxPos.y -= 30.0f;
					auxPos.z -= 10.0f;
					this.instPC = GameObject.Instantiate(this.ParticleCloud,auxPos,Quaternion.identity) as GameObject;
					Destroy(this.ObjectToFall);
				}
					

			}else
			{
				this.timeUpdateParticle += Time.deltaTime;
				if (this.timeUpdateParticle >= this.DelayParticle)
				{
					Destroy(this.instPC);
				}				
			}
		}		
	}
	
	
	void OnItemClick()
	{
		if (!this.IsFall)
		{
			this.IsFall = true;
			if (this.InteractiveIcon != null)
			{
				Destroy(this.InteractiveIcon);	
				
			}
			GameManager.Instance.Sound_Manager.PlaySoundEffect(this.FallBrickSound);
		}
	}


    void OnDrawGizmos()
    {
        if (this.ObjectToFall != null)
        {
            Gizmos.color = Color.yellow;
            Vector3 auxVecHitLeft = this.ObjectToFall.transform.position;
            auxVecHitLeft.x -= offsetSideRaycastLeft;
            Gizmos.DrawLine(auxVecHitLeft, auxVecHitLeft + Vector3.down * 25.0f);


            Vector3 auxVecHitRigth = this.ObjectToFall.transform.position;
            auxVecHitRigth.x += offsetSideRaycastRigth;
            Gizmos.DrawLine(auxVecHitRigth, auxVecHitRigth + Vector3.down * 25.0f);
        }
    }
}
