using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {
	
	private static GameManager gameManager = null;
	
	public static GameManager Instance 
	{
        get { return gameManager; } 
    } 
		
	
	#region SettingInitLevel
	// Type of game
	public enum TGAME
	{
		ARCADE = 0,
		CAMPAIGN		
	}
	
	[System.SerializableAttribute]
	public class SettingInitLevel
	{

		public bool helpMode 		= false;
		
		public bool tutorialMode 	= false;

		/// <summary>
		/// Number of world, number of level
		/// </summary>
		public int NumberWorld = 0;
		public int NumberLevel = 0;
		
		/// <summary>
		/// Name Next Level
		/// </summary>
		public string NameNextLevel = "";
		
		/// <summary>
		/// TypeGame
		/// </summary>
		public TGAME TypeOfGame = GameManager.TGAME.CAMPAIGN;
		
		/// <summary>
		/// Number of infected bronze,silver and gold
		/// </summary>
		public int NumberBronzeInfected = 0;
		public int NumberSilverInfected = 0;		
		public int NumberGoldInfected 	= 0;
		
		
		/// <summary>
		/// Music for each level
		/// </summary>
		//public SoundManager.TYPE_Music TypeMusicLevel = SoundManager.TYPE_Music.THEME_0;
		/// <summary>
		/// Music Level
		/// </summary>
		public AudioClip MusicLevel;
		
		private bool enablePanel = false;
		public bool EnablePanel
		{
		    get { return enablePanel; }
		    set { enablePanel = value; }
		}
		/// <summary>
		/// Indicates if waiting to show panel
		/// </summary>
		private bool waitingToShowPanel = false;
		public bool WaitingToShowPanel
		{
		    get { return waitingToShowPanel; }
		    set { waitingToShowPanel = value; }
		}


        //[System.NonSerializedAttribute]
        /// <summary>
        /// Setup Time in seconds to show panel for all cases
        /// </summary>
        public float DelayShowPanel = 0.8f;

        /// <summary>
        /// Time in seconds to show panel when sim is out of boundaries
        /// </summary>
        public float DelayShowPanelSimBoundaries = 2;

        /// <summary>
        /// Delay for show panel
        /// </summary>
        private float timeShowPanel = 0.5f;
        public float TimeShowPanel
        {
            get { return timeShowPanel; }
            set { timeShowPanel = value; }
        }

		/// <summary>
		/// Time to update to show panel
		/// </summary>
		private float timeUpdateShowPanel = 0.0f;
		public float TimeUpdateShowPanel
		{
		    get { return timeUpdateShowPanel; }
		    set { timeUpdateShowPanel = value; }
		}
		
		private GameManager.TypePanel tPanelToShow;
		public GameManager.TypePanel TPanelToShow
		{
		    get { return tPanelToShow; }
		    set { tPanelToShow = value; }
		}
		
		/// <summary>
		/// If there is more than one height
		/// </summary>
		public bool VariousHeights = false;

		
	}
	/// <summary>
	/// Initial setup for game
	/// </summary>
	public SettingInitLevel SetupInitLevel;	
	
	#endregion SettingInitLevel
	
	#region GameManagerStats
	[System.SerializableAttribute]
	public class GameManagerStats
	{
		#region Timers
		/// <summary>
		/// Time for scene in seconds
		/// </summary>
		public float TimeLevel = 150.0f;
		/// <summary>
		/// Time in seconds when the number of cats is 0 and list of zombies is 0 (0.5 seconds)
		/// </summary>
		public float TimeOutNoCats = 15.0f;
		
		/// <summary>
		/// Time left in level
		/// </summary>
		public int HoursLeft;
		public int MinutesLeft;
		public int SecondsLeft;
		
		
		/// <summary>
		/// Check if startTimer, and update timer
		/// </summary>
		private bool startTimer;
		public bool StartTimer
		{
		    get { return startTimer; }
		    set { startTimer = value; }
		}		
		private float updateTimer;
		public float UpdateTimer
		{
		    get { return updateTimer; }
		    set { updateTimer = value; }
		}
		/// <summary>
		/// Start time for no more cats
		/// </summary>
		private bool startTimerOutNoCats = false;
		public bool StartTimerOutNoCats
		{
		    get { return startTimerOutNoCats; }
		    set { startTimerOutNoCats = value; }
		}
		private float updateTimerOutNoCats;
		public float UpdateTimerOutNoCats
		{
		    get { return updateTimerOutNoCats; }
		    set { updateTimerOutNoCats = value; }
		}
		
		public void ConvertTime()
		{
			//this.TimeLevel;
			this.HoursLeft = (int)(this.updateTimer / (60 * 60));
			this.MinutesLeft = (int)(this.updateTimer / 60);
			this.SecondsLeft = (int)(this.updateTimer % 60);		
	
		}
		
		#endregion Timers
		
		#region EndLevel
		/// <summary>
		/// Indicates if win or loose level
		/// </summary>
		private bool winLevel;
		public bool WinLevel
		{
		    get { return winLevel; }
		    set { winLevel = value; }
		}
		
		/// <summary>
		/// Indicates if endLevel or not
		/// </summary>
		private bool endLevel = false;
		public bool EndLevel
		{
		    get { return endLevel; }
		    set { endLevel = value; }
		}
		#endregion EndLevel		
		
		#region StatsInGame
		/// <summary>
		/// Coins in this level
		/// </summary>
		private int levelCoins = 0;
		public int LevelCoins
		{
		    get { return levelCoins; }
		    set { levelCoins = value; }
		}
		
		/// <summary>
		/// Total coins in game
		/// </summary>
		private int totalCoins = 0;
		public int TotalCoins
		{
		    get { return totalCoins; }
		    set { totalCoins = value; }
		}
		#endregion StatsInGame	
		
	}
	public GameManagerStats GameStats;
	#endregion GameManagerStats
	
	#region SimSettings
	[System.SerializableAttribute]
	public class SimSettings
	{
		
		#region SettingsBoundaries
		/// <summary>
		/// Z Behind Stage
		/// </summary>
		private float zBackStage = 200.0f;
		public float ZBackStage
		{
		    get { return zBackStage; }
		    set { zBackStage = value; }
		}
		/// Maximum Z for sim
		/// </summary>
		private float farZSim = -10.0f;
		public float FarZSim
		{
		    get { return farZSim; }
		    set { farZSim = value; }
		}
		
		private float farZ = -5.0f;
		public float FarZ
		{
		    get { return farZ; }
		    set { farZ = value; }
		}
		
		
		// Bottom limit, Upper Limit
		private float[,] listFloorLimit = 
		{ 
			{ 14.0f, 168.0f}, 
			{ 345.0f, 567.0f } 
		};
		public float[,] ListFloorLimit
		{
			get { return listFloorLimit; }			
		}
		
		/// <summary>
		/// Second Floor upper limit
		/// </summary>
		private float floorDistance = 325.0f;
		public float FloorDistance
		{
			get { return floorDistance; }
		}
		
		/// <summary>
		/// Second Floor upper limit
		/// </summary>
		public float WidthFloor = 183.0f;
		
		// <summary>
		/// Second Floor upper limit
		/// </summary>
		public float OffsetZFloor = 60.0f;
		
		/// <summary>
		/// Floor Left limit
		/// </summary>
		private float floorLeftLimit = -2200.0f;	
		public float FloorLeftLimit
		{
		    get { return floorLeftLimit; }
		    set { floorLeftLimit = value; }
		}
		
		
		/// <summary>
		/// Floor Right limit
		/// </summary>
		private float floorRightLimit = 2200.0f;	
		public float FloorRightLimit
		{
		    get { return floorRightLimit; }
		    set { floorRightLimit = value; }
		}
		
		/// <summary>
		/// Farthest point of level in left
		/// </summary>
		private Vector3 leftFar = new Vector3(-3000.0f,100.0f,-10.0f);
		public Vector3 LeftFar
		{
			get { return this.leftFar; }	
		}
		/// <summary>
		/// Farthest point of level in right
		/// </summary>
		private Vector3 rightFar = new Vector3(3000.0f,100.0f,-10.0f);
		public Vector3 RightFar
		{
			get { return this.rightFar; }	
		}
		#endregion SettingsBoundaries	
		
	}
	public SimSettings SimSetupSettings;
	
	private List<SimController> listSims;
	public List<SimController> ListSims
	{
	    get { return listSims; }
	    set { listSims = value; }
	}	
	
	/// <summary>
	/// List of holes in firts places
	/// </summary>
	private List<Crack> listCracks;
	public List<Crack> ListCracks
	{
		get { return listCracks; }
		set { listCracks = value; }
	}

    /// <summary>
    /// Absolute number of infected, (counter to hold 
    /// number of infecte during level, without any death
    /// </summary>
    public int totalNumberInfected;
    public int TotalNumberInfected
    {
        get { return totalNumberInfected; }
        set { totalNumberInfected = value; }
    }

	/// <summary>
	/// The number infected.
	/// </summary>
	public int numberInfected = 0;	
	
	/// <summary>
	/// The number infected.
	/// </summary>
	public int NumberInfected 
	{
        get { return numberInfected; }
		set 
		{ 
			if (value >= 0)
			{
				numberInfected = value; 
				
				bool checkShowPanel = true;
				
				if (numberInfected >= this.SetupInitLevel.NumberBronzeInfected)
				{
					this.GameStats.WinLevel = true;					
				}else if (numberInfected < this.SetupInitLevel.NumberBronzeInfected)
				{
					this.GameStats.WinLevel = false;	
				}
				if (numberInfected >= this.SetupInitLevel.NumberGoldInfected)
				{

					
					this.GameStats.WinLevel = true;
					checkShowPanel = false;
					this.ShowEndPanel(TypePanel.WIN_PANEL,false);				
				}
				if (checkShowPanel && this.IsLastInfected())
				{
					this.ShowEndPanel(TypePanel.NONE_PANEL,false);						
				}				
				
			}else
			{				
				numberInfected = 0;
				this.GameStats.WinLevel = false;
			}			
		}
    }	

	/// <summary>
	/// Number of lost sims by fall or hide
	/// </summary>
	public int numberLostSims = 0;

	/// <summary>
	/// The number lost sims by fall or hide
	/// </summary>
	public int NumberLostSims 
	{
		get { return numberLostSims; }
		set 
		{ 
			if (value >= 0)
			{
				numberLostSims = value;
			}else
			{				
				numberLostSims = 0;
			}	
		}
	}

	/// <summary>
	/// Number death of zombie sims
	/// </summary>
	public int numberDeathZombies = 0;
	public int NumberDeathZombies 
	{
        get { return this.numberDeathZombies; }
		set 
		{ 	
			if (value >= 0)
			{
				// If number death is equal to number of sims loose 
                this.numberDeathZombies = value;
                if (numberDeathZombies >= this.listSims.Count)
				{
					
					this.ShowEndPanel(TypePanel.LOOSE_PANEL,false);
					
				}
                // If number of total sims minus number of death zombies is less than number bronze infected, 
                // Player won't win level
                if ((this.ListSims.Count - this.numberDeathZombies) < this.SetupInitLevel.NumberBronzeInfected)
                {
                    this.ShowEndPanel(TypePanel.LOOSE_PANEL, false);
                }
			}
		}
    }
	
	public bool IsLastInfected()
	{
		bool auxLastInfected = false;		
		if (this.numberInfected == this.listSims.Count)
		{
			auxLastInfected = true;
		}
        if ((this.numberDeathZombies + this.numberInfected) >= this.listSims.Count)
		{
			auxLastInfected = true;
		}
		
		return auxLastInfected;		
	}
	
	#endregion SimSettings	
	
	#region PauseGame
	private bool pauseGame = false;	
	public bool PauseGame 
	{
        get 
		{ return this.pauseGame; }
		set 
		{ 
			this.pauseGame = value; 
			if (this.pauseGame)
			{				
				Time.timeScale = 0.0f;
			}else
			{				
				Time.timeScale = 1.0f;
			}
		}
    }
	#endregion PauseGame
	
	#region CatsLevel
	[System.SerializableAttribute]
	public class Cats
	{
		// Type of game
		public enum TypeCat
		{
			NONE_CAT = 0,
			BASE_CAT,
			LOVE_CAT,
			TRIPLE_CAT,
			DUPLICATOR_CAT,
			SPEED_CAT,
			BOMB_CAT
			
		}
		
		public TypeCat[] CatCampaingLevel = new TypeCat[6];
		private int currentTotalCat = 0;		
		public int CurrentTotalCat
		{
		    get { return currentTotalCat; }
		    set { currentTotalCat = value; }
		}
		
		private int totalCat = 0;		
		
		
		/// <summary>
		/// Maximo numero used for golden star
		/// </summary>
		public int MaxCatsGoldenStar = 0;
		
		private List<Transform> listCatsFloor = new List<Transform>();
		public List<Transform> ListCatsFloor
		{
		    get { return listCatsFloor; }
		}

		/// <summary>
		/// Number of death cats
		/// </summary>
		private int numberDeathCats = 0;
		public int NumberDeathCats
		{
			get { return numberDeathCats; }
		}
		
		/// <summary>
		/// Determines whether if is used less or equalthan max golden
		/// </summary>
		/// <returns>
		/// <c>true</c> Returns if used minimun cats <c>false</c>.
		/// </returns>
		public bool IsUsedMinCats()
		{		
			bool auxUsedMin = false;
			// If number of max cat is 0, doesn't count
			if (this.MaxCatsGoldenStar <= 0)
			{
				auxUsedMin = true;
				
			}else
			{
				if ((this.totalCat - this.currentTotalCat) <= this.MaxCatsGoldenStar)
				{
					auxUsedMin = true;	
				}				
			}
			
			return auxUsedMin;	
		}			
		
		public void TakeCat()
		{
			this.currentTotalCat -=1;
			if (this.currentTotalCat < 0)
			{
				this.currentTotalCat = 0;
			}			
		}
		
		public void RemoveCatFloor(Transform cat)
		{
			this.listCatsFloor.Remove(cat);
			this.currentTotalCat -=1;
			if (this.currentTotalCat < 0)
			{
				this.currentTotalCat = 0;
			}			
		}
		
		public void AddCat(Transform cat)
		{
			this.listCatsFloor.Add(cat);
		}
		
		#region ArcadeCat
		/// <summary>
		/// Number of Base Cat
		/// </summary>
		public int BaseCat;
		
		/// <summary>
		/// Number of CurrentBaseCat in this level
		/// </summary>
		[System.NonSerializedAttribute]
		public int CurrentBaseCat;
		
		// <summary>
		/// Number of Love Cat
		/// </summary>
		public int LoveCat;
		
		// <summary>LoveCat in this level
		/// </summary>
		[System.NonSerializedAttribute]
		public int CurrentLoveCat;
		
		/// <summary>
		/// The triple cat.
		/// </summary>
		public int TripleCat;
		
		/// <summary>
		/// The current triple cat.
		/// </summary>
		[System.NonSerializedAttribute]
		public int CurrentTripleCat;
		
		/// <summary>
		/// The duplicator cat.
		/// </summary>
		public int DuplicatorCat;
		
		/// <summary>
		/// The current duplicator cat.
		/// </summary>
		[System.NonSerializedAttribute]
		public int CurrentDuplicatorCat;

		public int BombCat;
		
		// <summary>BombCat in this level
		/// </summary>
		[System.NonSerializedAttribute]
		public int CurrentBombCat;
		#endregion ArcadeCat
		
		/// <summary>
		/// Inits the cats.
		/// </summary>
		/// <param name='lCatsFloor'>
		/// List of cats in floor
		/// </param>
		public void InitCats(List<Transform> lCatsFloor)
		{
			this.listCatsFloor = lCatsFloor;
			// Setup cats for this level
			this.CurrentBaseCat = this.BaseCat;
			this.CurrentLoveCat = this.LoveCat;
			this.CurrentTripleCat = this.TripleCat;
			this.CurrentDuplicatorCat = this.DuplicatorCat;
			this.CurrentBombCat = this.BombCat;
			
			this.totalCat = 0;
			this.currentTotalCat = 0;
			
			for (int i= 0; i<this.CatCampaingLevel.Length; i++)
			{
				if (this.CatCampaingLevel[i] != TypeCat.NONE_CAT)
				{
					
					this.totalCat++;
				}				
			}
			this.totalCat += lCatsFloor.Count;
			this.currentTotalCat = this.totalCat;
			if (this.MaxCatsGoldenStar > this.totalCat)
			{
				this.MaxCatsGoldenStar = this.totalCat;				
			}	
			
		}	

		public void SetDeathCats()
		{
			this.numberDeathCats += 1;
			if (this.numberDeathCats > this.totalCat)
			{
				this.numberDeathCats = this.totalCat;
			}
		}

		public bool CheckDeathCats()
		{
			if (this.numberDeathCats >= this.totalCat)
			{
				return true;
			}			
			return false;
		}		
	}
	/// <summary>
	/// Settings components
	/// </summary>
	public Cats SettingCats;
	#endregion CatsLevel
	
	#region Prefabs
	[System.SerializableAttribute]
	public class SettingsPrefabs
	{
		/// <summary>
		/// Panel for win level
		/// </summary>
		private WinPanelController winPanel;
		public WinPanelController WinPanel
		{
		    get { return winPanel; }
		    set { winPanel = value; }
		}	
		/// <summary>
		/// Panel for lose level
		/// </summary>
		private LosePanelController losePanel;
		public LosePanelController LosePanel
		{
		    get { return losePanel; }
		    set { losePanel = value; }
		}	
		
		
		/// <summary>
		/// Panel for pause
		/// </summary>
		private GameObject pausePanel;
		public GameObject PausePanel
		{
		    get { return pausePanel; }
		    set { pausePanel = value; }
		}	
		
		/// <summary>
		/// Left boundary
		/// </summary>
		public GameObject LeftBoundary;
		/// <summary>
		/// Right boundary
		/// </summary>
		public GameObject RightBoundary;	
		
		#region DebugMode	
		public GameObject TableBoardParent;	
		public GameObject CellObject;	
		#endregion DebugMode
	
		public Transform GreenFloor;

		
		public void init()
		{
			GameObject guiCamperaCampaign =  GameObject.Find("GUICameraCampaign");
			this.winPanel = guiCamperaCampaign.transform.GetComponent<WinPanelController>();
			this.losePanel = guiCamperaCampaign.transform.GetComponent<LosePanelController>();
			this.pausePanel = guiCamperaCampaign.transform.FindChild("Anchor (MiddleCenter)").transform.FindChild("pausePanel").gameObject;

		}
		
	}
	public SettingsPrefabs SetupPrefabs;
	
	private SoundManager sound_Manager;
	public SoundManager Sound_Manager
	{
	    get { return sound_Manager; }
	    set { sound_Manager = value; }
	}	
	#endregion Prefabs

	#region sounds

	// <summary>
	/// Sound effect when win level
	/// </summary>
	public AudioClip WinLevelSound;
	/// <summary>
	/// Sound effect when lose level
	/// </summary>
	public AudioClip LoseLevelSound;

	#endregion sounds
	
	void Awake ()
	{
		//PlayerPrefs.DeleteAll();
        if ( GameManager.gameManager==null )
		{
             GameManager.gameManager = this;
        }		
	}

	//IPD
	void Start ()
	{
		// Si el numero de intentos es mayor o igual que 2 entre en helpMode
		if( PlayerPrefsManager.GetTries() < 2)
		{
			SetupInitLevel.helpMode = false;
		}
		else
		{
			SetupInitLevel.helpMode = true;
		}

		if( 20 <= PlayerPrefsManager.GetTotalStars() )
		{
			PlayerPrefsManager.UnlockNewSkin( CatSkinController.SkinType.BLACK_CAT );
		}
		if( 40 <= PlayerPrefsManager.GetTotalStars() )
		{
			PlayerPrefsManager.UnlockNewSkin( CatSkinController.SkinType.MAGIC_CAT );
		}
		if( 80 <= PlayerPrefsManager.GetTotalStars() )
		{
			PlayerPrefsManager.UnlockNewSkin( CatSkinController.SkinType.BAT_CAT );
		}
	}
	
	/// <summary>
	/// Method to reset scene (this method init scene, now is called in loadScene gameObject)
	/// </summary>
	public void InitScene()
	{
		// List sims
		this.listSims = new List<SimController>();
		// Index sim
		int indexSim = 0;
		foreach (GameObject auxGO in GameObject.FindGameObjectsWithTag("Sim"))
		{
			SimController auxSim = auxGO.GetComponent<SimController>();
			auxSim.Stats.IDSim = indexSim;
			auxGO.transform.name += "_" +  auxSim.Stats.IDSim;
			this.listSims.Add(auxSim);
			indexSim ++;			
			
		}		

		this.numberInfected = 0;
		this.numberLostSims = 0;
		this.GameStats.UpdateTimer = this.GameStats.TimeLevel;
		this.GameStats.UpdateTimerOutNoCats = this.GameStats.TimeOutNoCats;
		this.GameStats.StartTimer = true;
        		
		this.GameStats.HoursLeft = 0;
		this.GameStats.MinutesLeft = 0;
		this.GameStats.SecondsLeft = 0;
		this.GameStats.ConvertTime();

        this.SetupInitLevel.TimeShowPanel = this.SetupInitLevel.DelayShowPanel;
		
		// Initialize cats
		// List cats
		List<Transform> listCatsFloor = new List<Transform>();
		foreach (GameObject auxGO in GameObject.FindGameObjectsWithTag("Cat"))
		{
			listCatsFloor.Add(auxGO.transform);
		}
		this.SettingCats.InitCats(listCatsFloor);
		
		
		this.Sound_Manager = this.transform.GetComponent<SoundManager>();
		this.Sound_Manager.PlayMusic(this.SetupInitLevel.MusicLevel);
		
		this.GameStats.LevelCoins = 0;
		
		this.SetupInitLevel.NumberWorld = Mathf.Clamp(this.SetupInitLevel.NumberWorld,0, GameSettings.NumberWorlds);
		this.SetupInitLevel.NumberLevel = Mathf.Clamp(this.SetupInitLevel.NumberLevel,0, GameSettings.NumberLevelPerWorld);
		
		this.SetupPrefabs.init();

		// Setup green floor
		if (this.SetupPrefabs.GreenFloor != null)
		{
			Vector3 positionFloor = this.SetupPrefabs.GreenFloor.position;
			tk2dSlicedSprite floorSlice = this.SetupPrefabs.GreenFloor.GetComponent<tk2dSlicedSprite>();
			Vector2 sliceFloorDimension = floorSlice.dimensions;
			
			if (this.SetupInitLevel.VariousHeights)
			{
				sliceFloorDimension.y = 245.0f;
				positionFloor.y = 334.0f;
				
			}else
			{
				sliceFloorDimension.y = 195.0f;
				positionFloor.y = -23.36f;
			}
			this.SetupPrefabs.GreenFloor.position = positionFloor;
			floorSlice.dimensions = sliceFloorDimension;
		}

		
		// Load number level/world
		string nameCurrentLevel = Application.loadedLevelName;
		string[] splitCL = nameCurrentLevel.Split('_');
		if (splitCL != null)
		{
			if (splitCL.Length == 3)
			{
				this.SetupInitLevel.NumberWorld = int.Parse(splitCL[0]);
				this.SetupInitLevel.NumberLevel = int.Parse(splitCL[1]);
				
			}else
			{
				this.SetupInitLevel.NumberWorld = 0;
				this.SetupInitLevel.NumberLevel = 0;				
			}
			
		}else
		{
			this.SetupInitLevel.NumberWorld = 0;
			this.SetupInitLevel.NumberLevel = 0;

			
		}
		this.SetupInitLevel.NameNextLevel = LevelList.GetNextLevel(this.SetupInitLevel.NumberWorld,this.SetupInitLevel.NumberLevel);
		
		// Clamp number stars
		this.SetupInitLevel.NumberBronzeInfected = Mathf.Clamp(this.SetupInitLevel.NumberBronzeInfected,0,this.listSims.Count);
		this.SetupInitLevel.NumberSilverInfected = Mathf.Clamp(this.SetupInitLevel.NumberSilverInfected,0,this.listSims.Count);
		this.SetupInitLevel.NumberGoldInfected = Mathf.Clamp(this.SetupInitLevel.NumberGoldInfected,0,this.listSims.Count);
		
		
		// Setup list of cracks
		this.setListCracks();
		
	}		
	
	// Update is called once per frame
	void Update () 
	{		
		if (!GameManager.Instance.pauseGame)
		{
			//Si el ninja esta activado no se puede perder por timeOut
			if( !PowerUpNinja.powerUpNinjaActived )
			{
				this.updateTimeLevel();	
			}
			this.updateShowEndPanel();
		}
		
		if (Input.GetKeyDown(KeyCode.P))
		{
			GameManager.Instance.PauseGame = !GameManager.Instance.PauseGame;	
		}
	}

	#region Fin Sim by Distance
	/// <summary>
	/// TypeDirection
	/// </summary>
	public enum DirectionFindType
	{
		BOTH = 0,
		LEFT,
		RIGHT,
		
	}
	
	private Transform getNearCat(SimController currentSim, List<Transform> list)
	{
		Transform cat = null;
		if (list.Count > 1)
		{
			// Get minimun distance from first element
			float minDistance =  Vector2.Distance(new Vector2(list[0].position.x, list[0].position.y),
				new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
			cat = list[0];
			// Check with next elements
			for (int i= 1; i<list.Count; i++)
			{
				float auxDistance = Vector2.Distance(new Vector2(list[i].position.x, list[i].position.y),
					new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
				// If current min distance is greater than new distance, update
				if (minDistance >  auxDistance)
				{
					cat = list[i];
					minDistance = auxDistance;
				}
				
			}
		}else if (list.Count == 1)
		{
			// Only one element, return first in list
			cat = list[0];
			
		}
		
		return cat;
	}
	
	
	private SimController getNearSim(SimController currentSim, List<SimController> list)
	{
		SimController sim = null;
		if (list.Count > 1)
		{
			
			// Get minimun distance from first element
			float minDistance =  Vector2.Distance(new Vector2(list[0].transform.position.x, list[0].transform.position.y),
				new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
			sim = list[0];
			// Check with next elements
			for (int i= 1; i<list.Count; i++)
			{
				float auxDistance = Vector2.Distance(new Vector2(list[i].transform.position.x, list[i].transform.position.y),
					new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
				// If current min distance is greater than new distance, update
				if (minDistance >  auxDistance)
				{
					sim = list[i];
					minDistance = auxDistance;
				}
				
			}
		}else if (list.Count == 1)
		{
			// Only one element, return first in list
			sim = list[0];
			
		}
		
		return sim;
	}
	
		
	public List<SimController> GetSimsInDistance(SimController currentSim,float distance, bool isInfected, List<SimController.State> lStatesFind)
	{
		List<SimController> auxList = new List<SimController>();
		foreach (SimController auxSim in this.listSims)
		{
			// Not me, and is inside scenary	
			if ((auxSim.Stats.IDSim != currentSim.Stats.IDSim) && 
				(!auxSim.Stats.OutsideScenery) && 
				(auxSim.Stats.Number_Floor == currentSim.Stats.Number_Floor))
			{
				if (isInfected == auxSim.Stats.Infected)
				{
					bool matchState = false;
					foreach (SimController.State auxState in lStatesFind)
					{
						if (auxSim.FSMSim.CurrentStateID == (int)auxState)
						{
							matchState = true;
							break;								
						}						
					}
					// Added to list
					if (matchState)
					{
						// Check with distance
						float auxDistance = Vector2.Distance(new Vector2(auxSim.transform.position.x, auxSim.transform.position.y),
							new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
						// If current min distance is greater than new distance, update
						if (distance >= auxDistance)
						{
							auxList.Add(auxSim);
							
						}
					}
						
				}				
			}
		}
		
		return auxList;
	}
	
	public bool GetCatByDirection(SimController currentSim,out Transform oCat,DirectionFindType direction, float distanceSearch)
	{
		oCat = null;
		bool findCat = false;
		List<Transform> auxList = new List<Transform>();
		foreach (Transform auxCat in this.SettingCats.ListCatsFloor)
		{
			if (auxCat != null)
			{
				CatController aux_c = auxCat.GetComponent<CatController>();
				if (currentSim.Stats.Number_Floor == aux_c.floor)
				{
					bool matchDirection = false;
					// Check direction
					switch (direction)						
					{
						case DirectionFindType.LEFT:
							if (auxCat.position.x < currentSim.transform.position.x)
							{												
								matchDirection = true;						
							}
						break;
						case DirectionFindType.RIGHT:				
							if (auxCat.position.x >= currentSim.transform.position.x)
							{		
								matchDirection = true;								
							}
						break;
						
					}			
					if (matchDirection)
					{
						float auxDistance = Vector2.Distance(new Vector2(auxCat.position.x, auxCat.position.y),
								new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
						// If current min distance is greater than new distance, update
						if (distanceSearch >= auxDistance)
						{
							auxList.Add(auxCat);							
						}		
					}
				}
			}			
		}		
		
		// Find near sim
		if (auxList.Count > 0)
		{
			oCat = getNearCat(currentSim,auxList);
			findCat = true;			
		}
		
		return findCat;		
	}
	
	
	public  bool GetSimByDirection(SimController currentSim,out SimController otherSim,DirectionFindType direction,bool isInfected, bool isInfecting, List<SimController.State> lStatesFind, float distanceSearch)
	{
		otherSim = null;
		bool findSim = false;
		List<SimController> auxList = new List<SimController>();
		
		foreach (SimController auxSim in this.listSims)
		{			
			// Not me, and is inside scenary	
			if ((auxSim.Stats.IDSim != currentSim.Stats.IDSim) && 
				(!auxSim.Stats.OutsideScenery) &&
				(auxSim.Stats.Number_Floor == currentSim.Stats.Number_Floor))
			{	
							
				bool matchDirection = false;
				// Check direction
				switch (direction)						
				{
					case DirectionFindType.LEFT:
						if (auxSim.transform.position.x < currentSim.transform.position.x)
						{
													
							matchDirection = true;
							
						}
					break;
					case DirectionFindType.RIGHT:
					
						if (auxSim.transform.position.x >= currentSim.transform.position.x)
						{
							matchDirection = true;
							
						}
					break;
				}
				// Direction to find is the same Both state of infected has the same
				if ((matchDirection) &&
					((isInfected == auxSim.Stats.Infected) ||
					 (isInfecting == auxSim.Stats.IsInfecting))
				    )				
				{						
					// Search if auxSim has one of the states in list
					bool matchState = false;
					foreach (SimController.State auxState in lStatesFind)
					{
						if (auxSim.Stats.CurrentState == auxState)
						{
							matchState = true;
							break;	
							
						}
						
					}
					// Added to list
					if (matchState)
					{
						
						// Check with distance
						float auxDistance = Vector2.Distance(new Vector2(auxSim.transform.position.x, auxSim.transform.position.y),
							new Vector2(currentSim.transform.position.x, currentSim.transform.position.y));
						// If current min distance is greater than new distance, update
						if (distanceSearch >= auxDistance)
						{
							auxList.Add(auxSim);							
						}						
					}					
				}				
			}			
		}
		
		// Find near sim
		if (auxList.Count > 0)
		{
			otherSim = getNearSim(currentSim,auxList);
			findSim = true;
			
		}
		
		return findSim;
		
	}
	
	#endregion Fin Sim by Distance	
	
	#region Time
	
	private void updateTimeLevel()
	{
		// If not win level or not end level
		if ((!this.GameStats.WinLevel) && (!this.GameStats.EndLevel))
		{
			// Check number of infected an cats
			if (this.SettingCats.CurrentTotalCat <= 0)
			{
				// CountDown for cats
				if ((!this.GameStats.StartTimerOutNoCats) && (this.numberInfected == 0))
				{
					this.GameStats.StartTimerOutNoCats = true;
					this.GameStats.UpdateTimerOutNoCats = this.GameStats.TimeOutNoCats;
				
				}
				if (this.numberInfected > 0)
				{
					this.GameStats.StartTimerOutNoCats = false;	
					this.GameStats.UpdateTimerOutNoCats = this.GameStats.TimeOutNoCats;
					
				}
				
				if (this.GameStats.StartTimerOutNoCats)
				{
					this.GameStats.UpdateTimerOutNoCats -=Time.deltaTime;	
					if (this.GameStats.UpdateTimerOutNoCats < 0.0f)
					{
						this.GameStats.StartTimerOutNoCats = false;
						this.GameStats.UpdateTimerOutNoCats = 0.0f;
						this.ShowEndPanel(TypePanel.TIMEOUT_NOCATS_PANEL,false);
						
					}
				}
			}
			
			// Timer of level
			if (this.GameStats.StartTimer)
			{
				this.GameStats.ConvertTime();							
				this.GameStats.UpdateTimer -=Time.deltaTime;	
				if (this.GameStats.UpdateTimer <= 0)
				{
					this.GameStats.StartTimer = false;
					this.GameStats.UpdateTimer = 0.0f;
					this.ShowEndPanel(TypePanel.TIMEOUT_PANEL,false);
				}
			}
			
		}
		
		
	}
	#endregion Time	
	
	#region GUI
	public enum TypePanel
	{
		NONE_PANEL = 0,
		WIN_PANEL,
		LOOSE_PANEL,
		TIMEOUT_PANEL,
		TIMEOUT_NOCATS_PANEL,
	}
	
	private void updateShowEndPanel()
	{
        if (this.SetupInitLevel.WaitingToShowPanel)
        {
            this.SetupInitLevel.TimeUpdateShowPanel += Time.deltaTime;
            if (this.SetupInitLevel.TimeUpdateShowPanel >= this.SetupInitLevel.TimeShowPanel)
            {

                this.SetupInitLevel.WaitingToShowPanel = false;
                this.enablePanel(this.SetupInitLevel.TPanelToShow);

            }
        }
        else
        {
            // Check Number of dead cats
            if (!this.GameStats.EndLevel)
            {
                if (this.SettingCats.CheckDeathCats() && this.numberInfected <= 0)
                {
                    this.ShowEndPanel(TypePanel.LOOSE_PANEL, true);
                }
            }

            // Check number lost
            // Number of sims minus lost sims, minus infected == 0, end level
            int auxSims = this.listSims.Count - this.numberLostSims - this.numberInfected;
            if (auxSims <= 0)
            {
                this.ShowEndPanel(TypePanel.NONE_PANEL, true);
            }
        }		
	}
	
	public void ShowEndPanel(TypePanel tPanel,bool activeDelay)
	{
		if( GameStats.WinLevel )
		{
			//IPD20140807
			PlayerPrefsManager.AddWinLevel( Application.loadedLevelName );
		}

        // Enable panel
        this.SetupPrefabs.PausePanel.SetActive(false);
		this.GameStats.EndLevel = true;
		if (activeDelay)
		{
            
			this.SetupInitLevel.WaitingToShowPanel = true;
			this.SetupInitLevel.TPanelToShow = tPanel;
            this.SetupInitLevel.TimeUpdateShowPanel = 0.0f;
		}else

		{
			this.enablePanel(tPanel);
		}
	}
	
	private void enablePanel(TypePanel TPanel)
	{
        this.Sound_Manager.PauseMusic();
		//TypePanel auxTPanel = TPanel;
		if (!this.SetupInitLevel.EnablePanel)
		{
			this.SetupInitLevel.EnablePanel = true;
			

			int auxNumberStars = GameSettings.NoneStars;
			int auxNumberInfected = GameManager.Instance.NumberInfected;			
			if (auxNumberInfected < this.SetupInitLevel.NumberBronzeInfected)
			{					
				this.GameStats.WinLevel = false;
				//auxTPanel = TypePanel.LOOSE_PANEL;
			}else
			{
				
				this.GameStats.WinLevel = true;
			}
			
			if (this.GameStats.WinLevel)
			{
                 
                //this.Sound_Manager.PlaySoundEffect(SoundManager.TYPE_SOUND_EFFECT.SE_WIN);
				this.Sound_Manager.PlaySoundEffect(this.WinLevelSound);
				this.SetupPrefabs.WinPanel.panel.SetActive(true);

				SetupPrefabs.WinPanel.SetWinPanel();

				if (auxNumberInfected >= this.SetupInitLevel.NumberGoldInfected)
				{
				 	if (this.SettingCats.IsUsedMinCats())
					{
						auxNumberStars = GameSettings.GoldStars;
						this.SetupPrefabs.WinPanel.goldeStars.SetActive(true);
						this.SetupPrefabs.WinPanel.silverStars.SetActive(false);
						this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
						
					}else
					{
						auxNumberStars = GameSettings.SilverStars;
						this.SetupPrefabs.WinPanel.goldeStars.SetActive(false);
						this.SetupPrefabs.WinPanel.silverStars.SetActive(true);
						this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
					}
				}// If get sims by silver, check number of cats
				else if (auxNumberInfected >= this.SetupInitLevel.NumberSilverInfected)
				{
					if (this.SetupInitLevel.NumberSilverInfected == this.SetupInitLevel.NumberGoldInfected)
					{
						if (this.SettingCats.IsUsedMinCats())
						{
							auxNumberStars = GameSettings.GoldStars;
							this.SetupPrefabs.WinPanel.goldeStars.SetActive(true);
							this.SetupPrefabs.WinPanel.silverStars.SetActive(false);
							this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
							
						}else
						{
							auxNumberStars = GameSettings.SilverStars;
							this.SetupPrefabs.WinPanel.goldeStars.SetActive(false);
							this.SetupPrefabs.WinPanel.silverStars.SetActive(true);
							this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
						}
						
					}else
					{
						auxNumberStars = GameSettings.SilverStars;
						this.SetupPrefabs.WinPanel.goldeStars.SetActive(false);
						this.SetupPrefabs.WinPanel.silverStars.SetActive(true);
						this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
					}
					
				}
				else if (auxNumberInfected >= this.SetupInitLevel.NumberBronzeInfected)
				{
					// Same level silver and bronze stars, get silver stars
					if (this.SetupInitLevel.NumberBronzeInfected == this.SetupInitLevel.NumberSilverInfected)
					{
						auxNumberStars = GameSettings.SilverStars;	
						this.SetupPrefabs.WinPanel.silverStars.SetActive(true);
						this.SetupPrefabs.WinPanel.bronzeStars.SetActive(false);
					}else
					{
						auxNumberStars = GameSettings.BronzeStars;	
						this.SetupPrefabs.WinPanel.silverStars.SetActive(false);
						this.SetupPrefabs.WinPanel.bronzeStars.SetActive(true);
					}
					this.SetupPrefabs.WinPanel.goldeStars.SetActive(false);					
					
				}
				
				// Set number stars and levels
				PlayerPrefsManager.AddCoins(this.GameStats.LevelCoins);
				PlayerPrefsManager.AddStars(this.SetupInitLevel.NumberWorld,this.SetupInitLevel.NumberLevel,auxNumberStars);				
			
			}else
			{
				this.SetupPrefabs.LosePanel.panel.SetActive(true);
				this.Sound_Manager.PlaySoundEffect(this.LoseLevelSound);
                //this.Sound_Manager.PlaySoundEffect(SoundManager.TYPE_SOUND_EFFECT.SE_LOOSE);
				switch(TPanel)
				{
					// Time out 
					case TypePanel.TIMEOUT_PANEL:
						this.SetupPrefabs.LosePanel.SetTimeOutPanel();
					
					break;
					// Time out no cats
					case TypePanel.TIMEOUT_NOCATS_PANEL:
						this.SetupPrefabs.LosePanel.SetTimeOutPanel();
					break;					
					
				}				
			}
		}
	}
	
	#endregion			
	
	#region Cracks
	
	private void setListCracks()
	{
		this.listCracks = new List<Crack>();
		foreach (GameObject auxGO in GameObject.FindGameObjectsWithTag("Crack"))
		{
			Crack auxCrack = auxGO.transform.GetComponent<Crack>();
			if (auxCrack.TypeCrack != Crack.TCrack.TRAPDOOR_CRACK)
			{						
				this.listCracks.Add(auxCrack);			
			}
		}

	}
	
	#endregion Cracks
		
}
