	using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public static class PlayerPrefsManager 
{	
	public enum TypeAction
	{
		NONE_ACTION = 0	,
		LOAD_ACTION,
		SAVE_ACTION,
		CLEAN_ACTION,
		ADD_ACTION,
	}
	
	public enum TypePrefs
	{
		CURRENTWORLD_PREF = 0,
		LASTSELECTEDWORLD_PREF,
		CURRENTLEVEL_PREF,
		TOTALCOIN_PREF,
		TOTALSTARS_PREF, 
        SOUNDEFFECT_PREF,
        MUSIC_PREF,
		
	}
	
	public class LevelStat
	{
		public int WorldLevel;
		public int NumberLevel;
		//public int Coins;
		public int Stars;
		
	}
	
	public static void ResetLevels()
	{
		for (int indexWorld = 0; indexWorld<GameSettings.NumberWorlds; indexWorld++)
		{
			for (int indexLevel = 0; indexLevel<GameSettings.NumberLevelPerWorld; indexLevel++)
			{
				string namePref = GameSettings.NamePrefLevelStats + indexWorld + "_" + indexLevel ;
				string valuePref = namePref + "_" + 0;
				PlayerPrefs.SetString(namePref,valuePref);
				PlayerPrefs.Save();
							
			}
			
		}		
	}


    /// <summary>
    /// Get state sound effect
    /// </summary>
    /// <returns>True == mute, false == no mute</returns>
    public static bool GetIsSoundEffectMute()
    {
        string namePref = GameSettings.NamePrefSoundEffect;
        bool result = false;
        // Not exist, create
        if (!PlayerPrefs.HasKey(namePref))
        {
            // 0 false, no mute
            // 1 true, mute
            PlayerPrefs.SetInt(namePref, 0);
            PlayerPrefs.Save();
            return false;

        }else
        {
            int valuePref = PlayerPrefs.GetInt(namePref);
            if (valuePref == 0)
            {
                result = false;
            }
            else
            {
                result = true;
            }
        }

        return result;
    }

    /// <summary>
    /// Set sound effect
    /// </summary>
    /// <param name="isMute">If mute sound effect</param>
    public static void SetSoundEffect(bool isMute)
    {
        string namePref = GameSettings.NamePrefSoundEffect;
        if (isMute)
        {
            PlayerPrefs.SetInt(namePref, 1);
            PlayerPrefs.Save();
        }
        else 
        {
            PlayerPrefs.SetInt(namePref, 0);
            PlayerPrefs.Save();
        }
    }


    /// <summary>
    /// Get state music
    /// </summary>
    /// <returns>True == mute, false == no mute</returns>
    public static bool GetIsMusicMute()
    {
        string namePref = GameSettings.NamePrefMusic;
        bool result = false;
        // Not exist, create
        if (!PlayerPrefs.HasKey(namePref))
        {
            // 0 false, no mute
            // 1 true, mute
            PlayerPrefs.SetInt(namePref, 0);
            PlayerPrefs.Save();
            return false;

        }
        else
        {
            int valuePref = PlayerPrefs.GetInt(namePref);
            if (valuePref == 0)
            {
                result = false;
            }
            else
            {
                result = true;
            }
        }

        return result;
    }

    /// <summary>
    /// Set state of music
    /// </summary>
    /// <param name="isMute">If mute music</param>
    public static void SetMusic(bool isMute)
    {
        string namePref = GameSettings.NamePrefMusic;
        if (isMute)
        {
            PlayerPrefs.SetInt(namePref, 1);
            PlayerPrefs.Save();
        }
        else
        {
            PlayerPrefs.SetInt(namePref, 0);
            PlayerPrefs.Save();
        }
    }


	public static void SetLevel(int indexWorld, int indexLevel, int stars)
	{
		string namePref = GameSettings.NamePrefLevelStats + indexWorld + "_" + indexLevel ;
		string valuePref = namePref + "_" + stars;
		PlayerPrefs.SetString(namePref,valuePref);
		PlayerPrefs.Save();			
		
	}
	
	public static LevelStat GetLevel(int indexWorld, int indexLevel)
	{
		LevelStat auxLevel = new LevelStat();
		string namePref = GameSettings.NamePrefLevelStats + indexWorld + "_" + indexLevel ;
		
		string valuePref = PlayerPrefs.GetString(namePref);
		if (!PlayerPrefs.HasKey(namePref))
		{
			valuePref = namePref + "_" + 0  + "_"+ 0;
			auxLevel.WorldLevel = indexWorld;
			auxLevel.NumberLevel = indexLevel;
			auxLevel.Stars = -1;
			PlayerPrefs.SetString(namePref,valuePref);
			PlayerPrefs.Save();
			
		}else
		{
			valuePref = PlayerPrefs.GetString(namePref);
			string[] arrayPref = valuePref.Split('_');
			if (arrayPref != null)
			{
				if(arrayPref.Length >= 3)
				{
					auxLevel.WorldLevel = int.Parse(arrayPref[1]);
					auxLevel.NumberLevel = int.Parse(arrayPref[2]);
					auxLevel.Stars = int.Parse(arrayPref[3]);
				}	
			}
		}
		
		
		return auxLevel;
			
		
	}
	
	
	public static List<LevelStat> GetList()
	{
		List<LevelStat> laux = new List<LevelStat>();
		
		for (int indexWorld = 0; indexWorld<GameSettings.NumberWorlds; indexWorld++)
		{
			for (int indexLevel = 0; indexLevel<GameSettings.NumberLevelPerWorld; indexLevel++)
			{
				string namePref = GameSettings.NamePrefLevelStats + indexWorld + "_" + indexLevel ;		
				if (PlayerPrefs.HasKey(namePref))
				{
					string valuePref = PlayerPrefs.GetString(namePref);
					
					string[] arrayPref = valuePref.Split('_');
					if (arrayPref != null)
					{	
						if(arrayPref.Length >= 3)
						{
							LevelStat auxVal = new LevelStat();			
							auxVal.WorldLevel = int.Parse(arrayPref[1]);
							auxVal.NumberLevel = int.Parse(arrayPref[2]);
							auxVal.Stars = int.Parse(arrayPref[3]);
							laux.Add(auxVal);
						}					
					}
				}
			}
			
		}
		return laux;		
	}
	
	
	
	
	#region GeneralStats	
	
	public static int SetPlayerPrefs(TypeAction action,TypePrefs pref)
	{
		int returnPref = 0;
		returnPref = SetPlayerPrefs(action,pref,0);
		return returnPref;
	}
	
	public static int SetPlayerPrefs(TypeAction action,TypePrefs pref, int _returnPref)
	{
		int returnPref = 0;
		bool existPref = false;
		string namePref ="";
		
		switch(pref)
		{
			case TypePrefs.CURRENTWORLD_PREF:
				namePref = GameSettings.NamePrefCurrentWorld;
			break;
			case TypePrefs.LASTSELECTEDWORLD_PREF:
				namePref = GameSettings.NamePrefLastWorldSelect;
			break;
			case TypePrefs.CURRENTLEVEL_PREF:
				namePref = GameSettings.NamePrefCurrentLevel;
			break;
			case TypePrefs.TOTALCOIN_PREF:
				namePref = GameSettings.NamePrefTotalCoins;
			break;
			case TypePrefs.TOTALSTARS_PREF:
				namePref = GameSettings.NamePrefTotalStars;
			break;
		}
		
		// If that prefs don't exist, create and 
		if (!PlayerPrefs.HasKey(namePref))
		{
			PlayerPrefs.SetInt(namePref,returnPref);
			PlayerPrefs.Save();
			existPref = true;
		}else
		{
			existPref = true;
		}
		
		if (existPref)
		{
			switch(action)
			{
				case TypeAction.LOAD_ACTION:
					returnPref = PlayerPrefs.GetInt(namePref);
				
				break;
				case TypeAction.SAVE_ACTION:
					returnPref = _returnPref;
					PlayerPrefs.SetInt(namePref,returnPref);
					PlayerPrefs.Save();

				break;
				case TypeAction.ADD_ACTION:
					int currentCount = PlayerPrefs.GetInt(namePref);
					PlayerPrefs.SetInt(namePref, currentCount + _returnPref );
					PlayerPrefs.Save();
					break;
				case TypeAction.CLEAN_ACTION:
					returnPref = 0;
					PlayerPrefs.SetInt(namePref,returnPref);
					PlayerPrefs.Save();
				break;
			}
		}
		return returnPref;
	}
	


	public static void AddCoins(int numberCoins)
	{
		// Load coins, and add more coins
		//int currentCoins = PlayerPrefsManager.SetPlayerPrefs(PlayerPrefsManager.TypeAction.LOAD_ACTION,PlayerPrefsManager.TypePrefs.TOTALCOIN_PREF,0);		
		//currentCoins+= Mathf.Abs(numberCoins);
		PlayerPrefsManager.SetPlayerPrefs(PlayerPrefsManager.TypeAction.ADD_ACTION,
		                                  PlayerPrefsManager.TypePrefs.TOTALCOIN_PREF, Mathf.Abs(numberCoins) );	
		
	}
	
	public static void AddStars(int indexWorld, int indexLevel,int numberStars)
	{

		LevelStat auxLevel = GetLevel(indexWorld,indexLevel);
		
		if (auxLevel != null)
		{
			int cStars = auxLevel.Stars;

			int auxNS = Mathf.Abs(numberStars);
			if (auxNS > cStars)
			{
				SetLevel(indexWorld,indexLevel,auxNS);
				
				//IPD Se resta el total anterior de estrellas al actual
				int stars = auxNS - cStars;
				PlayerPrefsManager.SetPlayerPrefs(PlayerPrefsManager.TypeAction.ADD_ACTION,
							                                  PlayerPrefsManager.TypePrefs.TOTALSTARS_PREF, stars);	
			}
		}
	}


	#endregion GeneralStats
	//Ivan
	public static int GetTotalStars()
	{
		return PlayerPrefs.GetInt( GameSettings.NamePrefTotalStars );
	}

	public static void SubtractTotalCoins( int coins )
	{
		int currentCountCoins = PlayerPrefs.GetInt( GameSettings.NamePrefTotalCoins );
		PlayerPrefs.SetInt( GameSettings.NamePrefTotalCoins, currentCountCoins - coins );
	}

	public static int GetTotalCoins()
	{
		return PlayerPrefs.GetInt( GameSettings.NamePrefTotalCoins );
	}

	public static void UnlockNewSkin( CatSkinController.SkinType skinType )
	{
		PlayerPrefs.SetInt( "enableSkin_"+skinType, 1 );
	}

	public static int IsSkinUnlock( CatSkinController.SkinType skinType )
	{
		return PlayerPrefs.GetInt( "enableSkin_"+skinType );
	}

	public static void ActiveSkin( string catId, CatSkinController.SkinType skinType)
	{
		PlayerPrefs.SetInt( "activeSkin_" + catId, (int)skinType );
	}

	public static int GetSkinActive( string catI )
	{
		return PlayerPrefs.GetInt( "activeSkin_" + catI );
	}

	public static void ResetTries()
	{
		PlayerPrefs.SetInt( "tries", 0 );
	}

	public static void AddTry()
	{
		int countTries = PlayerPrefs.GetInt("tries");
		countTries++;
		PlayerPrefs.SetInt( "tries", countTries );
	}

	public static int GetTries()
	{
		return PlayerPrefs.GetInt("tries");
	}

	public static void AddWinLevel()
	{
		PlayerPrefs.SetInt( Application.loadedLevelName , 1 );
	}

	public static void AddWinLevel(string levelName)
	{
		PlayerPrefs.SetInt( levelName , 1 );
	}

	public static bool IsLevelWon( string loadLevelName )
	{
		if( 0 < PlayerPrefs.GetInt(loadLevelName) )
		{
			return true;
		}
		else
		{
			return false;
		}

	}

	public static void InitializeSeasonWins()
	{
		PlayerPrefs.SetInt( "seasonWins", 0 );
	}

	public static int GetSeasonWins()
	{
		return PlayerPrefs.GetInt("seasonWins");
	}

	public static void AddSeasonWin()
	{

		int countWins = PlayerPrefs.GetInt("seasonWins");
		countWins++;
		PlayerPrefs.SetInt( "seasonWins", countWins );
	}

	public static void InitializeSeasonTries()
	{
		PlayerPrefs.SetInt( "seasonTries", 0 );
	}
	
	public static int GetSeasonTries()
	{
		return PlayerPrefs.GetInt("seasonTries");
	}
	
	public static void AddSeasonTry()
	{
		int countTries = PlayerPrefs.GetInt("seasonTries");
		countTries++;
		PlayerPrefs.SetInt( "seasonTries", countTries );
	}

    public static void AddTutorialTry(string level)
    {
        int countTutorialTries = PlayerPrefs.GetInt("tutorialTry_"+level);
        countTutorialTries++;
        PlayerPrefs.SetInt("tutorialTry_" + level, countTutorialTries);
    }

    public static int GetTutorialTries(string level)
    {
        return PlayerPrefs.GetInt("tutorialTry_" + level);
    }

    public static void InitializeTutorialTries(string level)
    {
        PlayerPrefs.SetInt("tutorialTry_" + level, 0);
    }

    public static bool IsLoggedFB()
    {
        if (0 < PlayerPrefs.GetInt("loggedFB"))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public static void LoginFB()
    {
        PlayerPrefs.SetInt("loggedFB", 1);
    }

    public static void LogoutFB()
    {
        PlayerPrefs.SetInt("loggedFB", 0);
    }

    public static void FBCoinsGave()
    {
        PlayerPrefs.SetInt("FBCoinsGave", 1);
    }

    public static bool IsFBCoinsGave()
    {
        if (0 < PlayerPrefs.GetInt("FBCoinsGave"))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public static void CBSeasonFirstTimeMenu()
    {
        PlayerPrefs.SetInt("CBStartSeason", 0);
    }

    public static void CBSeasonNoFirstTimeMenu()
    {
        PlayerPrefs.SetInt("CBStartSeason", 1);
    }

    public static bool CBIsSeasonFirstTimeMenu()
    {
        if (0 < PlayerPrefs.GetInt("CBStartSeason"))
        {
            return false;
        }
        else
        {
            return true;
        }
    }

}
