using UnityEngine;
using System.Collections;

public static class GameSettings
{
	
	/// <summary>
	/// Localization languages
	/// </summary>
	public enum Language
	{
		/// <summary>
		/// Español
		/// </summary>
		SPANISH = 0,
		
		/// <summary>
		/// English
		/// </summary>
		ENGLISH = 1,
		
		/// <summary>
		/// Japanese
		/// </summary>
		JAPANESE = 2,
		
		/// <summary>
		/// Japanese
		/// </summary>
		CHINESE = 3,
		
		/// <summary>
		/// Japanese
		/// </summary>
		KOREAN = 4		
	}
	
	
	/// <summary>
	/// Device Type
	/// </summary>
	public enum DeviceType
	{
		/// <summary>
		/// Touch devices
		/// </summary>
		TACTILE_DEVICE = 0,
		
		/// <summary>
		/// Desktop devices
		/// </summary>
		DESKTOP_DEVICE = 1
		
	}
	
	/// <summary>
	/// Conection type
	/// </summary>
	public enum ConnectionType
	{
		// No conection
		NO_CONNECTION,
		// Wifi or lan
		DSL_LAN_CONNECTION,
	}
	
	
	/// <summary>
	/// Device language
	/// </summary>
	public static GameSettings.Language GameLanguage = GameSettings.Language.SPANISH;
	
	/// <summary>
	/// Device type
	/// </summary>
	public static GameSettings.DeviceType GameDeviceType  = GameSettings.DeviceType.DESKTOP_DEVICE;
	
	/// <summary>
	/// Connection Type
	/// </summary>
	public static GameSettings.ConnectionType GameConnectionType  = GameSettings.ConnectionType.NO_CONNECTION;
	
	/// <summary>
	/// Mute music
	/// </summary>
	public static bool MuteMusic = false;

	
	/// <summary>
	/// Mute Sound effects
	/// </summary>
	public static bool isMuteSoundEffects = false;


	/// <summary>
	/// Swich mute sound effect
	/// </summary>
	public static void SwitchSoundEffect()
	{
		isMuteSoundEffects = !isMuteSoundEffects;
        PlayerPrefsManager.SetSoundEffect(isMuteSoundEffects);
		SetSoundEffecttk2d();
	}
	
	/// <summary>
	/// Set sound manager of 2dtookit
	/// </summary>
	public static void SetSoundEffecttk2d()
	{
		tk2dUIAudioManager.Instance.muteAudio  = isMuteSoundEffects;
	}

	
	/// <summary>
	/// Volumen for music
	/// </summary>
	public static float VolumenMusic = 0.3f;
	
	/// <summary>
	/// Volumen for sound Effect
	/// </summary>
	public static float VolumenSoundEffect = 1.0f;
	
	
	/// <summary>
	/// Debug mode for test
	/// </summary>
	public static bool DEBUG_MODE = false;
	
	/// <summary>
	/// Game Pause
	/// </summary>
	public static bool Pause = false;
	
	
	#region NamePrefs
	
	public static int NumberWorlds = 3;
	public static int NumberLevelPerWorld = 21;
	
	/// <summary>
	/// Name of player pref for current level
	/// </summary>
	public static string NamePrefCurrentWorld = "CurrentWorld";
	/// <summary>
	/// Name of player pref for current level
	/// </summary>
	public static string NamePrefLastWorldSelect = "LastWorldSelect";
	
	/// <summary>
	/// Name of player pref for current level
	/// </summary>
	public static string NamePrefCurrentLevel = "CurrentLevel";
	
	/// <summary>
	/// Name for player prefs total coins
	/// </summary>
	public static string NamePrefTotalCoins = "TotalCoins";
	
	/// <summary>
	/// Name for player prefs total coins
	/// </summary>
	public static string NamePrefTotalStars = "TotalStars";		
	
	/// <summary>
	/// Name for player prefs total coins
	/// </summary>
	public static string NamePrefLevelStats = "LevelStats_";

    /// <summary>
    /// Name for player prefs sound effect
    /// </summary>
    public static string NamePrefSoundEffect = "SoundEffect";

    /// <summary>
    /// Name for player prefs sound effect
    /// </summary>
    public static string NamePrefMusic = "Music";

	/// <summary>
	/// Index for none stars
	/// </summary>
	public static int NoneStars = 0;
	/// <summary>
	/// Index for bronze stars
	/// </summary>
	public static int BronzeStars = 1;
	/// <summary>
	/// Index for silver stars
	/// </summary>
	public static int SilverStars = 2;
	/// <summary>
	/// Index for golden stars
	/// </summary>
	public static int GoldStars = 3;
	
	#endregion NamePrefs
	
		
	
}
