﻿using UnityEngine;
using System.Collections;

public class ButtonMusicSoundEffect : MonoBehaviour {
	
	
	public enum TypeButton
	{
		MUSIC_BUTTON = 0,
		SOUNDEFFECT_BUTTON,
		
	}
	
	/// <summary>
	/// Type of music button
	/// </summary>
	public TypeButton TButton;
	
	/// <summary>
	/// Graphic for button of music
	/// </summary>
	public tk2dSlicedSprite SpriteButton;
	
		
	/// <summary>
	/// Id for sprite of music On
	/// </summary>
	private int idGraphicOn;
	
	/// <summary>
	/// Id for sprite of music Off
	/// </summary>
	private int idGraphicOff;
	
	// Boton para de musica para main menu
	public bool IsMainMenu = false;	
	public Camera GuiCamera;
	private AudioSource musicSource;
	
	void Start () 
	{
        
        if (this.TButton == TypeButton.MUSIC_BUTTON)
		{
			this.idGraphicOn = SpriteButton.GetSpriteIdByName("buttonMusic");
			this.idGraphicOff = SpriteButton.GetSpriteIdByName("buttonMusicOff");	
			if (this.IsMainMenu)
			{
                // Check player prefs
                GameSettings.MuteMusic = PlayerPrefsManager.GetIsMusicMute();                
                this.setSpriteButton();	
				
                
                this.musicSource = this.GuiCamera.transform.GetComponent<AudioSource>();				
                
                if ((this.musicSource != null) && (!GameSettings.MuteMusic))
				{
					this.musicSource.Play();
				}
			}else
            {
                this.setSpriteButton();	
            }
			
		}else if (this.TButton == TypeButton.SOUNDEFFECT_BUTTON)
		{
            if (this.IsMainMenu)
            {
                // Check player prefs
                GameSettings.isMuteSoundEffects = PlayerPrefsManager.GetIsSoundEffectMute();
                GameSettings.SetSoundEffecttk2d();
            }
			this.idGraphicOn = SpriteButton.GetSpriteIdByName("buttonSound");
			this.idGraphicOff = SpriteButton.GetSpriteIdByName("buttonSoundOff");
            this.setSpriteButton();	
			
		}
		
	}
	
	// Update is called once per frame
	void Update () 
	{
		
	
	}
	
	/// <summary>
	/// Method to setup sprite
	/// </summary>
	private void setSpriteButton()
	{
		if (this.TButton == TypeButton.MUSIC_BUTTON)
		{	
			if (GameSettings.MuteMusic)
			{
				
				this.SpriteButton.SetSprite(this.idGraphicOff);					
				
								
			}else
			{
				
				this.SpriteButton.SetSprite(this.idGraphicOn);	
				
			}
			
		}else if (this.TButton == TypeButton.SOUNDEFFECT_BUTTON)
		{
			if (GameSettings.isMuteSoundEffects)
			{
				this.SpriteButton.SetSprite(this.idGraphicOff);	

			}else
			{
				this.SpriteButton.SetSprite(this.idGraphicOn);	

			}
			
		}
		
	}
	
	public void PauseMusic()
	{
		if (this.musicSource != null)
		{
			if (this.musicSource.clip != null)
			{
				this.musicSource.Pause();	
			}
		}
		
	}
	
	public void ResumeMusic()
	{
		if (this.musicSource != null)
		{
			if (this.musicSource.clip != null)
			{
				this.musicSource.Play();	
			}
		}

		
	}
	
	void OnSoundMusicClick()
	{
		if (this.TButton == TypeButton.MUSIC_BUTTON)
		{
			GameSettings.MuteMusic = !GameSettings.MuteMusic;
            PlayerPrefsManager.SetMusic(GameSettings.MuteMusic);
			if (this.IsMainMenu)
			{
				if (GameSettings.MuteMusic)
				{
					this.PauseMusic();
				}else
				{
					this.ResumeMusic();
				}
				
			}else
			{
				if (GameManager.Instance != null)
				{
					if (GameSettings.MuteMusic)
					{
						GameManager.Instance.Sound_Manager.PauseMusic();
					}else
					{
						GameManager.Instance.Sound_Manager.ResumeMusic();
					}
					
				}
				
			}
		
		}else if (this.TButton == TypeButton.SOUNDEFFECT_BUTTON)
		{
			GameSettings.SwitchSoundEffect();						
			
		}		
		this.setSpriteButton();	
		
	}
}
