﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TutorialManager : MonoBehaviour 
{

	public static TutorialManager instance;

	public GameObject circleHelperPrefab;

	public TutorialHand tutorialHand;

	public TutorialPickCatArrow pickCatArrow;

	public GameObject tut_putBaseCat;
	public GameObject tut_putLoveCat;
	public GameObject tut_putBombCat;

    public AudioClip nodeTutorial;

	[System.Serializable]
	public class EventNode
	{
		public enum TypeEventNode
		{
			DELAY_PAUSE,
			ACTIVATION,
			ANIMATION,
			HAND,
			HAND_AND_ARROW,
			CIRCLE_CAT,
			CIRCLE_ZOMBIE,
			CIRCLE_SCARY,
		}
		public TypeEventNode typeEventNode;
		
		public GameObject target;
		public float time			= 0.5f;

		public enum TypePutCatHelper
		{
			NONE,
			BASE,
			LOVE,
			BOMB,
		}
		public TypePutCatHelper typePutCatHelper = TypePutCatHelper.NONE;
		
	}
	public EventNode[] eventNodes;

	private int currentIndex	= 0;


	//Put Cat Here
	private bool putCatMode = false;
	private int lastIndexCatMachine = -1;
	private int currentIndexPutCat	= 0;

	[System.Serializable]
	public class EventPutCatNode
	{
		//public TypePutCatHelper typePutCatHelper = TypePutCatHelper.NONE;
		public GameObject putCatPrefab;

        public GameObject targetInstance;

        public Vector3 targetPosition;


	}
	public List<EventPutCatNode> eventPutCatNodes = new List<EventPutCatNode>();

	void Awake()
	{
		instance = this;
	}

	// Use this for initialization
	void Start () {

        //int tutorialTries = PlayerPrefsManager.GetTutorialTries(Application.loadedLevelName);

        tutorialHand.gameObject.SetActive(false);
		PlayerPrefsManager.InitializeTutorialTries(Application.loadedLevelName);
		Initialize();
        /*if (tutorialTries == 0 || tutorialTries == 2)
        {
            if (tutorialTries == 0)
            {
                //PlayerPrefsManager.AddTutorialTry(Application.loadedLevelName);
            }
            else if(tutorialTries == 2)
            {
                PlayerPrefsManager.InitializeTutorialTries(Application.loadedLevelName);
                //PlayerPrefsManager.AddTutorialTry(Application.loadedLevelName);
            }
            Initialize();
        }
        else
        { 
            //If enter here means tutorialTries is 1
            //PlayerPrefsManager.AddTutorialTry(Application.loadedLevelName);
            GameManager.Instance.SetupInitLevel.tutorialMode = false;
        }*/
		
	}

    private void Initialize()
    {
        if (currentIndex < eventNodes.Length)
        {
            StartCoroutine(TimerIndex());
            GameManager.Instance.SetupInitLevel.tutorialMode = true;
        }

        tutorialHand.gameObject.SetActive(false);


        EventPutCatNode eventPutCatNodeAux;
        for (int i = 0; i < eventNodes.Length; i++)
        {
            eventPutCatNodeAux = new EventPutCatNode();
            if (eventNodes[i].typePutCatHelper != EventNode.TypePutCatHelper.NONE)
            {
                eventPutCatNodeAux.targetPosition = eventNodes[i].target.transform.position;

                switch (eventNodes[i].typePutCatHelper)
                {
                    case EventNode.TypePutCatHelper.BASE:
                        eventPutCatNodeAux.targetPosition += Vector3.up * 50;
                        eventPutCatNodeAux.putCatPrefab = tut_putBaseCat;
                        break;
                    case EventNode.TypePutCatHelper.LOVE:

                        eventPutCatNodeAux.putCatPrefab = tut_putLoveCat;
                        break;
                    case EventNode.TypePutCatHelper.BOMB:

                        eventPutCatNodeAux.putCatPrefab = tut_putBombCat;
                        break;
                }

                eventPutCatNodes.Add(eventPutCatNodeAux);
            }
        }
    }
	// Update is called once per frame
	void Update () {
        if (putCatMode)
        {
            if (CatMachineControllerCampaign.Instance.indexColumnCat < lastIndexCatMachine && 0 <= CatMachineControllerCampaign.Instance.indexColumnCat)
            {
                lastIndexCatMachine = CatMachineControllerCampaign.Instance.indexColumnCat;
                NextPutCatState();
            }

            if (CatMachineControllerCampaign.Instance.indexColumnCat < 0)
            {
                putCatMode = false;
               
				if ((currentIndex-1) <eventPutCatNodes.Count)
				{
                	GameObject.Destroy(eventPutCatNodes[currentIndexPutCat - 1].targetInstance);
				}
            }
        }
	}

	public void SetCurrentDroppableCat( DroppableBaseCat cat )
	{
		tutorialHand.currentDroppableCat = cat;
	}

	IEnumerator TimerIndex()
	{
		yield return new WaitForSeconds( eventNodes[currentIndex].time );

        if (eventNodes[currentIndex].typeEventNode == EventNode.TypeEventNode.CIRCLE_CAT
           || eventNodes[currentIndex].typeEventNode == EventNode.TypeEventNode.CIRCLE_ZOMBIE)
        {
            GameManager.Instance.Sound_Manager.PlaySoundEffect(nodeTutorial);
        }

		switch( eventNodes[currentIndex].typeEventNode )
		{
		case EventNode.TypeEventNode.DELAY_PAUSE:
			
			//tiempo de espera sin hacer nada
			
			break;
		case EventNode.TypeEventNode.ACTIVATION:

			eventNodes[currentIndex].target.SetActive( true );

			break;
		case EventNode.TypeEventNode.ANIMATION:
			
			eventNodes[currentIndex].target.SetActive( true );
			eventNodes[currentIndex].target.animation.Play();

			break;
		case EventNode.TypeEventNode.HAND:

			tutorialHand.gameObject.SetActive(true);
			tutorialHand.ActiveHand( eventNodes[currentIndex].target.transform );

			break;
		case EventNode.TypeEventNode.HAND_AND_ARROW:
			
			StartCoroutine( TimerHand() );
			
			pickCatArrow.gameObject.SetActive(true);
			pickCatArrow.ActiveArrow();
			
			break;
		case EventNode.TypeEventNode.CIRCLE_CAT:

			CircleSim circleSimCat = ((GameObject)GameObject.Instantiate( circleHelperPrefab, eventNodes[currentIndex].target.transform.position, Quaternion.identity )).GetComponent<CircleSim>();
			circleSimCat.ActiveHelp( eventNodes[currentIndex].target.transform, 0 );

			break;
		case EventNode.TypeEventNode.CIRCLE_ZOMBIE:

			CircleSim circleSimZombie = ((GameObject)GameObject.Instantiate( circleHelperPrefab, eventNodes[currentIndex].target.transform.position, Quaternion.identity )).GetComponent<CircleSim>();
			circleSimZombie.ActiveHelp( eventNodes[currentIndex].target.transform, 1 );

			break;
		case EventNode.TypeEventNode.CIRCLE_SCARY:
			
			CircleSim circleSimScary = ((GameObject)GameObject.Instantiate( circleHelperPrefab, eventNodes[currentIndex].target.transform.position, Quaternion.identity )).GetComponent<CircleSim>();
			circleSimScary.ActiveHelp( eventNodes[currentIndex].target.transform, 2 );
			
			break;
		
		}

		if( eventNodes[currentIndex].typeEventNode != EventNode.TypeEventNode.HAND
		   && eventNodes[currentIndex].typeEventNode != EventNode.TypeEventNode.HAND_AND_ARROW )
		{
			currentIndex++;
			
			if( currentIndex < eventNodes.Length )
			{

				StartCoroutine( TimerIndex() );
			
			}
			else
			{
				GameManager.Instance.SetupInitLevel.tutorialMode = false;

                if (0 < eventPutCatNodes.Count)
                {
                    putCatMode = true;
				    lastIndexCatMachine = CatMachineControllerCampaign.Instance.indexColumnCat;

                    NextPutCatState();
                }
				
			}
		}
	}


	public void NextPutCatState()
	{
		if (currentIndex < this.eventPutCatNodes.Count)
		{
        	eventPutCatNodes[currentIndexPutCat].targetInstance = ((GameObject)GameObject.Instantiate(eventPutCatNodes[currentIndexPutCat].putCatPrefab, eventPutCatNodes[currentIndexPutCat].targetPosition, Quaternion.identity));
		}
        if( 0 < currentIndexPutCat )
        {
            GameObject.Destroy( eventPutCatNodes[currentIndexPutCat - 1].targetInstance );
        }

		currentIndexPutCat++;
	}

	public void NextState()
	{
        
		currentIndex++;
		
		if( currentIndex < eventNodes.Length )
		{
			StartCoroutine( TimerIndex() );
		}
	}

	private IEnumerator TimerHand()
	{
		yield return new WaitForSeconds(2);
		tutorialHand.gameObject.SetActive(true);
		
		tutorialHand.ActiveHand( eventNodes[currentIndex].target.transform );
	}

}
