using UnityEngine;
using System.Collections;

public class CameraController : MonoBehaviour {
	
	public static tk2dCamera cameraScript;
	
	private float defaultY;
	
	public float maxZoom = 1;
	public float minZoom = 0.4f;
	
	public float speed = 30;
	public float speedZoom = 3f;
	
	//Drag util variables
	private Vector3 initialScreenPositionInput;
	
	//Pitch util variables
	private float initialDistanceTouchAB;
	private float initialZoom;
	
	public static float minX;
	public static float maxX;

	public static float minScreenX;
	public static float maxScreenX;

	public float minScreenY = 0;
	public float maxScreenY = 0;
	public float offsetLimitCamera = 100;
	
	public float dragSpeed = 1;
	
	public static float zoomFactorResolution = 0;
	
	private bool isDragLocked = true;
	
	public static CameraController Instance;
	
	
	private Transform targetSim;
	private bool lostGame = false;
	private float lostZoomSpeed = 10f;
	private float lostSpeed = 5;
	
	private bool firstTimeDrag = true;
	//private float minZoomFactor = 1;
	//private float targetZoomFactor = 0;
	// Use this for initialization
	private Vector3 offsetVibrate = Vector3.zero;
	private bool isVibrating = false;
	private float vibrateIntensity = 10;
	void Start ()
	{
		cameraScript 	= GetComponent<tk2dCamera>();
		Instance 		= GetComponent<CameraController>();
		
		
		minX 		= GameManager.Instance.SetupPrefabs.LeftBoundary.transform.position.x + 33;
		maxX 		= GameManager.Instance.SetupPrefabs.RightBoundary.transform.position.x - 33;

		minScreenX 			= minX;
		maxScreenX 			= maxX;

		if( GameManager.Instance.SetupInitLevel.VariousHeights )
		{
			minX += 175;
			maxX -= 175;
		}
		
		
		//Gesture.onPinchE += OnPinch;
		//Gesture.onDraggingE += OnDrag;
		
		SetZoomFactorResolution();
		Initialize();
		
	}
	
	
	// Update is called once per frame
	void Update ()
	{
		if( firstTimeDrag )
		{
			ControlY();
		}
		if( !GameManager.Instance.GameStats.EndLevel && Time.timeScale >= 1 )
		{
			OnPinchKeyboard();
			
			ControlX();
			//ControlY();
		}
		//Si ha perdido porque un sim ha salido dle rango
		else if( lostGame && targetSim != null )
		{
			float x = Mathf.Lerp( transform.position.x, targetSim.position.x, Time.deltaTime*lostSpeed );
			transform.position = new Vector3( x, transform.position.y, transform.position.z );
			
			LostGameZoom(lostZoomSpeed);
			ControlY();
		}
	}
	
	void Initialize()
	{
		//Calcula la proporcion que es currentWidth para maxWith y la establece como zoom maximo
		float maxWidth = Mathf.Abs(minScreenX) + Mathf.Abs(maxScreenX) + offsetLimitCamera + 33;
		float currentWidth = cameraScript.ScreenExtents.width;
		
		minZoom = currentWidth/maxWidth;
		
		cameraScript.ZoomFactor = minZoom;
		
		//centrar posicion camera
		transform.position = new Vector3( minScreenX+(maxScreenX-minScreenX)/2, transform.position.y, transform.position.z );
	}	
	
	void OnDrag( DragInfo dragInfo )
	{
		//las varaibles de ScreenY se inicializan una vez cuadno empieza a hacer drag porque primero se tiene que haber calculado el zoomFactor pertinente
		if( firstTimeDrag )
		{
			firstTimeDrag = false;
			
			minScreenY = transform.position.y - cameraScript.ScreenExtents.height/2;
			maxScreenY = transform.position.y + cameraScript.ScreenExtents.height/2;
		}
		if( !DroppableBaseCat.IS_DRAGGING && !GameManager.Instance.GameStats.EndLevel && Time.timeScale >= 1 && !isDragLocked)
		{
			float deltaX = -dragInfo.delta.x;
			
			float possibleXPosition = transform.position.x + (deltaX)*Time.deltaTime*dragSpeed;
			/*
			if( deltaX < 0 && possibleXPosition < minScreenX - offsetLimitCamera + cameraScript.ScreenExtents.width/2 )
			{
				transform.position = new Vector3( minScreenX - offsetLimitCamera + cameraScript.ScreenExtents.width/2, transform.position.y, transform.position.z);
				deltaX = 0;
			}*/
			
			if( deltaX < 0 && (minScreenX + 33 - offsetLimitCamera) < (possibleXPosition  - cameraScript.ScreenExtents.width/2)
			   || 0 < deltaX && (possibleXPosition  + cameraScript.ScreenExtents.width/2) < (maxScreenX - 33 + offsetLimitCamera) )
			{
				transform.position += Vector3.right*deltaX;
			}else if( deltaX < 0 )
			{
				transform.position = new Vector3( minScreenX  + 33 - offsetLimitCamera + cameraScript.ScreenExtents.width/2, transform.position.y, transform.position.z);
			}else if( 0 < deltaX )
			{
				transform.position = new Vector3( maxScreenX - 33 + offsetLimitCamera - cameraScript.ScreenExtents.width/2, transform.position.y, transform.position.z);
			}
			
			float deltaY = -dragInfo.delta.y;
			//transform.position += Vector3.up*deltaY;
			float possibleYPosition = transform.position.y + (deltaY)*Time.deltaTime*dragSpeed;
			
			if( deltaY < 0 && (minScreenY) < (possibleYPosition  - cameraScript.ScreenExtents.height/2)
			   || 0 < deltaY && (possibleYPosition  + cameraScript.ScreenExtents.height/2) < (maxScreenY) )
			{
				transform.position += Vector3.up*deltaY;
			}else if( deltaY < 0 )
			{
				//transform.position = new Vector3( transform.position.x, maxScreenY - cameraScript.ScreenExtents.height/2, transform.position.z);
				
			}else if( 0 < deltaY )
			{
				//transform.position = new Vector3( transform.position.x, minScreenY + cameraScript.ScreenExtents.height/2, transform.position.z);
				
			}
			
			
			
			
			//defaultY = cameraScript.TargetResolution.y/2 - 60;
			
			//float y = cameraScript.ScreenExtents.height/2 - cameraScript.TargetResolution.y/2 + defaultY;
			///transform.position = new Vector3(transform.position.x, y ,transform.position.z);
			
		}
	}
	
	void OnPinchKeyboard()
	{
		if( !DroppableBaseCat.IS_DRAGGING && !GameManager.Instance.GameStats.EndLevel )
		{
			if( !Mathf.Approximately(Input.GetAxis("Mouse ScrollWheel"),0) )
			{
				float magnitudeZoom = -Input.GetAxis("Mouse ScrollWheel")*100;
				if( magnitudeZoom < 0|| (0 < magnitudeZoom && cameraScript.ScreenExtents.width < (Mathf.Abs(minScreenX) + Mathf.Abs(maxScreenX) + offsetLimitCamera)) )
				{
					//IPD de momento quito el zoom
					//Zoom( magnitudeZoom );	
				}
				else if( 0 < magnitudeZoom )
				{
					//si el zoom esta al minimo no puede hacer pan para evitar que vibre la pantalla
					isDragLocked = true;
				}
			}
		}
	}
	
	void OnPinch(PinchInfo pinfo)
	{
		if( !DroppableBaseCat.IS_DRAGGING && !GameManager.Instance.GameStats.EndLevel )
		{
			if( pinfo.magnitude < 0|| (0 < pinfo.magnitude && cameraScript.ScreenExtents.width < (Mathf.Abs(minScreenX) + Mathf.Abs(maxScreenX) + offsetLimitCamera)) )
			{
				//IPD de momento quito el zoom
				//Zoom(pinfo.magnitude);
			}
			else if( 0 < pinfo.magnitude )
			{
				//si el zoom esta al minimo no puede hacer pan para evitar que vibre la pantalla
				isDragLocked = true;
			}
		}
	}
	
	void LostGameZoom(float speed)
	{
		float magnitudeZoom = -speed;
		if( magnitudeZoom < 0|| (0 < magnitudeZoom && cameraScript.ScreenExtents.width < (Mathf.Abs(minScreenX) + Mathf.Abs(maxScreenX) + offsetLimitCamera)) )
		{
			Zoom( magnitudeZoom );	
		}
		else if( 0 < magnitudeZoom )
		{
			//si el zoom esta al minimo no puede hacer pan para evitar que vibre la pantalla
			isDragLocked = true;
		}
	}
	
	void Zoom( float distanceTouchAB )
	{
		float resolutionY = cameraScript.TargetResolution.y;
		
		float posibleZoomFactor = cameraScript.ZoomFactor - (maxZoom - minZoom)/3f*speedZoom*distanceTouchAB/resolutionY;
		DebugConsole.text = "posibleZoomFactor: " + posibleZoomFactor;
		
		if( posibleZoomFactor < minZoom)
		{
			posibleZoomFactor = minZoom;
			//si el zoom esta al minimo no puede hacer pan para evitar que vibre la pantalla
			isDragLocked = true;
		}
		else
		{
			isDragLocked = false;
		}
		if( minZoom <= posibleZoomFactor && posibleZoomFactor <= maxZoom )
		{
			//IPD20130811 Provisional
			cameraScript.ZoomFactor = posibleZoomFactor;
		}
	}
	
	void ControlX()
	{
		//minPosX
		if(  ( transform.position.x - cameraScript.ScreenExtents.width/2) < (minScreenX + 33 - offsetLimitCamera) )
		{
			transform.position = new Vector3( minScreenX + 33 - offsetLimitCamera + cameraScript.ScreenExtents.width/2, transform.position.y, transform.position.z);
		}
		//maxPosX
		if(  ( (maxScreenX -33 + offsetLimitCamera)< transform.position.x + cameraScript.ScreenExtents.width/2) )
		{
			transform.position = new Vector3( maxScreenX - 33 + offsetLimitCamera - cameraScript.ScreenExtents.width/2, transform.position.y, transform.position.z);
		}
		
		if( isVibrating )
		{
			offsetVibrate = new Vector3( Random.Range(-vibrateIntensity,vibrateIntensity), 0,0 );
			transform.position += offsetVibrate;
		}
		else
		{
			offsetVibrate = Vector3.zero;
		}
	}
	
	void ControlY()
	{
		//defaultY = cameraScript.TargetResolution.y/2 + 30;
		//defaultY = cameraScript.TargetResolution.y/2 - 60;
		defaultY = cameraScript.TargetResolution.y/2 - 120;
		
		float y = cameraScript.ScreenExtents.height/2 - cameraScript.TargetResolution.y/2 + defaultY;
		transform.position = new Vector3(transform.position.x, y ,transform.position.z);
	}
	
	void SetZoomFactorResolution()
	{
		if( Screen.width >= 1920 )
		{
			zoomFactorResolution = 1.25f;
		}
		else if( Screen.width >= 1024 )
		{
			zoomFactorResolution = 0.25f;
		}
		else if( Screen.width >= 1280 )
		{
			zoomFactorResolution = 0.3f;
		}
		cameraScript.ZoomFactor = 1 + zoomFactorResolution;
		initialZoom 			= cameraScript.ZoomFactor;
		maxZoom += zoomFactorResolution;
		//minZoom += zoomFactorResolution;
		
	}
	
	public void ZoomEndLevel( Transform sim )
	{
		if (!lostGame )
		{
			targetSim = sim;
			lostGame = true;
		}
	}
	
	public void ActiveVibration()
	{
		StartCoroutine( Vibrate() );
	}
	
	IEnumerator Vibrate()
	{
		isVibrating = true;
		yield return new WaitForSeconds( 0.3f );
		isVibrating = false;
	}
	
	void OnDrawGizmos()
	{
		Gizmos.color = Color.yellow;
		Gizmos.DrawLine( transform.position, transform.position+Vector3.right*minScreenX );
		Gizmos.DrawLine( transform.position, transform.position+Vector3.right*minScreenX );
		Gizmos.DrawLine( transform.position, Vector3.up*maxScreenY );
		
		Gizmos.color = Color.blue;
		if( cameraScript != null )
		{
			Gizmos.DrawSphere( transform.position+cameraScript.ScreenExtents.height/2*Vector3.down+Vector3.left*75+Vector3.forward*50, 20 );
			Gizmos.DrawSphere( transform.position+cameraScript.ScreenExtents.height/2*Vector3.up+Vector3.left*75+Vector3.forward*50, 20 );
		}
		
		Gizmos.color = Color.red;
		
		Gizmos.DrawSphere( minScreenY*Vector3.up+Vector3.left*75+Vector3.forward*50, 20 );
		Gizmos.DrawSphere( maxScreenY*Vector3.up+Vector3.left*75+Vector3.forward*50, 20 );
	}
	
	//Es importante agregar y quitar los eventos de este modo porque cuadno reinicias el nivel te da problemas.
	void OnEnable(){
		Gesture.onDraggingE += OnDrag;
		Gesture.onPinchE += OnPinch;
	}
	
	//Es importante agregar y quitar los eventos de este modo porque cuadno reinicias el nivel te da problemas.
	void OnDisable(){
		Gesture.onDraggingE -= OnDrag;
		Gesture.onPinchE -= OnPinch;
	}
}
