﻿using UnityEngine;
using System.Collections;

public class BombCatController : CatController {
	
	public GameObject bombCatEffect;
	public float distanceExplosion = 2;
	public float startCountDown = 0.5f;
	[System.NonSerialized]
	public float startTime_StartCountDown = -1;
	
	public float delayExplosion = 2;
	
	[System.NonSerialized]
	public float startTime_DelayExplosion = -1;

    public AudioClip explosionCat;

	void Start () {
		Initialize();
		InitializeFSM();
	}
	
	void InitializeFSM()
	{
		fsm = new FSMSystem();
		
		if( !startInGame )
		{
			fsm.AddState(new Fall_LoveCatState(this.gameObject,(int)State.FALL));
		}
		else
		{
			helperCircle.SetActive(true);
		}
		fsm.AddState(new Idle_BombCatState(this.gameObject,(int)State.IDLE));
		fsm.AddState(new Land_CatState(this.gameObject,(int)State.LAND));
		fsm.AddState(new Explode_BombCatState(this.gameObject,(int)State.EXPLODE));
		
	}
	void Update () {
		
		
		if( (fsm.CurrentStateID == (int)State.LAND || fsm.CurrentStateID == (int)State.FALL) && Vector2.Distance( targetPosition,transform.position) < 10 )
		{
			if( startTimeAnimation < 0 )
			{
				fsm.ChangeState( (int)State.LAND );
				startTimeAnimation = Time.time;
			}
			if( startTimeAnimation + timeLanding < Time.time )
			{
				fsm.ChangeState( (int)State.IDLE );
			}
			
		}
		else if( fsm.CurrentStateID == (int)State.IDLE && startTime_StartCountDown + startCountDown < Time.time )
		{
			fsm.ChangeState( (int)State.EXPLODE );
		}
		
		fsm.UpdateState ();
		currentState = fsm.CurrentStateID;
		
	}
	
	public void Explode()
	{
		for(int i = 0; i<GameManager.Instance.ListCracks.Count; i++)
		{
			if( GetDistance2D( GameManager.Instance.ListCracks[i].transform.position, transform.position) < distanceExplosion )
			{
				GameManager.Instance.ListCracks[i].Open();
			}
		}

        GameManager.Instance.Sound_Manager.PlaySoundEffect(explosionCat);
        //SoundManager.PlaySoundEffect();

		Instantiate(bombCatEffect,transform.position,Quaternion.identity);
		
		CameraController.Instance.ActiveVibration();
		
		Destroy(gameObject);
	}
	
	void OnDrawGizmos()
	{
		Gizmos.color = Color.cyan;
		Gizmos.DrawWireSphere(transform.position + GetComponent<CharacterController>().center, distanceExplosion);
		
		Gizmos.color = Color.red;
		Gizmos.DrawWireSphere(transform.position + GetComponent<CharacterController>().center, distanceAttack);
	}
}
