using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CatController : MonoBehaviour {
	
	public enum State : int{
		IDLE,
		FALL,
		LAND,
		JUMP,
		ZOMBIE,
		DEAD,
		PATROL,
		SIT,
		EXPLODE,
	}
	
	public enum Animations{
		fall01,
		idle,
		jump,
		land01_part1,
		land01_part2,
		sitDown,
		sitIdle,
		sitIdle_lickBalls,
		sitIdle_lickLeg,
		sitUp,
		zombie,
		walk,
		sleep,
		bomb,
	}
	
	public enum RareIdle : int{
		IDLE,
		PATROL,
		SIT,
	}
	protected RareIdle currentRareIdle;
	
	public enum RareIdle1 : int{
		IDLE,
		PATROL,
		SIT,
	}
	
	[System.NonSerializedAttribute]
	public FSMSystem fsm;
	
	public bool isZombie = false;
	
	public int floor = 0;
	private ConfigZ configZ;
	
	public float fallSpeed = 600;
	public float jumpSpeed = 400;
	public float speed = 75;
	
	public int dir = -1;
	
	public int currentState;
	
	public float distanceAttack = 80;
	
	[System.NonSerializedAttribute]
	public Vector3 targetPosition;
	
	protected float timeLanding = 1.47f;
	protected float startTimeAnimation = -1;
	
	public GameObject shadow;
	public GameObject deadParticles;
	
	public GameObject targetSim;
	
	public float distancePatrol	= 160;
	public Vector3 destinationPatrol;

	
	protected bool timerRareIdleStarted 	= false;
	
	public float delayBetweenRareIdle = 5;
	
	public GameObject helperCircle;
	
	/*
	[System.NonSerializedAttribute]
	public float maxDistancePatrol;
	[System.NonSerializedAttribute]
	public float minDistancePatrol;
	*/
	/*
	[System.NonSerializedAttribute]
	public float maxX;
	[System.NonSerializedAttribute]
	public float minX;
	*/
	
	public bool startInGame = false;
	public bool instantZombie = false;

    public AudioClip meowCat;
    public AudioClip deadCat;

	void Awake()
	{
		configZ = GetComponent<ConfigZ>();
		
	}
	// Use this for initialization
	void Start () {
		
		//maxDistancePatrol = areaPatrol*0.6f;
		//minDistancePatrol = areaPatrol*0.3f;
		//randomTimeStartPatrol = Random.Range(timeStartPatrol/2, timeStartPatrol);
		//Si hay varias alturas se entiende que entrara aqui a traves de la gui, si startInGame estuviera a true, el gato empezaria directamente en el nivel y se expecificaria en que planta esta

		Initialize();
		InitializeFSM();
		/*if( !isZombie )
		{
			RaycastHit hit;
			if( Physics.Raycast( transform.position, Vector3.down, out hit, 3000 ) )
			{
				if( hit.transform.gameObject.layer == LayerMask.NameToLayer("bodyCollider") )
				{
					hit.transform.parent.GetComponent<CatAttacher>().AttachCat( gameObject );
					hit.transform.parent.GetComponent<SimController>().InfectedSimByCat();
					InfectSim();
				}
			}
		}*/
	}

	protected void Initialize()
	{
		if( GameManager.Instance.SetupInitLevel.VariousHeights && !startInGame )
		{
			floor = 1;
		}
	}

	void InitializeFSM()
	{

		fsm = new FSMSystem();
		
		//Si esta ene l juego desde el principio no necesita el estado fall y entra directamente en idle
		if( !startInGame )
		{
			fsm.AddState(new Fall_CatState(this.gameObject,(int)State.FALL));
			
		}
		else
		{
			helperCircle.SetActive(true);
		}
		fsm.AddState(new Idle_CatState(this.gameObject,(int)State.IDLE));
		fsm.AddState(new Zombie_CatState(this.gameObject,(int)State.ZOMBIE));
		fsm.AddState(new Land_CatState(this.gameObject,(int)State.LAND));
		fsm.AddState(new Jump_CatState(this.gameObject,(int)State.JUMP));
		fsm.AddState(new Patrol_CatState(this.gameObject,(int)State.PATROL));
		fsm.AddState(new SitIdle_CatState(this.gameObject,(int)State.SIT));	
	

	}
	
	// Update is called once per frame
	void Update () {
		
		if( targetSim != null )
		{
			//Si ya esta cerca de la cabeza del sim para convertir en zombie isZombie sera true o si el gato empieza emplazado en el asfalto desde el principio
			//del juego y infecta al instante entonces..
			/*if( isZombie || startInGame && instantZombie)
			{
				shadow.SetActive(false);
				helperCircle.SetActive( false );
				
				InfectSim();
				
				fsm.ChangeState( (int)State.ZOMBIE );
			}*/
			if( isZombie )
			{
				fsm.ChangeState( (int)State.ZOMBIE );
			}
			else
			{
				fsm.ChangeState( (int)State.JUMP );
			}
		}
		else
		{
			if( (fsm.CurrentStateID == (int)State.LAND || fsm.CurrentStateID == (int)State.FALL) && Vector2.Distance( targetPosition,transform.position) < 10 )
			{
				if( startTimeAnimation < 0 )
				{
					fsm.ChangeState( (int)State.LAND );
					startTimeAnimation = Time.time;
				}
				if( startTimeAnimation + timeLanding < Time.time )
				{
					currentRareIdle = RareIdle.IDLE;
					fsm.ChangeState( (int)State.IDLE );
				}
			}
			//IDLES
			else if( fsm.CurrentStateID != (int)State.FALL && fsm.CurrentStateID != (int)State.LAND )
			{
				switch ( currentRareIdle )
				{
					case RareIdle.IDLE:
					
						fsm.ChangeState( (int)State.IDLE );
					
						if ( !timerRareIdleStarted )
						{
							timerRareIdleStarted = true;
							StartCoroutine( TimerChangeRareIdle( Random.Range(delayBetweenRareIdle/2, delayBetweenRareIdle) ) );
						}
						break;
					
					case RareIdle.PATROL:
					
						fsm.ChangeState( (int)State.PATROL );
					
						if( GetDistance2D( transform.position, destinationPatrol ) < 10 )
						{
							fsm.ChangeState( (int)State.IDLE );
							InitializeRareIdle();
						}	
					
						break;
					
					case RareIdle.SIT:
					
						fsm.ChangeState( (int)State.SIT );
					
						if ( !timerRareIdleStarted )
						{
							timerRareIdleStarted = true;
							StartCoroutine( TimerChangeRareIdle( Random.Range(delayBetweenRareIdle/2, delayBetweenRareIdle) ) );
						}
						break;
				}
			}
		}
		
		fsm.UpdateState ();
		currentState = fsm.CurrentStateID;
		
	}
	
	public void InfectSim()
	{
		isZombie = true;
		if( targetSim != null )
		{
			if( targetSim.GetComponent<SimController>().Stats.HasShield )
			{
				targetSim.GetComponent<SimController>().RemoveShield();
				targetSim.GetComponent<SimController>().Stats.IsInfecting = false;
				targetSim = null;
				
				//provisional1!!
				//GameObject.Destroy( gameObject );
				DestroyCat();
			}
			else if( targetSim.GetComponent<SimController>().InfectedSimByCat( instantZombie ) )
			{
				targetSim.GetComponent<CatAttacher>().AttachCat( gameObject );
			}
		}
	}
	
	protected void InitializeRareIdle()
	{
		int randomIdle;
		do
		{
			randomIdle = Random.Range(0,5);
		}while( (int)currentRareIdle == randomIdle );
		
		if( randomIdle == 0 )
		{
			currentRareIdle = RareIdle.IDLE;
		}else if( randomIdle == 1 )
		{
			//currentRareIdle = RareIdle.PATROL;
			currentRareIdle = RareIdle.IDLE;
		}
		else
		{
			currentRareIdle = RareIdle.SIT;
		}
	}
	
	
	
	protected void LateUpdate()
	{
		if( transform.position.x < CameraController.minX-175 || CameraController.maxX + 175 < transform.position.x)
		{
			Destroy(gameObject);
		}
		if( !isZombie )
		{
			ControlZPositionShadow();
		}
		//ControlZ();
		configZ.Number_Floor = floor;
		
	}
		/*
	
	void ControlZ()
	{
		float auxZ = transform.position.z;
		auxZ = Mathf.Clamp(auxZ,GameManager.Instance.SimSetupSettings.FarZ - 200, GameManager.Instance.SimSetupSettings.FarZ);
		
		transform.position = new Vector3(transform.position.x,
										transform.position.y,
										auxZ);
	}*/
	
	protected void ControlZPositionShadow()
	{
		if( 0 < dir  )
		{
			shadow.transform.localPosition = new Vector3(shadow.transform.localPosition.x, shadow.transform.localPosition.y, -0.1f);
		}
		else
		{
			shadow.transform.localPosition = new Vector3(shadow.transform.localPosition.x, shadow.transform.localPosition.y, 0.1f);
			
		}
	}
	
	protected IEnumerator TimerChangeRareIdle( float secs )
	{
		
		yield return new WaitForSeconds( secs );
		timerRareIdleStarted = false;
		InitializeRareIdle();
	}
	
	
	
	public float GetDistance2D( Vector3 pos1, Vector3 pos2 )
	{
		return Vector2.Distance( new Vector2(pos1.x, pos1.y), new Vector2(pos2.x, pos2.y) );
	}

	public void DestroyCat()
	{
		Instantiate( deadParticles,transform.position,Quaternion.identity );

		if( GetComponent<LoveCatController>() )
		{
			NotAttractSim( GetComponent<LoveCatController>().distanceAttraction );
		}

        GameManager.Instance.Sound_Manager.PlaySoundEffect(deadCat);
        
		Destroy( gameObject );
	}

	private void NotAttractSim(float distanceAttraction)
	{
		
		List<SimController> listSims = DetectCleanSimsAux();
		
		if( 0 < listSims.Count )
		{
			if( 0 < listSims.Count )
			{
				for(int i = 0;i<listSims.Count;i++)
				{
					if( GetDistance2D(transform.position + Vector3.up*GetComponent<CharacterController>().radius, listSims[i].transform.position) < GetComponent<LoveCatController>().distanceAttraction )
					{
						Vector2 position2D = new Vector2( transform.position.x, transform.position.y);
						
						//listSims[i].SetLoveCat( position2D, deep );
						listSims[i].removeEffectLoveCat();
					}
				}
				
			}
		}
	}

	public List<SimController> DetectCleanSimsAux()
	{
		List<SimController> cleanSims = new List<SimController>();
		
		SimController simControllerAux;
		for (int i = 0; i < GameManager.Instance.ListSims.Count; i++)
		{
			simControllerAux = GameManager.Instance.ListSims[i].GetComponent<SimController>();
			if( !simControllerAux.Stats.IsInfecting 
			   && !simControllerAux.Stats.Infected 
			   && GameManager.Instance.ListSims[i].GetComponent<CatAttacher>().cat == null
			   && simControllerAux.Stats.Number_Floor == floor
			   && simControllerAux.Stats.CurrentState != SimController.State.SIM_FALL )
			{
				cleanSims.Add( GameManager.Instance.ListSims[i] );
			}
		}
		
		return cleanSims;
	}

	void OnDrawGizmos()
	{
		Gizmos.color = Color.red;
		Gizmos.DrawWireSphere(transform.position + GetComponent<CharacterController>().center, distanceAttack);
		
		Gizmos.color = Color.green;
		Gizmos.DrawWireSphere(transform.position + GetComponent<CharacterController>().center, distancePatrol);
	}
}
