/*******************************************************************************
 * Copyright (c) 2013, Esoteric Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
using System;
using System.IO;
using System.Collections.Generic;
using UnityEngine;
using Spine;

/** Extends SkeletonComponent to apply an animation. */
[ExecuteInEditMode, RequireComponent(typeof(MeshFilter), typeof(MeshRenderer))]
public class SkeletonAnimation : SkeletonComponent {
	public bool useAnimationName;
	public String animationName;
	public bool loop;
	public Spine.AnimationState state;

	public bool noPause		= false;
	//IPD20140611
	private bool gamePaused = false;

	override public void Initialize () {
		base.Initialize(); // Call overridden method to initialize the skeleton.
		
		state = new Spine.AnimationState(skeletonDataAsset.GetAnimationStateData());
	}

	override public void UpdateSkeleton () {
		//IPD20140611
		if( noPause )
		{
			if( Time.timeScale == 0 )
			{
				gamePaused = true;
				
				Time.timeScale = 1;
			}
			else
			{
				gamePaused = false;
			}
		}


		if (useAnimationName) {
			// Keep AnimationState in sync with animationName and loop fields.
			if (animationName == null || animationName.Length == 0) {
				if (state.Animation != null)
					state.ClearAnimation();
			} else if (state.Animation == null || animationName != state.Animation.Name) {
				Spine.Animation animation = skeleton.Data.FindAnimation(animationName);
				if (animation != null)
					state.SetAnimation(animation, loop);
			}
			state.Loop = loop;
		}
		
		// Apply the animation.

		//IPD20140611
		//state.Update(Time.deltaTime * timeScale);


		//IPD20140611
		if( gamePaused )
		{
			state.Update( 0.02f );
		}
		else
		{
			state.Update(Time.deltaTime * timeScale);
		}

		state.Apply(skeleton);

		// Call overridden method to call skeleton Update and UpdateWorldTransform.
		base.UpdateSkeleton();



		//IPD20140611
		if( gamePaused )
		{
			Time.timeScale = 0;
		}
	}
}
